import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Azuretenant, AzuretenantDetails } from '../../model/azuretenant.model';
import { FormsModule } from '@angular/forms';
import { Log } from '../../model/project.model';
import { ToastrService } from 'ngx-toastr';
import { MsalService } from '@azure/msal-angular';
import { environment } from 'src/environments/environment';
import { Router } from '@angular/router';

@Component({
  selector: 'app-azuretenant',
  templateUrl: './azuretenant.component.html',
  styleUrls: ['./azuretenant.component.css']
})
export class AzureTenantComponent implements OnInit {

  azuretenantdetails: Azuretenant[] = [];
  azuretenantform: any;
  newazuretenantdetails: Azuretenant = new Azuretenant();
  errorlog: Log = new Log();
  isProcessing: boolean = false;
  isLoggedInUserAdmin: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private toastr: ToastrService,private _router: Router) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Azure Tenant - ngOnInit";
    this.errorlog.type = "Error";
    try {
      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
    //this.webapiService._commonadmintenant.tenantid="";
      //this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;
      //if (this.isLoggedInUserAdmin) {
        this.ResetForm();
        this.LoadAzureTenantDetails();
      //}
      //else
      //{
        //this._router.navigateByUrl('/home');
      //}
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {
    this.azuretenantform = this.formbulider.group({
      tenantid: ['', Validators.required],
      tenantname: ['', Validators.required],
      _id:['']
    });
  }

  Savetenant() {
    this.errorlog.modulename = "Azure Tenant - Savetenantdetails";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _proj = this.azuretenantform.value;
        this.newazuretenantdetails = _proj;
        if (this.newazuretenantdetails._id == '')
         { this.newazuretenantdetails.createdby = localStorage.getItem("azapUserEmail");}
        this.newazuretenantdetails.updatedby = localStorage.getItem("azapUserEmail");
        this.webapiService.saveAzureTenantDetails(JSON.stringify(this.newazuretenantdetails)).subscribe(res => {
          this.toastr.success("Tenant details saved successfully!", "Azure Tenant");
          this.ResetForm();
          this.LoadAzureTenantDetails();
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }

  }

  LoadAzureTenantDetails() {
    this.errorlog.modulename = "Azure Tenant - LoadAzureTenantDetails";
    try {

      this.azuretenantdetails = [];

      this.webapiService.getallAzureTenantDetails().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          res.forEach(ele=>{
            if(ele.tenantid!="")
            {
              this.azuretenantdetails.push(ele);
            }
          });
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteAzureTenant(id: string) {
    this.errorlog.modulename = "Azure Tenant - DeleteAzureTenant";
    try {
      this.webapiService.deleteAzureTenantDetails(id).subscribe(res => {
        this.LoadAzureTenantDetails();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  EditAzureTenant(subscriptionDetails: Azuretenant) {
    this.errorlog.modulename = "Azure Tenant - EditAzureTenant";
    try {
      this.azuretenantform.controls['tenantname'].setValue(subscriptionDetails.tenantname);
      this.azuretenantform.controls['tenantid'].setValue(subscriptionDetails.tenantid);
      this.azuretenantform.controls['_id'].setValue(subscriptionDetails._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  EditTenantContent(tenantid:string)
  {
    this.webapiService._commonadmintenant._id=tenantid;
    this._router.navigateByUrl('/administration/subscription');
  }
}
