import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { Project, Environment } from 'src/app/model/project.model'
import { Resources, ResourcesJson } from 'src/app/model/resourcegroup.model';
import { ResourceGroup, ResourceJsonTemplateDetails, CreateResource } from 'src/app/model/resourcegroup.model';
import { MSTBluePrint, blueprintrg, blueprintrgresources, blueprintrgUI, blueprintrgresourcesUI, BlueprintId } from 'src/app/model/template.model';
import { Mstresource, MstIACProviders, Mstiacs } from '../../model/mstresource.model';

import { ActivatedRoute, Router } from '@angular/router';
import { Log } from '../../model/project.model';
import { ToastrService } from 'ngx-toastr';
import { MsalService } from '@azure/msal-angular';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-mstblueprint',
  templateUrl: './mstblueprint.component.html',
  styleUrls: ['./mstblueprint.component.css']
})
export class MstblueprintComponent implements OnInit {

  blueprintForm: any;
  blueprintcategory = ['General', 'IaaS', 'PaaS', 'iPaaS', 'CaaS'];
  masterProviders: MstIACProviders[] = [];
  mstiacs: Mstiacs[];
  blueprintdetails: MSTBluePrint;
  blueprintresourcegroup: blueprintrg[] = [];
  blueprintrgresources: blueprintrgresources[] = [];
  mstresourceproviders: MstIACProviders[];
  addrgresource: boolean = false;
  isloading = false; _id: string = ""; dataSource: ResourceGroup[] = [];
  filteredresources: Resources[] = [];

  blueprintrgUI: blueprintrgUI[] = [];
  blueprintrgresourcesUI: blueprintrgresourcesUI[] = [];
  blueprintid: string = '';
  masterresources: Mstresource[] = [];

  errorlog: Log = new Log();
  isLoggedInUserAdmin: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router, private toastr: ToastrService) { }


  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Azure Tenant";
    try {
      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;

      //this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;

      //if (this.isLoggedInUserAdmin) {
        this.ResetForm();

        this.LoadMSTIAC();
      //}
      //else {
      //  this._router.navigateByUrl('/home');
      //}
    } catch (error: any) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {

    this.blueprintForm = this.formbulider.group({
      blueprintid: [''],
      blueprintname: ['', Validators.required],
      description: [''],
      referencearchitecture: [''],
      category: ['', Validators.required],
      provider: ['', Validators.required],
      resourceprovider: [""],
      projectname: ['Update project name'],
      environmentname: ['Update environment name'],
      resourcesInput: [''],
      resourceId: [''],
      isactive: ['false']
    });

  }

  LoadProviders() {
    this.webapiService.getmstiacproviders().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.masterProviders = res;
        this.ModifyBlueprintdetails();
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  LoadMSTIAC() {
    this.webapiService.getmstiac().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.mstiacs = res;
        this.LoadProviders();
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  filterresource(value: string) {
    if (value != undefined) {
      let mstiac: Mstiacs;
      mstiac = this.mstiacs.find(x => x._id === value);
      this.mstresourceproviders = this.masterProviders.filter(x => x.mstiacid.includes(mstiac._id));
    }
    else {
      this.mstresourceproviders = this.masterProviders;
    }
  }

  editblueprint() {
    this.blueprintdetails = new MSTBluePrint();

    this.webapiService.getblueprintdetails(this.blueprintid).subscribe(res => {
      this.blueprintdetails = res;

      this.blueprintForm.controls['blueprintid'].setValue(this.blueprintid);

      this.blueprintForm.controls['blueprintname'].setValue(this.blueprintdetails.templatename);
      this.blueprintForm.controls['description'].setValue(this.blueprintdetails.description);

      this.blueprintForm.controls['category'].setValue(this.blueprintdetails.category);
      this.blueprintForm.controls['provider'].setValue(this.blueprintdetails.mstiacid);
      this.blueprintForm.controls['resourceprovider'].setValue(this.blueprintdetails.mstiacproviderids);
      this.filterresource(this.blueprintdetails.mstiacid);
      this.blueprintForm.controls['projectname'].setValue(this.blueprintdetails.project.name);
      this.blueprintForm.controls['environmentname'].setValue(this.blueprintdetails.environment.name);
      this.blueprintForm.controls['isactive'].setValue(this.blueprintdetails.isactive ? 'true' : 'false');
      this.blueprintForm.controls['referencearchitecture'].setValue(this.blueprintdetails.referencearchitecture);
    });
  }

  ModifyBlueprintdetails() {
    try {
      if (this.webapiService._commonblueprintedit.blueprintid != '' &&
        this.webapiService._commonblueprintedit.blueprintid != undefined) {
        this.blueprintid = this.webapiService._commonblueprintedit.blueprintid;
        this.editblueprint();
        this.webapiService._commonblueprintedit.blueprintid = '';
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadMasterResources() {
    this.webapiService.getmstresources().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.masterresources = res;
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  addResourceGroup() {
    try {
      const blueprint = this.blueprintForm.value;

      if (blueprint.blueprintid == "") {
        this.blueprintdetails = new MSTBluePrint();
      }

      this.blueprintdetails._id = blueprint.blueprintid;
      this.blueprintdetails.templatename = blueprint.blueprintname;
      this.blueprintdetails.category = blueprint.category;
      this.blueprintdetails.description = blueprint.description;
      this.blueprintdetails.referencearchitecture = blueprint.referencearchitecture;
      this.blueprintdetails.mstiacid = blueprint.provider;
      this.blueprintdetails.mstiacproviderids = blueprint.resourceprovider;

      let projectdetails = new Project();
      projectdetails.name = blueprint.projectname;
      projectdetails.description = "";

      let environmentdetails = new Environment();
      environmentdetails.name = blueprint.environmentname;
      environmentdetails.description = "";

      this.blueprintdetails.project = projectdetails;
      this.blueprintdetails.environment = environmentdetails;

      if (blueprint.isactive == "true")
        this.blueprintdetails.isactive = true;
      else
        this.blueprintdetails.isactive = false;

      this.blueprintdetails.createdby = localStorage.getItem("azapUserEmail");
      this.blueprintdetails.updatedby = localStorage.getItem("azapUserEmail");

      this.webapiService._commonblueprintdetails = this.blueprintdetails;

      this._router.navigateByUrl('/administration/blueprintresource');
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}