import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { Project, Environment } from 'src/app/model/project.model'
import { Resources, ResourcesJson } from 'src/app/model/resourcegroup.model';
import { ResourceGroup, ResourceJsonTemplateDetails, CreateResource } from 'src/app/model/resourcegroup.model';
import { MSTBluePrint, blueprintrg, blueprintrgresources, blueprintrgUI, blueprintrgresourcesUI, BlueprintId } from 'src/app/model/template.model';
import { Mstresource, MstIACProviders, Mstiacs } from '../../model/mstresource.model';
import { debounceTime, finalize, ignoreElements, map, startWith, switchMap, tap } from 'rxjs/operators';
import { Console } from 'console';
import { environment } from 'src/environments/environment';
import { ActivatedRoute, Router } from '@angular/router';
import { Log } from '../../model/project.model';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-mstblueprintresources',
  templateUrl: './mstblueprintresources.component.html',
  styleUrls: ['./mstblueprintresources.component.css']
})
export class MstbluepritresourceComponent implements OnInit {

  blueprintForm: any;
  blueprintdetails: MSTBluePrint;
  blueprintresourcegroup: blueprintrg[] = [];
  blueprintrgresources: blueprintrgresources[] = [];

  isloading = false; _id: string = ""; dataSource: ResourceGroup[] = [];
  filteredresources: Resources[] = [];

  blueprintrgUI: blueprintrgUI[] = [];
  blueprintrgresourcesUI: blueprintrgresourcesUI[] = [];
  blueprintid: string = '';
  masterresources: Mstresource[] = [];
  errorlog: Log = new Log();
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Mstbluepritresource - ngOnInit";
    this.errorlog.type = "Error";
    try {
      this.ResetForm();

      this.blueprintid = this.webapiService._commonblueprintdetails._id;
      this.LoadMasterResources();

      this.blueprintForm
        .get('resourcesInput')
        .valueChanges
        .pipe(
          debounceTime(500),
          tap(() => this.isloading = true),
          switchMap(value => this.webapiService.typeheadSearch({ name: value as string }, 1)
            .pipe(
              finalize(() => this.isloading = false),
            )
          )
        ).subscribe(users => {
          this.filteredresources = users;
          if (this.blueprintForm.get('resourcesInput').value == '') { $(".resbutton").prop('disabled', true); }
        }
        )
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {

    this.blueprintForm = this.formbulider.group({
      blueprintid: [''],
      resourcesInput: [''],
      resourceId: ['']
    });

  }
  LoadMasterResources() {
    this.webapiService.getmstresources().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.masterresources = res;
        if (this.blueprintid != null) {
          this.blueprintForm.controls['blueprintid'].setValue(this.blueprintid);
          this.LoadBlueprintResource();
        }
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  saveBlueprint() {

    this.errorlog.modulename = "Mstbluepritresource - saveBlueprint";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const blueprint = this.blueprintForm.value;

        this.blueprintdetails._id = blueprint.blueprintid;
        this.blueprintdetails.resourcegroup = [];

        if (this.blueprintrgUI != undefined && this.blueprintrgUI.length > 0) {
          this.blueprintrgUI.forEach(rg => {
            let resourcegroup = new blueprintrg();

            resourcegroup.name = rg.name;
            resourcegroup.location = rg.location;
            resourcegroup.subscription = rg.subscription;
            resourcegroup.tags = rg.tags;
            resourcegroup.resources = [];

            if (rg.resources != undefined && rg.resources.length > 0) {
              rg.resources.forEach(res => {
                let resource = new blueprintrgresources();
                resource.mstresourceid = res.mstresourceid;
                resourcegroup.resources.push(resource);
              });
            }
            this.blueprintdetails.resourcegroup.push(resourcegroup);
          });
        }

        this.blueprintdetails.createdby = localStorage.getItem("azapUserEmail");
        this.blueprintdetails.updatedby = localStorage.getItem("azapUserEmail");

        this.webapiService.saveblueprintdetails(JSON.stringify(this.blueprintdetails)).subscribe(
          res => {
            this.ResetForm();
            if (this.isProcessing) {
              this.isProcessing = false;
            }
            this.toastr.success("Blueprint details saved successfully!", "Create Blueprint");
            this.webapiService._commonblueprintdetails = new MSTBluePrint();
            this._router.navigateByUrl('/administration/blueprintdetails');
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
            if (this.isProcessing) {
              this.isProcessing = false;
            }
          });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  DeleteResourceGroup(rgIndex: number) {
    this.errorlog.modulename = "Mstbluepritresource - DeleteResourceGroup";
    try {
      this.blueprintrgUI.splice(rgIndex, 1);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteResource(rgIndex: number, resIndex: number) {
    this.errorlog.modulename = "Mstbluepritresource - DeleteResource";
    try {
      this.blueprintrgUI[rgIndex].resources.splice(resIndex, 1);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  getoptionid(input: string) {
    $(".res_id").val(input);
  };

  displayFn(res: Resources) {

    try {
      if (res) {
        this._id = $(".resoption").attr('data-message-id');
        $(".resbutton").prop('disabled', false);
        return res;
      }
    } catch (error) {
      this.errorlog.modulename = "Mstbluepritresource - displayFn";
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadBlueprintResource() {
    this.errorlog.modulename = "Mstbluepritresource - LoadBlueprintResource";
    try {
      this.blueprintdetails = new MSTBluePrint();
      this.blueprintdetails = this.webapiService._commonblueprintdetails;

      if (this.blueprintdetails != null && this.blueprintdetails.resourcegroup != null &&
        this.blueprintdetails.resourcegroup.length > 0) {
        this.blueprintdetails.resourcegroup.forEach(rg => {
          let resourcegroup = new blueprintrgUI();

          resourcegroup.name = rg.name;
          resourcegroup.location = rg.location;
          resourcegroup.subscription = rg.subscription;
          resourcegroup.tags = rg.tags;
          resourcegroup.resources = [];

          if (rg.resources != undefined && rg.resources.length > 0) {
            rg.resources.forEach(res => {
              let resource = new blueprintrgresourcesUI();
              resource.mstresourceid = res.mstresourceid;

              let resname = this.masterresources.find(x => x._id == res.mstresourceid);
              if (resname != null) {
                resource.resourcename = resname.userreadablename;
              }
              resourcegroup.resources.push(resource);
            });
          }
          this.blueprintrgUI.push(resourcegroup);
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  createResourceGroup() {
    this.errorlog.modulename = "Mstbluepritresource - createResourceGroup";
    try {
      let newRg = new blueprintrgUI();
      newRg.name = "Update_Resource_Group_Name_timestamp";
      this.blueprintrgUI.push(newRg);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  addresource(rgIndex: number) {
    this.errorlog.modulename = "Mstbluepritresource - addresource";
    try {
      const blprintformdetail = this.blueprintForm.value;

      let resource = new blueprintrgresourcesUI();

      resource.resourcename = blprintformdetail.resourcesInput;
      resource.mstresourceid = $(".res_id").val().toString();

      if (resource.mstresourceid != '') {
        if (this.blueprintrgUI[rgIndex].resources == undefined) {
          this.blueprintrgUI[rgIndex].resources = [];
        }
        this.blueprintrgUI[rgIndex].resources.push(resource);
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

}
