import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { ToastrService } from 'ngx-toastr';
import { ActivatedRoute, Router } from '@angular/router';
import { MstConfigRoles } from '../../model/configproject.model';
import { Mstresource, MstIACProviders, Mstiacs } from '../../model/mstresource.model';
import { Log } from '../../model/project.model';
import { MsalService } from '@azure/msal-angular';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-mstconfigrolelist',
  templateUrl: './mstconfigrolelist.component.html',
  styleUrls: ['./mstconfigrolelist.component.css']
})
export class MstconfigrolesComponent implements OnInit {

  mstconfigroleform: any;
  configroles: MstConfigRoles[] = [];
  mstconfigroles: MstConfigRoles[] = [];
  configrole: MstConfigRoles;
  mstiacs: Mstiacs[] = [];
  masterresources: Mstresource[] = [];
  mstresourceproviders: MstIACProviders[] = [];
  masterProviders: MstIACProviders[] = [];
  errorlog: Log = new Log();
  isProcessing: boolean = false;
  isLoggedInUserAdmin: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router,
    private toastr: ToastrService, private route: ActivatedRoute) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Mstconfigroles - ngOnInit";
    this.errorlog.type = "Error";

    try {
      //Removed Admin Check for Desktop App
      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;

      //this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;

      //if (this.isLoggedInUserAdmin) {
        this.ResetForm();
        this.LoadMSTIAC();
        this.LoadMasterResources();
        this.LoadProviders();
        this.LoadConfigRoles();
      //}
      //else {
          //this._router.navigateByUrl('/home');
      //}
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {
    this.mstconfigroleform = this.formbulider.group({
      rolename: ['', Validators.required],
      description: [''],
      resourcename: ['', Validators.required],
      readablename: [''],
      ansibleregistrypath: ['', Validators.required],
      mstiac: ['', Validators.required],
      masterresource: ['', Validators.required],
      version: [''],
      mstconfigroleid: [''],
      required: ['false']
    });
  }

  SaveConfigrole() {
    this.errorlog.modulename = "Mstconfigroles - SaveConfigrole";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _configrole = this.mstconfigroleform.value;

        this.configrole = new MstConfigRoles();
        this.configrole.rolename = _configrole.rolename;
        this.configrole.description = _configrole.description;
        this.configrole.userreadablename = _configrole.readablename;
        this.configrole.mstiacid = _configrole.mstiac;

        let mstiac = this.mstiacs.find(x => x._id === _configrole.mstiac);
        this.configrole.ansibleregistrypath = _configrole.ansibleregistrypath;
        this.configrole.mstresourceid = _configrole.masterresource;
        this.configrole._id = _configrole.mstconfigroleid;
        this.configrole.versionno = _configrole.version;
        this.configrole.isrequired = _configrole.required;
        this.configrole.createdby = localStorage.getItem("azapUserEmail");
        this.configrole.updatedby = localStorage.getItem("azapUserEmail");

        this.webapiService.createconfigrole(JSON.stringify(this.configrole)).subscribe(res => {
          this.ResetForm();
          this.toastr.success("Config Role details saved successfully!", "Create Config Role");
          if (this.isProcessing) {
            this.isProcessing = false;
          }
          this.LoadConfigRoles();
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  EditConfigRole(configrole: MstConfigRoles) {
    this.errorlog.modulename = "Mstconfigroles - EditConfigRole";
    try {
      this.ResetForm();
      this.mstconfigroleform.controls["mstconfigroleid"].setValue(configrole._id);
      this.mstconfigroleform.controls["rolename"].setValue(configrole.rolename);
      this.mstconfigroleform.controls["readablename"].setValue(configrole.userreadablename);
      this.mstconfigroleform.controls["description"].setValue(configrole.description);
      this.mstconfigroleform.controls["version"].setValue(configrole.versionno);
      this.mstconfigroleform.controls["masterresource"].setValue(configrole.mstresourceid);
      this.mstconfigroleform.controls["mstiac"].setValue(configrole.mstiacid);
      this.mstconfigroleform.controls["ansibleregistrypath"].setValue(configrole.ansibleregistrypath);
      this.mstconfigroleform.controls["required"].setValue(configrole.isrequired ? 'true' : 'false');
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteConfigRole(roleId: string) {
    this.errorlog.modulename = "Mstconfigroles - DeleteConfigRole";
    try {
      this.webapiService.deleteconfigrole(roleId).subscribe(res => {
        this.LoadConfigRoles();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadConfigRoles() {
    this.errorlog.modulename = "Mstconfigroles - LoadConfigRoles";
    try {
      this.webapiService.getconfigroles().subscribe(res => {
        if (res != undefined) {
          this.configroles = res;
          this.mstconfigroles = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadMSTIAC() {
    this.errorlog.modulename = "Mstconfigroles - LoadMSTIAC";
    try {
      this.webapiService.getmstiac().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.mstiacs = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadMasterResources() {
    this.errorlog.modulename = "Mstconfigroles - LoadMasterResources";
    try {
      this.webapiService.getmstresources().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.masterresources = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  addplaybookproperties(id: string) {
    this.webapiService._commonmstconfigroleid.mstconfigroleid = id;
    this._router.navigateByUrl('/administration/playbookproperties');
  }

  applyFilter(filterValue: string) {
    filterValue = filterValue.trim(); // Remove whitespace
    filterValue = filterValue.toLowerCase();

    this.configroles = this.mstconfigroles;
    this.configroles = this.configroles.filter(x => x.rolename.toLowerCase().includes(filterValue));

    if (filterValue.trim() === '') {
      this.configroles = this.mstconfigroles;
    }
  }

  LoadProviders() {
    this.webapiService.getmstiacproviders().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.masterProviders = res;
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

}