import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { MstPlaybookIPProperties, MstPlaybookOPProperties, MSTPlayBookProperties } from '../../model/configproject.model';
import { Router } from '@angular/router';
import { Log } from '../../model/project.model';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-mstconfigroleproperties',
  templateUrl: './mstconfigroleproperties.component.html',
  styleUrls: ['./mstconfigroleproperties.component.css']
})
export class MstplaybookpropertiesComponent implements OnInit {

  isInput: boolean = false;
  isOutput: boolean = false;
  playbookpropertiesForm: any;
  datatypes = ["string", "number", "bool", "map", "list(map)", "list(object)", "list(string)", "list(number)", "map(string)"];
  argumenttype = ["Block", "Normal"];
  mstplaybookipproperties: MstPlaybookIPProperties[] = [];
  mstplaybookopproperties: MstPlaybookOPProperties[] = [];
  mstplaybookproperties: MSTPlayBookProperties[] = [];
  configroleid: string = '';
  mstplaybookpropertyid: string;
  showDelete: boolean = false;
  validPropertype: boolean = false;
  invalidIPinput: boolean = false;
  invalidOPinput: boolean = false;
  errorlog: Log = new Log();
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "MSTPlaybookProperties - ngOnInit";
    this.errorlog.type = "Error";
    try {
      this.ResetForm();

      if (this.webapiService._commonmstconfigroleid.mstconfigroleid != '' &&
        this.webapiService._commonmstconfigroleid.mstconfigroleid != undefined) {
        this.configroleid = this.webapiService._commonmstconfigroleid.mstconfigroleid;
        this.webapiService._commonmstconfigroleid.mstconfigroleid = '';
        this.LoadMstPlaybookProperties();
      }
      else {
        this._router.navigateByUrl('/administration/configrole');
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  ResetForm() {
    this.playbookpropertiesForm = this.formbulider.group({
      rpname: ['', Validators.required],
      urname: ['', Validators.required],
      description: [''],
      defaultvalue: [''],
      propertytype: ['', Validators.required], // radio
      type: [''], // dropdown
      requiredproperty: ['No'], // radio
      outputparentattributeid: [''],// dropdown
      argumenttype: [''],// dropdown
      parentargumentid: [''],// dropdown
      defaultdeploymentvalue: [''],
      attributetype: [''],// dropdown
      mstiprolepropertyid: [''],
      mstoprolepropertyid: ['']
    });
    this.isInput = false;
    this.isOutput = false;
    this.showDelete = false;
    this.validPropertype = false;
    this.invalidIPinput = false;
    this.invalidOPinput = false;
  }

  checkPropertyType(value: string) {
    if (value != undefined && value != '') {
      if (value.toLocaleLowerCase() == 'input') {
        this.isInput = true;
        this.isOutput = false;
      }
      else if (value.toLocaleLowerCase() == 'output') {
        this.isInput = false;
        this.isOutput = true;
      }
    }
  }

  LoadMstPlaybookProperties() {
    this.errorlog.modulename = "MSTPlaybookProperties - LoadMstPlaybookProperties";
    try {
      this.LoadMstPlayBookIPProperties();
      this.LoadMstPlayBookOPProperties();
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  LoadMstPlayBookIPProperties() {
    this.errorlog.modulename = "MSTPlaybookProperties - LoadMstPlayBookIPProperties";
    try {
      this.webapiService.getmstplaybookipproperties(this.configroleid).subscribe(res => {

        if (typeof res != 'undefined' && res.length > 0) {
          this.mstplaybookproperties = res;
          if (this.mstplaybookproperties != null) {
            for (let i = 0; i < this.mstplaybookproperties.length; i++) {
              this.mstplaybookipproperties = this.mstplaybookproperties[i].ipvariables;
              this.mstplaybookpropertyid = this.mstplaybookproperties[i]._id;
            }
          }
          else {
            this.mstplaybookipproperties = [];
            this.mstplaybookpropertyid = '';
          }
          this.showDelete = ((this.mstplaybookipproperties != null &&
            this.mstplaybookipproperties.length > 0) || (this.mstplaybookopproperties != null && this.mstplaybookopproperties.length > 0)) ? true : false;
        }
      }
        , (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        }
      );
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  LoadMstPlayBookOPProperties() {
    this.errorlog.modulename = "MSTPlaybookProperties - LoadMstPlayBookIPProperties";
    try {
      this.webapiService.getmstplaybookopproperties(this.configroleid).subscribe(res => {
        //this.mstplaybookopproperties = res;

        if (typeof res != 'undefined' && res.length > 0) {
          this.mstplaybookproperties = res;

          if (this.mstplaybookproperties.length > 0) {
            for (let i = 0; i < this.mstplaybookproperties.length; i++) {
              this.mstplaybookopproperties = this.mstplaybookproperties[i].opvariables;
              this.mstplaybookpropertyid = this.mstplaybookproperties[i]._id;
            }
          }
          else {
            this.mstplaybookopproperties = [];
            this.mstplaybookpropertyid = '';
          }
          this.showDelete = ((this.mstplaybookipproperties != null &&
            this.mstplaybookipproperties.length > 0) || (this.mstplaybookopproperties != null && this.mstplaybookopproperties.length > 0)) ? true : false;
        }
      }
        , (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        }
      );
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  DeleteOPResourcePropertyVariable(opvariableId: string) {
    this.errorlog.modulename = "MSTPlaybookProperties - LoadMstPlayBookIPProperties";
    try {
      this.webapiService.deleteMstPlaybookOpProperties(this.mstplaybookpropertyid, opvariableId).subscribe(res => {
        this.LoadMstPlayBookOPProperties();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  DeleteIPResourcePropertyVariable(ipvariableid: string) {
    this.errorlog.modulename = "MSTPlaybookProperties - LoadMstPlayBookIPProperties";
    try {
      this.webapiService.deleteMstPlaybookIpProperties(this.mstplaybookpropertyid, ipvariableid).subscribe(res => {
        this.LoadMstPlayBookIPProperties();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.type = "Error";
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }

  EditOPResource(opvariableDetails: MstPlaybookOPProperties) {
    this.errorlog.modulename = "MSTPlaybookProperties - EditOPResource";
    try {
      this.ResetForm();

      this.playbookpropertiesForm.controls['rpname'].setValue(opvariableDetails.name);
      this.playbookpropertiesForm.controls['urname'].setValue(opvariableDetails.userreadablename);
      this.playbookpropertiesForm.controls['description'].setValue(opvariableDetails.description);
      this.playbookpropertiesForm.controls['defaultvalue'].setValue(opvariableDetails.defaultvalue.toString());

      this.playbookpropertiesForm.controls['propertytype'].setValue('output');
      this.checkPropertyType('output');

      this.playbookpropertiesForm.controls['attributetype'].setValue(opvariableDetails.attributetype);
      if (opvariableDetails.parentattributeid != null)
        this.playbookpropertiesForm.controls['outputparentattributeid'].setValue(opvariableDetails.parentattributeid);
      else
        this.playbookpropertiesForm.controls['outputparentattributeid'].setValue('');
      this.playbookpropertiesForm.controls['mstoprolepropertyid'].setValue(opvariableDetails._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.loggedby=localStorage.getItem("azapUserEmail");
      this.webapiService.createlogapi(this.errorlog).subscribe(res => {
      });
      this.webapiService.showErrorNotification("Please try again later!", "Something went wrong!!");
    }
  }

  EditIPResource(ipvariableDetails: MstPlaybookIPProperties) {
    this.errorlog.modulename = "MSTPlaybookProperties - EditIPResource";
    try {
      this.ResetForm();
      this.playbookpropertiesForm.controls['rpname'].setValue(ipvariableDetails.name);
      this.playbookpropertiesForm.controls['urname'].setValue(ipvariableDetails.userreadablename);
      this.playbookpropertiesForm.controls['description'].setValue(ipvariableDetails.description);
      this.playbookpropertiesForm.controls['defaultvalue'].setValue(ipvariableDetails.defaultvalue.toString());

      this.playbookpropertiesForm.controls['propertytype'].setValue('input');
      this.checkPropertyType('input');

      this.playbookpropertiesForm.controls['type'].setValue(ipvariableDetails.type);
      this.playbookpropertiesForm.controls['requiredproperty'].setValue(ipvariableDetails.required);
      this.playbookpropertiesForm.controls['argumenttype'].setValue(ipvariableDetails.argumenttype);
      if (ipvariableDetails.parentargumentid != null && ipvariableDetails.parentargumentid != '' && ipvariableDetails.parentargumentid.toLowerCase() != 'null')
        this.playbookpropertiesForm.controls['parentargumentid'].setValue(ipvariableDetails.parentargumentid);
      else
        this.playbookpropertiesForm.controls['parentargumentid'].setValue('');
      this.playbookpropertiesForm.controls['defaultdeploymentvalue'].setValue(ipvariableDetails.defaultdeploymentvalue);
      this.playbookpropertiesForm.controls['mstiprolepropertyid'].setValue(ipvariableDetails._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.errorlog.loggedby=localStorage.getItem("azapUserEmail");
      this.webapiService.createlogapi(this.errorlog).subscribe(res => {
      });
      this.webapiService.showErrorNotification("Please try again later!", "Something went wrong!!");
    }
  }

  applyIPFilter(filterValue: string) {
    this.errorlog.modulename = "MSTPlaybookProperties - applyIPFilter";
    try {
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      for (let i = 0; i < this.mstplaybookproperties.length; i++) {
        this.mstplaybookipproperties = this.mstplaybookproperties[i].ipvariables;
      }

      this.mstplaybookipproperties = this.mstplaybookipproperties.filter(x => x.userreadablename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        for (let i = 0; i < this.mstplaybookproperties.length; i++) {
          this.mstplaybookipproperties = this.mstplaybookproperties[i].ipvariables;
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }


  applyOPFilter(filterValue: string) {
    this.errorlog.modulename = "MSTPlaybookProperties - applyOPFilter";
    try {
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      for (let i = 0; i < this.mstplaybookproperties.length; i++) {
        this.mstplaybookopproperties = this.mstplaybookproperties[i].opvariables;
      }

      this.mstplaybookopproperties = this.mstplaybookopproperties.filter(x => x.userreadablename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        for (let i = 0; i < this.mstplaybookproperties.length; i++) {
          this.mstplaybookopproperties = this.mstplaybookproperties[i].opvariables;
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  SavePlaybookProperty() {
    this.errorlog.modulename = "MSTPlaybookProperties - applyOPFilter";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _propertiesForm = this.playbookpropertiesForm.value;
        if (_propertiesForm.propertytype == '') {
          this.validPropertype = true;
        }
        else if (_propertiesForm.propertytype == 'input') {
          this.validPropertype = false;

          if (_propertiesForm.argumenttype == 'Select' || _propertiesForm.type == 'Select') {
            this.invalidIPinput = true;
          }
          else {
            let createipresourceproperty = new MSTPlayBookProperties();
            let ipProperties = new MstPlaybookIPProperties();

            createipresourceproperty.isdeleted = false;
            createipresourceproperty.createdby = localStorage.getItem("azapUserEmail");
            createipresourceproperty.updatedby = localStorage.getItem("azapUserEmail");
            createipresourceproperty.mstconfigroleid = this.configroleid;

            ipProperties.name = _propertiesForm.rpname;
            ipProperties.argumenttype = _propertiesForm.argumenttype;
            ipProperties.userreadablename = _propertiesForm.urname;
            ipProperties.description = _propertiesForm.description;
            ipProperties.defaultvalue = (_propertiesForm.defaultvalue != '' && _propertiesForm.defaultvalue != undefined) ? _propertiesForm.defaultvalue.split(',') : [];
            ipProperties.type = _propertiesForm.type;
            ipProperties.required = _propertiesForm.requiredproperty;
            if (_propertiesForm.parentargumentid != '' && _propertiesForm.parentargumentid != null)
              ipProperties.parentargumentid = _propertiesForm.parentargumentid;
            ipProperties.defaultdeploymentvalue = _propertiesForm.defaultdeploymentvalue;

            if (_propertiesForm.mstiprolepropertyid != '' && _propertiesForm.mstiprolepropertyid != undefined) {
              ipProperties._id = _propertiesForm.mstiprolepropertyid
            }
            createipresourceproperty.ipvariables = [];
            createipresourceproperty.ipvariables.push(ipProperties);
            createipresourceproperty._id = this.mstplaybookpropertyid;

            if (this.mstplaybookpropertyid == '' || this.mstplaybookpropertyid == undefined) {
              createipresourceproperty.opvariables = [];
              this.webapiService.createMSTPlaybookProperty(JSON.stringify(createipresourceproperty)).subscribe(() => {
                this.toastr.success("Config Role Input properties details saved successfully!", "Create Config Role Properties");
                this.ResetForm();
                this.LoadMstPlaybookProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
            else {
              createipresourceproperty._id = this.mstplaybookpropertyid;
              this.webapiService.updateMSTPlaybookProperty(JSON.stringify(createipresourceproperty)).subscribe(() => {
                this.toastr.success("Config Role Input properties details updated successfully!", "Update Config Role Properties");
                this.ResetForm();
                this.LoadMstPlaybookProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });

            }
          }
        }
        else if (_propertiesForm.propertytype == 'output') {
          let createopresourceproperty = new MSTPlayBookProperties();
          let opProperties = new MstPlaybookOPProperties();

          if (_propertiesForm.attributetype == 'Select') {
            this.invalidOPinput = true;
          }
          else {

            createopresourceproperty.isdeleted = false;
            createopresourceproperty.createdby = localStorage.getItem("azapUserEmail");
            createopresourceproperty.updatedby = localStorage.getItem("azapUserEmail");
            createopresourceproperty.mstconfigroleid = this.configroleid;

            opProperties.name = _propertiesForm.rpname;
            opProperties.userreadablename = _propertiesForm.urname;
            opProperties.description = _propertiesForm.description;
            opProperties.defaultvalue = (_propertiesForm.defaultvalue != '' || _propertiesForm.defaultvalue != undefined) ? _propertiesForm.defaultvalue.split(',') : [];
            opProperties.attributetype = (_propertiesForm.attributetype == undefined || _propertiesForm.attributetype.toLowerCase() == 'select') ? null : _propertiesForm.attributetype;
            opProperties.parentattributeid = (_propertiesForm.outputparentattributeid == '' || _propertiesForm.outputparentattributeid.toLowerCase() == 'select') ? null : _propertiesForm.outputparentattributeid;

            if (_propertiesForm.mstoprolepropertyid != '' && _propertiesForm.mstoprolepropertyid != undefined) {
              opProperties._id = _propertiesForm.mstoprolepropertyid
            }

            createopresourceproperty.opvariables = [];
            createopresourceproperty.opvariables.push(opProperties);


            if (this.mstplaybookpropertyid == '' || this.mstplaybookpropertyid == undefined) {
              createopresourceproperty.ipvariables = [];
              this.webapiService.createMSTPlaybookProperty(JSON.stringify(createopresourceproperty)).subscribe(() => {
                this.toastr.success("Config Role Output properties details saved successfully!", "Create Config Role Properties");
                this.ResetForm();
                this.LoadMstPlaybookProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
            else {
              createopresourceproperty._id = this.mstplaybookpropertyid;

              this.webapiService.updateMSTPlaybookProperty(JSON.stringify(createopresourceproperty)).subscribe(() => {
                this.toastr.success("Config Role Output properties details updated successfully!", "Update Config Role Properties");
                this.ResetForm();
                this.LoadMstPlaybookProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
          }
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  DeletePlaybookProperty() {
    this.errorlog.modulename = "MSTPlaybookProperties - DeletePlaybookProperty";
    try {
      this.webapiService.deleteMstPlaybookpropertybyId(this.mstplaybookpropertyid).subscribe(es => {
        this.ResetForm();
        //this.GetResourceProperties();
        this.mstplaybookproperties = [];
        this.mstplaybookipproperties = [];
        this.mstplaybookopproperties = [];
        this.mstplaybookpropertyid = null;
        this.showDelete = false;
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}
