import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ToastrService } from 'ngx-toastr';
import { MSTIPProperties, MSTIPVariablesDetails, MSTOPProperties, MSTOPVariablesDetails } from '../../model/mstresource.model';
import { Log } from '../../model/project.model';
import { WebapiService } from '../../service/webapi.service';

@Component({
  selector: 'app-mstresourcedataproperties',
  templateUrl: './mstresourcedataproperties.component.html',
  styleUrls: ['./mstresourcedataproperties.component.css']
})
export class MstdataresourcepropertiesComponent  implements OnInit {

  resouredatapropertiesForm: any;
  
  isInput: boolean = false;
  isOutput: boolean = false;
  datatypes = ["string", "number", "bool", "map", "list(map)", "list(object)", "list(string)", "list(number)", "map(string)"];
  argumenttype = ["Block", "Normal"];
  mstipproperties: MSTIPProperties[];
  mstopproperties: MSTOPProperties[];
  ipvariableDetails = MSTIPVariablesDetails;
  opvariableDetails = MSTOPVariablesDetails;
  resourcepropertyid: string = '';
  showDelete: boolean = false;
  validPropertype: boolean = false;
  invalidIPinput: boolean = false;
  invalidOPinput: boolean = false;
  errorlog: Log = new Log();
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Mstdataresourceproperties - ngOnInit";
    this.errorlog.type = "Error";
    try {
      this.ResetForm();
      if (this.webapiService._commonmstresource.resourceId != '' && this.webapiService._commonmstresource.resourceId != undefined) {
        this.GetDataResourceProperties();
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {
    this.resouredatapropertiesForm = this.formbulider.group({
      rpname: ['', Validators.required],
      urname: ['', Validators.required],
      description: [''],
      defaultvalue: [''],
      propertytype: ['', Validators.required], // radio
      type: ['Select'], // dropdown
      requiredproperty: ['No'], // radio
      outputparentattributeid: ['Select'],// dropdown
      argumenttype: ['Select'],// dropdown
      parentargumentid: ['Select'],// dropdown
      defaultdeploymentvalue: [''],
      rule: [''],
      attributetype: ['Select'],// dropdown
      mstipresourcepropertyid: [''],
      mstopresourcepropertyid: [''],
      validationpattern:[]
    });
    this.isInput = false;
    this.isOutput = false;
    this.showDelete = false;
    this.validPropertype = false;
    this.invalidIPinput = false;
    this.invalidOPinput = false;
  }

  checkPropertyType(value: string) {
    if (value != undefined && value != '') {
      if (value.toLocaleLowerCase() == 'input') {
        this.isInput = true;
        this.isOutput = false;
      }
      else if (value.toLocaleLowerCase() == 'output') {
        this.isInput = false;
        this.isOutput = true;
      }
    }
  }

  GetDataResourceProperties() {
    this.errorlog.modulename = "Mstdataresourceproperties - GetDataResourceProperties";
    try {
      this.webapiService.getinputdatapropertiesbyresourceid(this.webapiService._commonmstresource.resourceId).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.ipvariableDetails = res;
          if (this.ipvariableDetails.length > 0) {
            for (let i = 0; i < this.ipvariableDetails.length; i++) {
              this.mstipproperties = this.ipvariableDetails[i].ipvariables;
              this.resourcepropertyid = this.ipvariableDetails[i]._id;
            }
          }
          else {
            this.mstipproperties = [];
            this.resourcepropertyid = '';
          }
          this.showDelete = ((this.mstipproperties != null &&
            this.mstipproperties.length > 0) || (this.mstopproperties != null && this.mstopproperties.length > 0)) ? true : false;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });

      this.webapiService.getoutputdatapropertiesbyresourceid(this.webapiService._commonmstresource.resourceId).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.opvariableDetails = res;
          if (this.opvariableDetails.length > 0) {
            for (let i = 0; i < this.opvariableDetails.length; i++) {
              this.mstopproperties = this.opvariableDetails[i].opvariables;
              this.resourcepropertyid = this.opvariableDetails[i]._id;
            }
          }
          else {
            this.mstopproperties = [];
            this.resourcepropertyid = '';
          }
          this.showDelete = ((this.mstipproperties != null &&
            this.mstipproperties.length > 0) || (this.mstopproperties != null && this.mstopproperties.length > 0)) ? true : false;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  EditIPResource(inputproperty: MSTIPProperties) {
    this.errorlog.modulename = "Mstdataresourceproperties - EditIPdataResource";
    try {
      this.ResetForm();
      this.resouredatapropertiesForm.controls['rpname'].setValue(inputproperty.name);
      this.resouredatapropertiesForm.controls['urname'].setValue(inputproperty.userreadablename);
      this.resouredatapropertiesForm.controls['description'].setValue(inputproperty.description);
      this.resouredatapropertiesForm.controls['defaultvalue'].setValue(inputproperty.defaultvalue.toString());

      this.resouredatapropertiesForm.controls['propertytype'].setValue('input');
      this.checkPropertyType('input');

      this.resouredatapropertiesForm.controls['type'].setValue(inputproperty.type);
      this.resouredatapropertiesForm.controls['requiredproperty'].setValue(inputproperty.required);
      this.resouredatapropertiesForm.controls['argumenttype'].setValue(inputproperty.argumenttype);
      if (inputproperty.parentargumentid != null)
        this.resouredatapropertiesForm.controls['parentargumentid'].setValue(inputproperty.parentargumentid);
      else
        this.resouredatapropertiesForm.controls['parentargumentid'].setValue('Select');
      this.resouredatapropertiesForm.controls['defaultdeploymentvalue'].setValue(inputproperty.defaultdeploymentvalue);
      this.resouredatapropertiesForm.controls['rule'].setValue(inputproperty.rule);
      this.resouredatapropertiesForm.controls['validationpattern'].setValue(inputproperty.validationpattern);
      this.resouredatapropertiesForm.controls['mstipresourcepropertyid'].setValue(inputproperty._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  EditOPResource(outputproperty: MSTOPProperties) {
    this.errorlog.modulename = "Mstdataresourceproperties - EditOPDataResource";
    try {
      this.ResetForm();

      this.resouredatapropertiesForm.controls['rpname'].setValue(outputproperty.name);
      this.resouredatapropertiesForm.controls['urname'].setValue(outputproperty.userreadablename);
      this.resouredatapropertiesForm.controls['description'].setValue(outputproperty.description);
      this.resouredatapropertiesForm.controls['defaultvalue'].setValue(outputproperty.defaultvalue.toString());

      this.resouredatapropertiesForm.controls['propertytype'].setValue('output');
      this.checkPropertyType('output');

      this.resouredatapropertiesForm.controls['attributetype'].setValue(outputproperty.attributetype);
      if (outputproperty.parentattributeid != null)
        this.resouredatapropertiesForm.controls['outputparentattributeid'].setValue(outputproperty.parentattributeid);
      else
        this.resouredatapropertiesForm.controls['outputparentattributeid'].setValue('Select');
      this.resouredatapropertiesForm.controls['mstopresourcepropertyid'].setValue(outputproperty._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  applyIPFilter(filterValue: string) {
    this.errorlog.modulename = "Mstdataresourceproperties - applyIPFilter";
    try {
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      for (let i = 0; i < this.ipvariableDetails.length; i++) {
        this.mstipproperties = this.ipvariableDetails[i].ipvariables;
      }

      this.mstipproperties = this.mstipproperties.filter(x => x.userreadablename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        for (let i = 0; i < this.ipvariableDetails.length; i++) {
          this.mstipproperties = this.ipvariableDetails[i].ipvariables;
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  applyOPFilter(filterValue: string) {
    this.errorlog.modulename = "Mstdataresourceproperties - applyOPFilter";
    try {
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      for (let i = 0; i < this.opvariableDetails.length; i++) {
        this.mstopproperties = this.opvariableDetails[i].opvariables;
      }

      this.mstopproperties = this.mstopproperties.filter(x => x.userreadablename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        for (let i = 0; i < this.opvariableDetails.length; i++) {
          this.mstopproperties = this.opvariableDetails[i].opvariables;
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  SaveResourceProperty() {
    this.errorlog.modulename = "Mstdataresourceproperties - SaveResourceProperty";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _propertiesForm = this.resouredatapropertiesForm.value;
        if (_propertiesForm.propertytype == '') {
          this.validPropertype = true;
        }
        else if (_propertiesForm.propertytype == 'input') {
          this.validPropertype = false;

          if (_propertiesForm.argumenttype == 'Select' || _propertiesForm.type == 'Select') {
            this.invalidIPinput = true;
          }
          else {
            let createipresourceproperty = new MSTIPVariablesDetails();
            let ipProperties = new MSTIPProperties();

            createipresourceproperty.isdeleted = false;
            createipresourceproperty.createdby = localStorage.getItem("azapUserEmail");
            createipresourceproperty.updatedby = localStorage.getItem("azapUserEmail");
            createipresourceproperty.mstresourceid = this.webapiService._commonmstresource.resourceId;

            ipProperties.name = _propertiesForm.rpname;
            ipProperties.argumenttype = (_propertiesForm.argumenttype == undefined || _propertiesForm.argumenttype.toLowerCase() == 'select') ? null : _propertiesForm.argumenttype;
            ipProperties.userreadablename = _propertiesForm.urname;
            ipProperties.description = _propertiesForm.description;
            ipProperties.defaultvalue = (_propertiesForm.defaultvalue != '' && _propertiesForm.defaultvalue != undefined) ? _propertiesForm.defaultvalue.split(',') : [];
            ipProperties.type = (_propertiesForm.type == undefined || _propertiesForm.type.toLowerCase() == 'select') ? null : _propertiesForm.type;
            ipProperties.required = _propertiesForm.requiredproperty;
            ipProperties.parentargumentid = (_propertiesForm.parentargumentid == undefined || _propertiesForm.parentargumentid.toLowerCase() == 'select') ? null : _propertiesForm.parentargumentid;
            ipProperties.defaultdeploymentvalue = _propertiesForm.defaultdeploymentvalue;
            ipProperties.rule = _propertiesForm.rule;
            ipProperties.validationpattern = _propertiesForm.validationpattern;

            if (_propertiesForm.mstipresourcepropertyid != '' && _propertiesForm.mstopresourcepropertyid != undefined) {
              ipProperties._id = _propertiesForm.mstipresourcepropertyid
            }
            createipresourceproperty.ipvariables = [];
            createipresourceproperty.ipvariables.push(ipProperties);

            if (this.resourcepropertyid == '' || this.resourcepropertyid == undefined) {
              createipresourceproperty.opvariables = [];
              this.webapiService.createdataresourceproperty(JSON.stringify(createipresourceproperty)).subscribe(() => {
                this.toastr.success("Resource Input Properties details saved successfully!", "Create Resource Properties");
                this.ResetForm();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
                this.GetDataResourceProperties();
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
            else {
              createipresourceproperty._id = this.resourcepropertyid;
              this.webapiService.updatedataresourceproperty(JSON.stringify(createipresourceproperty)).subscribe(() => {
                this.toastr.success("Resource Input Properties details updated successfully!", "Update Resource Properties");
                this.ResetForm();
                this.GetDataResourceProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });

            }
          }
        }
        else if (_propertiesForm.propertytype == 'output') {
          let createopresourceproperty = new MSTOPVariablesDetails();
          let opProperties = new MSTOPProperties();

          if (_propertiesForm.attributetype == 'Select') {
            this.invalidOPinput = true;
          }
          else {

            createopresourceproperty.isdeleted = false;
            createopresourceproperty.createdby = localStorage.getItem("azapUserEmail");
            createopresourceproperty.updatedby =localStorage.getItem("azapUserEmail");
            createopresourceproperty.mstresourceid = this.webapiService._commonmstresource.resourceId;

            opProperties.name = _propertiesForm.rpname;
            opProperties.userreadablename = _propertiesForm.urname;
            opProperties.description = _propertiesForm.description;
            opProperties.defaultvalue = (_propertiesForm.defaultvalue != '' || _propertiesForm.defaultvalue != undefined) ? _propertiesForm.defaultvalue.split(',') : [];
            opProperties.attributetype = (_propertiesForm.attributetype == undefined || _propertiesForm.attributetype.toLowerCase() == 'select') ? null : _propertiesForm.attributetype;
            opProperties.parentattributeid = (_propertiesForm.outputparentattributeid == '' || _propertiesForm.outputparentattributeid.toLowerCase() == 'select') ? null : _propertiesForm.outputparentattributeid;

            if (_propertiesForm.mstopresourcepropertyid != '' && _propertiesForm.mstopresourcepropertyid != undefined) {
              opProperties._id = _propertiesForm.mstopresourcepropertyid
            }

            createopresourceproperty.opvariables = [];
            createopresourceproperty.opvariables.push(opProperties);
            if (this.resourcepropertyid == '' || this.resourcepropertyid == undefined) {
              createopresourceproperty.ipvariables = [];
              this.webapiService.createdataresourceproperty(JSON.stringify(createopresourceproperty)).subscribe(() => {
                this.toastr.success("Resource Output Properties details saved successfully!", "Create Resource Properties");
                this.ResetForm();
                this.GetDataResourceProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
            else {
              createopresourceproperty._id = this.resourcepropertyid;

              this.webapiService.updatedataresourceproperty(JSON.stringify(createopresourceproperty)).subscribe(() => {
                this.toastr.success("Resource Output Properties details updated successfully!", "Update Resource Properties");
                this.ResetForm();
                this.GetDataResourceProperties();
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              }, (error) => {
                this.errorlog.remark = error;
                this.webapiService.ErrorLogfn(this.errorlog);
                if (this.isProcessing) {
                  this.isProcessing = false;
                }
              });
            }
          }
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  DeleteIPResourcePropertyVariable(value: string) {
    this.errorlog.modulename = "Mstdataresourceproperties - DeleteIPDataResourcePropertyVariable";
    try {
      this.webapiService.deleteipdataresourceproperty(this.resourcepropertyid, value)
        .subscribe(res => {
          this.ResetForm();
          this.GetDataResourceProperties();
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteOPResourcePropertyVariable(value: string) {
    this.errorlog.modulename = "Mstdataresourceproperties - DeleteOPDataResourcePropertyVariable";
    try {
      this.webapiService.deleteopdataresourceproperty(this.resourcepropertyid, value)
        .subscribe(res => {
          this.ResetForm();
          this.GetDataResourceProperties();
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteResourceProperty() {
    this.errorlog.modulename = "Mstdataresourceproperties - DeleteDataResourceProperty";
    try {
      if (this.resourcepropertyid != '' && this.resourcepropertyid != undefined) {
        this.webapiService.deletemstdataresourceproperty(this.resourcepropertyid)
          .subscribe(res => {
            this.ResetForm();
            //this.GetResourceProperties();
            this.mstipproperties = [];
            this.mstopproperties = [];
            this.resourcepropertyid = '';
            this.showDelete = false;
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

}
