import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { ToastrService } from 'ngx-toastr';
import { ActivatedRoute, Router } from '@angular/router';
import { Mstresource, MstIACProviders, Mstiacs } from '../../model/mstresource.model';
import { Log } from '../../model/project.model';
import { MsalService } from '@azure/msal-angular';
import { environment } from 'src/environments/environment';

@Component({
  selector: 'app-mstresourcelist',
  templateUrl: './mstresourcelist.component.html',
  styleUrls: ['./mstresourcelist.component.css']
})

export class MSTResourcesComponent implements OnInit {

  mstresourceForm: any;
  mstresourcedetails: Mstresource;
  masterresources: Mstresource[] = [];
  masterresourcesDetails: Mstresource[] = [];
  masterProviders: MstIACProviders[] = [];
  mstresourceproviders: MstIACProviders[];
  mstiacs: Mstiacs[];
  validateProvider: boolean = false;
  errorlog: Log = new Log();
  isProcessing: boolean = false;
  isLoggedInUserAdmin: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router,
    private toastr: ToastrService, private route: ActivatedRoute) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "MSTResources - ngOnInit";
    this.errorlog.type = "Error";
    try {
      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;

      //this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;

      //if (this.isLoggedInUserAdmin) {
        this.ResetForm();
        this.LoadProviders();
        this.LoadMSTIAC();
        this.LoadMasterResources();
        this.webapiService._commonmstresource.resourceId = "";
      //}
      //else {
       // this._router.navigateByUrl('/home');
      //}
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {
    this.mstresourceForm = this.formbulider.group({
      modulename: ['', Validators.required],
      description: [''],
      resourcename: ['', Validators.required],
      readablename: [''],
      resourceProvider: ['', Validators.required],
      mstiac: ['', Validators.required],
      version: [''],
      mstresourceid: [''],
      configurable: ['false'],
      terraformregistrypath: ['', Validators.required]
    });
  }

  LoadProviders() {
    this.errorlog.modulename = "MSTResources - LoadProviders";
    try {
      this.webapiService.getmstiacproviders().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.masterProviders = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadMSTIAC() {
    this.errorlog.modulename = "MSTResources - LoadMSTIAC";
    try {
      this.webapiService.getmstiac().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.mstiacs = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  LoadMasterResources() {
    this.errorlog.modulename = "MSTResources - LoadMasterResources";
    try {
      this.webapiService.getmstresources().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.masterresources = res;
          this.masterresourcesDetails = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  DeleteMstResource(resourceId: string) {
    this.errorlog.modulename = "MSTResources - DeleteMstResource";
    try {
      this.webapiService.deletemstresource(resourceId).subscribe(res => {
        if (typeof res != 'undefined') {
          this.ResetForm();
          this.LoadMasterResources();
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  AddResourceProperties(resourceId: string) {

  }

  EditResource(resource: Mstresource) {
    this.errorlog.modulename = "MSTResources - EditResource";
    try {
      this.ResetForm();
      this.mstresourceForm.controls['mstresourceid'].setValue(resource._id);
      this.mstresourceForm.controls['modulename'].setValue(resource.modulename);
      this.mstresourceForm.controls['description'].setValue(resource.description);
      this.mstresourceForm.controls['resourcename'].setValue(resource.resourcename);
      this.mstresourceForm.controls['readablename'].setValue(resource.userreadablename);
      this.mstresourceForm.controls['version'].setValue(resource.versionno);
      this.mstresourceForm.controls['configurable'].setValue(resource.isconfigurable ? 'true' : 'false');
      this.mstresourceForm.controls['terraformregistrypath'].setValue(resource.terraformregistrypath);
      let mstiac: string;
      mstiac = resource.terraformregistrypath;
      mstiac = mstiac.substring(0, mstiac.indexOf('/'));
      mstiac = this.GetTitleCase(mstiac);

      this.mstresourceForm.controls['mstiac'].setValue(mstiac);
      this.filterresource(mstiac);
      this.mstresourceForm.controls['resourceProvider'].setValue(resource.providerid);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  GetTitleCase(value: string) {
    this.errorlog.modulename = "MSTResources - GetTitleCase";
    try {
      if (value != undefined) {
        let firstchar = value.substring(0, 1).toUpperCase();
        return firstchar + value.substring(1);
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  SaveMasterResource() {
    this.errorlog.modulename = "MSTResources - SaveMasterResource";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _mstresourceform = this.mstresourceForm.value;
        if (_mstresourceform.resourceProvider == 'Select' || _mstresourceform.mstiac == 'Select') {
          this.validateProvider = true;
        }
        else {
          this.validateProvider = false;
          this.mstresourcedetails = new Mstresource();

          this.mstresourcedetails._id = _mstresourceform.mstresourceid;
          this.mstresourcedetails.modulename = _mstresourceform.modulename;
          this.mstresourcedetails.description = _mstresourceform.description;
          this.mstresourcedetails.resourcename = _mstresourceform.resourcename;
          this.mstresourcedetails.userreadablename = _mstresourceform.readablename;
          this.mstresourcedetails.isdeleted = false;

          let cloudprovider: string;
          cloudprovider = this.getResourceProviderName(_mstresourceform.resourceProvider);

          this.mstresourcedetails.terraformregistrypath = _mstresourceform.terraformregistrypath;
          //_mstresourceform.mstiac + "/" + cloudprovider + "/" + _mstresourceform.modulename;
          this.mstresourcedetails.terraformregistrypath = this.mstresourcedetails.terraformregistrypath.toLowerCase();
          this.mstresourcedetails.providerid = _mstresourceform.resourceProvider;
          this.mstresourcedetails.versionno = _mstresourceform.version;

          if (_mstresourceform.configurable == "true")
            this.mstresourcedetails.isconfigurable = true;
          else
            this.mstresourcedetails.isconfigurable = false;

          if (this.mstresourcedetails._id === '' || this.mstresourcedetails._id == undefined) {
            this.mstresourcedetails.createdby = localStorage.getItem("azapUserEmail");

            let createresource: string;
            createresource = JSON.stringify(this.mstresourcedetails);

            this.webapiService.createmstresource(createresource).subscribe(() => {
              this.toastr.success("Resource details saved successfully!", "Save Master Resource");
              this.ResetForm();
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.LoadMasterResources();
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
          else {
            this.mstresourcedetails.updatedby =localStorage.getItem("azapUserEmail");

            this.webapiService.updatemstresource(this.mstresourcedetails).subscribe(() => {
              this.toastr.success("Resource details updated successfully!", "Update Master Resource");
              this.ResetForm();
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.LoadMasterResources();
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  getResourceProviderName(filterValue: string) {
    this.errorlog.modulename = "MSTResources - getResourceProviderName";
    try {
      if (filterValue != null) {
        filterValue = filterValue.toLowerCase();

        let provider: MstIACProviders;
        provider = this.mstresourceproviders.find(x => x._id.toLowerCase() === filterValue);
        return provider.cloudprovidername;
      }
      return '';
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  applyFilter(filterValue: string) {
    this.errorlog.modulename = "MSTResources - applyFilter";
    try {
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      this.masterresources = this.masterresourcesDetails;
      this.masterresources = this.masterresources.filter(x => x.userreadablename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        this.masterresources = this.masterresourcesDetails;
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  filterresource(value: string) {
    this.errorlog.modulename = "MSTResources - filterresource";
    try {
      if (value != undefined) {
        let mstiac: Mstiacs;
        mstiac = this.mstiacs.find(x => x.iactoolname.toLowerCase() === value.toLowerCase());
        this.mstresourceproviders = this.masterProviders.filter(x => x.mstiacid.includes(mstiac._id));
      }
      else {
        this.mstresourceproviders = this.masterProviders;
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  EditMstResource(value: string) {
    this.errorlog.modulename = "MSTResources - EditMstResource";
    try {
      this.webapiService._commonmstresource.resourceId = value;
      this._router.navigateByUrl('/administration/mstresourceproperties');
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  EditMstDataResource(value: string) {
    this.errorlog.modulename = "MSTResources - EditMstDataResource";
    try {
      this.webapiService._commonmstresource.resourceId = value;
      this._router.navigateByUrl('/administration/mstresourcedataproperties');
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}
