import { ThrowStmt } from '@angular/compiler';
import { Component, OnInit } from '@angular/core';
import { FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { MsalService } from '@azure/msal-angular';
import { ToastrService } from 'ngx-toastr';
import { environment } from 'src/environments/environment';
import { AzureResourceGroup, SaveAzureResourceGroup } from '../../model/azuretenant.model';
import { Log } from '../../model/project.model';
import { WebapiService } from '../../service/webapi.service';

@Component({
  selector: 'app-resourcegroup',
  templateUrl: './resourcegroup.component.html',
  styleUrls: ['./resourcegroup.component.css']
})
export class AdminresourcegroupComponent implements OnInit {
  azureResourceGroupdetails: AzureResourceGroup[] = [];
  errorlog: Log = new Log();
  saveAzureRG:SaveAzureResourceGroup=new SaveAzureResourceGroup();
  azureResourceGroupform: FormGroup;
  isLoggedInUserAdmin: boolean = false;
  ispointerchecked: string = "yes";
    ResourceGroupContent:AzureResourceGroup=new AzureResourceGroup();
    isProcessing: boolean = false;

  spid: string = ""; tenantid: string = ""; subscriptionid: string = ""; storageaccountid = ""; containerid = "";

  constructor(private webapiService: WebapiService, private formbuilder: FormBuilder, private toastr: ToastrService, private _router: Router) { }
  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Azure Admin ResourceGroup - ngOnInit";
    this.errorlog.type = "Error";
    try {
      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
      //this.isLoggedInUserAdmin = (userRoles != undefined) ? userRoles.indexOf(environment.adminRole) > -1 : false;
      //if (this.isLoggedInUserAdmin) {
        this.ResetForm();
        this.webapiService._commonadmintenant.rgid="";
        this.tenantid = this.webapiService._commonadmintenant._id;
        this.subscriptionid = this.webapiService._commonadmintenant.subscriptionid;
        this.spid = this.webapiService._commonadmintenant.spid;

        this.storageaccountid = this.webapiService._commonadmintenant.storageaccountid
        this.containerid = this.webapiService._commonadmintenant.containerid

        if (this.spid != undefined && this.spid != "" && this.subscriptionid != undefined && this.subscriptionid != "" && this.tenantid != undefined && this.tenantid != "") {
          this.ispointerchecked = "no";
          this.LoadAzureRGDetails();
        //}
        //else {
        //  this._router.navigateByUrl('/administration/tenant');
        //}
      }
      else {
        this._router.navigateByUrl('/home');
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
    this.ResetForm();
  }
  LoadAzureRGDetails() {
    try {
      this.errorlog.modulename = "Admin TenantServicePriciple - LoadAzureServicePricipleDetails";
      this.azureResourceGroupdetails = [];
      this.webapiService.getallAzurergDetails(this.tenantid, this.subscriptionid,this.spid).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.azureResourceGroupdetails = res;
          this.ResetForm();
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  ResetForm() {
    this.azureResourceGroupform = this.formbuilder.group({
      _id:[''],
      name: ['', Validators.required],
      storageaccounts: this.formbuilder.array([])
    });
  }
  EditResourceGroupContent(id: string) {
    this.ResetForm(); 
    this.errorlog.modulename = "Azure ResourceGroup - EditAzureResourceGroup";
    try {
      this.webapiService.getallAzurergDetails(this.tenantid, this.subscriptionid,this.spid).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.azureResourceGroupdetails = res;
          this.ResourceGroupContent = this.azureResourceGroupdetails.find(x=>x._id == id );
          this.azureResourceGroupform.controls['name'].setValue(this.ResourceGroupContent.name);
          this.azureResourceGroupform.controls['_id'].setValue(this.ResourceGroupContent._id);
          this.ResourceGroupContent.storageaccounts.forEach(element => {
            this.newAccount(element);
          });
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
   }
  DeleteAzureResourceGroup(id: string) {
    this.errorlog.modulename = "Azure ResourceGroup - DeleteAzureResourceGroup";
    try {
      this.webapiService.deleteAzureRG(this.tenantid, this.subscriptionid,this.spid,id).subscribe(res => {
        this.LoadAzureRGDetails();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  storageaccounts(): FormArray {
    return this.azureResourceGroupform.get('storageaccounts') as FormArray;
  }
  newAccount(user?: any) {
    let fg =  this.formbuilder.group({
      accountname:  [user ? user.accountname : ''],
      _id: [user ? user._id : ''],
      containers: this.formbuilder.array([])
    });
    
    (<FormArray>this.azureResourceGroupform.get('storageaccounts')).push(fg);
    let userIndex = (<FormArray>this.azureResourceGroupform.get('storageaccounts')).length - 1;
    if (!user) {
        this.newContainer(userIndex);
    }
    else {
        user.containers.forEach(phone => {
            this.newContainer(userIndex, phone);
        });
    }
  }
  removeAccount(accIndex: number) {
    this.storageaccounts().removeAt(accIndex);
  }
  accContainer(accIndex: number): FormArray {
    return this.storageaccounts()
      .at(accIndex)
      .get('containers') as FormArray;
  }
  newContainer(userIndex: number, data?: any) {
    let fg = this.formbuilder.group({
      'name': [data ? data.name : ''],
      '_id': [data ? data._id : '']
  });
  (<FormArray>(<FormGroup>(<FormArray>this.azureResourceGroupform.controls['storageaccounts'])
      .controls[userIndex]).controls['containers']).push(fg);

  }
  removeAccContainer(accIndex: number, containerIndex: number) {
    this.accContainer(accIndex).removeAt(containerIndex);
  }
  onSubmit() {
    this.errorlog.modulename = "Azure ResourceGroup - Saveresourcegroupdetails";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _proj = this.azureResourceGroupform.value;
        this.saveAzureRG.tenantid = this.tenantid;
        this.saveAzureRG.subscriptionid = this.subscriptionid;
        this.saveAzureRG.serviceprincipalid = this.spid;

        this.saveAzureRG.Resourcegroup = _proj;
        this.webapiService.saveRGDetails(JSON.stringify(this.saveAzureRG)).subscribe(res => {
          this.toastr.success("ResourceGroup details saved successfully!", "Azure ResourceGroup");
          this.ResetForm();
          this.LoadAzureRGDetails();
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }

  }
}
