import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { MsalService } from '@azure/msal-angular';
import { ToastrService } from 'ngx-toastr';
import { environment } from 'src/environments/environment';
import { AzureServiceprinciple, SaveAzureServiceprinciple } from '../../model/azuretenant.model';
import { Log } from '../../model/project.model';
import { WebapiService } from '../../service/webapi.service';

@Component({
  selector: 'app-serviceprincipal',
  templateUrl: './serviceprincipal.component.html',
  styleUrls: ['./serviceprincipal.component.css']
})
export class AdminserviceprincipleComponent implements OnInit {
  azureServicePrincipledetails:AzureServiceprinciple[]=[];
  newSPdetails:SaveAzureServiceprinciple=new SaveAzureServiceprinciple();
  azureServicePrincipleform: any;
  errorlog: Log = new Log();
  isLoggedInUserAdmin: boolean = false; 
  ispointerchecked: string = "yes";
  isProcessing: boolean = false;
  tenantid: string = "";subscriptionid: string = "";
  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private toastr: ToastrService,private _router: Router) { }
 ngOnInit(): void {
  this.errorlog.application = "Azap";
  this.errorlog.modulename = "Azure Admin ServicePrinciple - ngOnInit";
  this.errorlog.type = "Error";
  
  try {
    //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
    //this.isLoggedInUserAdmin = (userRoles != undefined) ? userRoles.indexOf(environment.adminRole) > -1 : false;
    //if (this.isLoggedInUserAdmin) {
      this.ResetForm();
      this.tenantid = this.webapiService._commonadmintenant._id;
      this.subscriptionid =this.webapiService._commonadmintenant.subscriptionid;
      this.webapiService._commonadmintenant.spid ="";
      if (this.subscriptionid != undefined && this.subscriptionid != ""&&this.tenantid != undefined && this.tenantid != "") {
        this.ispointerchecked = "no";
        this.LoadAzureSPDetails();
      }
      else {
        this._router.navigateByUrl('/administration/tenant');
      }
    //}
    //else {
    //  this._router.navigateByUrl('/home');
   // }
  } catch (error) {
    this.errorlog.remark = error.message + "-" + error.stack;
    this.webapiService.ErrorLogfn(this.errorlog);
  }
  this.ResetForm();
  }
  ResetForm() {
    this.azureServicePrincipleform = this.formbulider.group({
      name: ['', Validators.required],
      uniquekeyprefix: [''],
      _id:['']
    });
  }
  LoadAzureSPDetails()
  {
    try {
      this.errorlog.modulename = "Admin TenantServicePriciple - LoadAzureServicePricipleDetails";
      this.azureServicePrincipledetails = [] ;
      this.webapiService.getallAzureSPDetails(this.tenantid,this.subscriptionid).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.azureServicePrincipledetails = res;
          this.ResetForm();
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog) ;
    }
  }
  EditServicePrincipleContent(sp:AzureServiceprinciple){
    this.errorlog.modulename = "Azure ServicePrinciple - EditAzureServicePrinciple";
    try {
      this.azureServicePrincipleform.controls['name'].setValue(sp.name);
      this.azureServicePrincipleform.controls['uniquekeyprefix'].setValue(sp.uniquekeyprefix);
      this.azureServicePrincipleform.controls['_id'].setValue(sp._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  SaveServicePrinciple(){
    this.errorlog.modulename = "Azure Tenant - Saveserviceprincipledetails";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _proj = this.azureServicePrincipleform.value;
        this.newSPdetails.tenantid = this.tenantid;
        this.newSPdetails.subscriptionid = this.subscriptionid;
        this.newSPdetails.serviceprincipal = _proj;
        this.webapiService.saveSPDetails(JSON.stringify(this.newSPdetails)).subscribe(res => {
          this.toastr.success("Serviceprinciple details saved successfully!", "Azure Serviceprinciple");
          this.ResetForm();
          this.LoadAzureSPDetails();
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  
  DeleteAzureServicePrinciple(id:string)
  {
    this.errorlog.modulename = "Azure ServicePrinciple - DeleteAzureServicePrinciple";
    try {
      this.webapiService.deleteAzureSP(this.tenantid, this.subscriptionid,id).subscribe(res => {
        this.LoadAzureSPDetails();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  EditAzureServicePrinciple(id:string)
  {
    this.webapiService._commonadmintenant.spid = id;
    this._router.navigateByUrl('/administration/resourcegroup');
  }
}
