import { Component, OnInit } from '@angular/core';
import { AzureSubscription, SaveAzureSubscription } from '../../model/azuretenant.model';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { WebapiService } from '../../service/webapi.service';
import { ToastrService } from 'ngx-toastr';
import { MsalService } from '@azure/msal-angular';
import { Router } from '@angular/router';
import { Log } from '../../model/project.model';
import { environment } from 'src/environments/environment';
@Component({
  selector: 'app-subscription',
  templateUrl: './subscription.component.html',
  styleUrls: ['./subscription.component.css']
})
export class AdminsubscriptionComponent implements OnInit {
  azuresubscriptiondetails: AzureSubscription[] = [];
  azuresubscriptionform: any;
  errorlog: Log = new Log();
  newazuresubscriptiondetails: SaveAzureSubscription = new SaveAzureSubscription();
  isLoggedInUserAdmin: boolean = false; tenantid: string = "";
  ispointerchecked: string = "yes";
  isProcessing: boolean = false;
  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private toastr: ToastrService, private _router: Router) { }
  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Azure TenantSubscription - ngOnInit";
    this.errorlog.type = "Error";
    try {
      // let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
      // this.isLoggedInUserAdmin = (userRoles != undefined) ? userRoles.indexOf(environment.adminRole) > -1 : false;
      // if (this.isLoggedInUserAdmin) {
        this.ResetForm();
        this.tenantid = this.webapiService._commonadmintenant._id;
        this.webapiService._commonadmintenant.subscriptionid="";
        if (this.tenantid != undefined && this.tenantid != "") {
          this.ispointerchecked = "no";
          this.LoadAzureSubscriptionDetails();
        }
        else {
          this._router.navigateByUrl('/administration/tenant');
        }
      // }
      // else {
      //   this._router.navigateByUrl('/home');
      // }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
    this.ResetForm();
  }
  ResetForm() {
    this.azuresubscriptionform = this.formbulider.group({
      subscriptionid: ['', Validators.required],
      subscriptionname: ['', Validators.required],
      _id: ['']
    });
  }
  LoadAzureSubscriptionDetails() {
    try {
      this.errorlog.modulename = "Admin TenantSubscription - LoadAzureSubscriptionDetails";
      this.azuresubscriptiondetails = [] ;
      this.webapiService.getallAzureSubscribeDetails(this.tenantid).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.azuresubscriptiondetails = res;
          this.ResetForm();
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog) ;
    }
  }
  SaveSubscription() {
    this.errorlog.modulename = "Azure Tenant - Savesubscriptiondetails";
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _proj = this.azuresubscriptionform.value;
        this.newazuresubscriptiondetails.tenantid = this.tenantid;
        this.newazuresubscriptiondetails.subscription = _proj;
        this.webapiService.saveSubscriptionDetails(JSON.stringify(this.newazuresubscriptiondetails)).subscribe(res => {
          this.toastr.success("Subscription details saved successfully!", "Azure Subscription");
          this.ResetForm();
          this.LoadAzureSubscriptionDetails();
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  DeleteAzureSubscription(subid: string) {
    this.errorlog.modulename = "Azure Subscription - DeleteAzuresubscription";
    try {
      this.webapiService.deleteAzureSubscription(this.tenantid, subid).subscribe(res => {
        this.LoadAzureSubscriptionDetails();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  EditAzureSubscription(id: string) {
    this.webapiService._commonadmintenant.subscriptionid = id;
    this._router.navigateByUrl('/administration/serviceprincipal');
  }
  EditSubscriptionContent(sub: AzureSubscription) {
    this.errorlog.modulename = "Azure Subscription - EditAzureSubscription";
    try {
      this.azuresubscriptionform.controls['subscriptionname'].setValue(sub.subscriptionname);
      this.azuresubscriptionform.controls['subscriptionid'].setValue(sub.subscriptionid);
      this.azuresubscriptionform.controls['_id'].setValue(sub._id);
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}
