import { StringMap } from '@angular/compiler/src/compiler_facade_interface';
import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ConfigAssociation, ConfigRoles, MstRole } from '../../model/configproject.model';
import { WebapiService } from '../../service/webapi.service';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { Log } from '../../model/project.model';

//import {  IDropdownSettings } from 'ng-multiselect-dropdown';
@Component({
  selector: 'app-association',
  templateUrl: './association.component.html',
  styleUrls: ['./association.component.css']
})
export class AssociationComponent implements OnInit {
  projectId: string;
  environmentId: string;
  hostgroupid: string;
  mstresid: string;
  mstroles: MstRole[] = [];
  roleForm: any;
  hostgroupname: string;
  role: ConfigAssociation = new ConfigAssociation();
  rolelist: ConfigAssociation[] = [];
  ispointerchecked: string = "yes";
  dropdownSettings = {};
  reqrolelist: string[] = [];
  isProcessing: boolean = false;
  errorlog: Log = new Log();

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router, private toastr: ToastrService) {
  }

  ngOnInit(): void {

    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Association - ngOnInit";
    this.errorlog.type = "Error";
    try {
      this.projectId = this.webapiService._commonconfigproject.pcid;
      this.environmentId = this.webapiService._commonconfigproject.eid;
      this.hostgroupid = this.webapiService._commonconfigproject.hgid;
      this.mstresid = this.webapiService._commonconfigproject.resid;
      this.ResetForm();
      if (this.projectId != undefined && this.projectId != "" && this.environmentId != "" && this.environmentId != undefined && this.hostgroupid != undefined && this.hostgroupid != "" && this.mstresid != "" && this.mstresid != undefined) {
        this.LoadConfigRoles(this.hostgroupid, this.mstresid);
        this.Getrolelist();
        this.ispointerchecked = "no";
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  ResetForm() {
    this.roleForm = this.formbulider.group({
      _id: [''],
      hostgroupid: [this.hostgroupid],
      name: ['', Validators.required],
      hostgroupname: [this.hostgroupname],
      roles: [[], Validators.required],
    });
  }
  LoadConfigRoles(hostgroupid: string, mstresid: string) {
    try {
      this.errorlog.modulename = "Association - LoadConfigRoles";

      this.webapiService.getexistHostGroupbyId(hostgroupid).subscribe(res => {
        if (typeof res != 'undefined') {
          let reqarr = [];
          this.hostgroupname = res.hostgroupname;
          this.webapiService.getroleslistbymstId(mstresid).subscribe(res => {
            this.mstroles = res;
            reqarr = res.filter(item => item.isrequired == true);
            reqarr.forEach(x => {
              this.reqrolelist.push(x._id);
            });
            this.ResetForm();
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  saveroles() {
    try {
      this.errorlog.modulename = "Association - saveroles";
      if (!this.isProcessing) {
        this.isProcessing = true;
        const _role = this.roleForm.value;
        _role.createdby = localStorage.getItem("azapUserEmail");;
        this.reqrolelist.forEach(x => {
          if (!(_role.roles.indexOf(x) !== -1))
            _role.roles.push(x);
        });
        _role.updatedby = localStorage.getItem("azapUserEmail");;
        this.role = _role;

        this.webapiService.saveAssociation(this.role).subscribe(res => {
          this.toastr.success("Association details saved successfully!", "Create Association");
          if (this.isProcessing) {
            this.isProcessing = false;
          }
          this.Getrolelist();
          this.LoadConfigRoles(this.hostgroupid, this.mstresid);
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
          if (this.isProcessing) {
            this.isProcessing = false;
          }
        });
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  Getrolelist() {
    try {
      this.errorlog.modulename = "Association - Getrolelist";

      this.webapiService.getrolelistbyhgId(this.hostgroupid).subscribe(res => {
        if (typeof res != 'undefined') {
          this.rolelist = res;

          this.rolelist.forEach(y => {
            let customroles = [];
            y.rolelist.forEach(x => {
              customroles.push(x.rolename);
            });
            y.roles = customroles;
          });
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  editrole(roleid: string) {
    try {
      this.errorlog.modulename = "Association - editrole";

      this.webapiService.getexistrolebyroleId(roleid).subscribe(res => {
        if (typeof res != 'undefined') {
          let customroles = [];
          res[0].rolelist.forEach(x => {
            customroles.push(x.id);
          });
          res[0].roles = customroles;
          this.roleForm.controls['_id'].setValue(res[0]._id);
          this.roleForm.controls['name'].setValue(res[0].name);
          this.roleForm.controls['roles'].setValue(res[0].roles);
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  editrolevar(roleid: string) {
    try {
      this.webapiService._commonconfigproject.configassid = roleid;
      this._router.navigateByUrl('/configuration/playbook');
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  deleterole(roleid: string) {
    this.webapiService.deleterole(roleid).subscribe(() => {
      this.Getrolelist();
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
}