import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { DeploymentDetails, DeployAnsiblePackage } from '../../model/template.model';
import { WebapiService } from '../../service/webapi.service';
import { Log } from '../../model/project.model';
import { environment } from 'src/environments/environment';
import { DatePipe } from '@angular/common';
import { MsalService } from '@azure/msal-angular';

@Component({
  selector: 'app-deploymentconfig',
  templateUrl: './deployment.component.html',
  styleUrls: ['./deployment.component.css']
})
export class DeploymentconfigComponent implements OnInit {
  pipe = new DatePipe('en-US');
  deployDeploymentPackage = "Initialized Package Deployment!!!";
  deploymentStatusMsg: string = "";
  deployDetails: DeploymentDetails[] = [];
  deployAnsiblePackage: DeployAnsiblePackage;
  errorlog: Log = new Log();
  logDetails: any = [];
  isLoggedInUserAdmin: boolean = false;
  ansibleDeployPipelineName:string='';

  constructor(private webapiService: WebapiService, private _router: Router, private authService: MsalService) {
    this.ansibleDeployPipelineName = environment.ansibleDeployPipelineName.toLowerCase();
   }

  ngOnInit(): void {
    try {
      this.errorlog.application = "Azap"; this.errorlog.type = "error";
      this.errorlog.modulename = "Config Deployment - ngOnInit";

      let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
      this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;

      this.getDeploymentPackageDetails();
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  groupBy = function (xs, key) {
    return xs.reduce(function (rv, x) {
      (rv[x[key]] = rv[x[key]] || []).push(x);
      return rv;
    }, {});
  };

  getDeploymentPackageDetails() {
    try {
      this.errorlog.modulename = "Config Deployment - getDeploymentPackageDetails";
      this.webapiService.getconfigDeploymentPackageDetails().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.deployDetails = res;
          this.deployDetails = this.groupBy(this.deployDetails, 'name');
        }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  deployPackage(packageName: string, environmentName: string) {
    try {
      this.errorlog.modulename = "Config Deployment - getDeploymentPackageDetails";
      this.deploymentStatusMsg = this.deployDeploymentPackage;
      this.showDeploymentStatus();

      this.deployAnsiblePackage = new DeployAnsiblePackage();
      this.deployAnsiblePackage.environment = environmentName;
      this.deployAnsiblePackage.packageName = packageName;

      this.webapiService.deployAnsibleDeploymentPackage(JSON.stringify(this.deployAnsiblePackage)).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {

        }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  showDeploymentStatus() {
    $('#Modal').modal('show');
  }
  downloadpackage(packageName: string, packageVersion: string) {
     let packagelink = environment.ansiblepackagedownloadlink.replace(environment.packagename, packageName).replace(environment.packageversion, packageVersion);
     window.open(packagelink, "_blank");
    //let packagelink = 'https://vsrm.dev.azure.com/WiproAzAP/AzAP-Terraform-Pipelines/_apis/release/releases/202/logs?api-version=7.1-preview.2';
    // this.webapiService.getReleaseLog(packagelink).subscribe(res => {
    //   const blob = new Blob([res], { 
    //     type: 'application/zip'
    //   });
    //   const url = window.URL.createObjectURL(blob);
    //   var anchor = document.createElement("a");
    //   anchor.download = packageName + "_" + packageVersion +  ".nupkg";
    //   anchor.href = url;
    //   anchor.click();
    // });
  }
  
  displayLogDetails(packageid, action) {
    //packageid='62ecc9e911517abf70309a24'; 
    this.webapiService.getDeploymentLog(packageid, action).subscribe(res => {
      if (typeof res != 'undefined') {
        this.logDetails = res;
        $('#ModalLogDetails').modal('show');
      }
    });
  }
  
  downloareleaseid(releaseid: string) {
    let packagelink = environment.ansiblereleaseLogDetails.replace("{{releaseId}}", releaseid);
    this.webapiService.getReleaseLog(packagelink).subscribe(res => {
      const blob = new Blob([res], { 
        type: 'application/zip'
      });
      const url = window.URL.createObjectURL(blob);
      var anchor = document.createElement("a");
      anchor.download = "ReleaseLogs_"+releaseid+".zip";
      anchor.href = url;
      anchor.click();
      });
  }
  downloadtasklog(tasklogurl:string,taskId:string)
  {
    this.webapiService.getReleaseLog(tasklogurl).subscribe(res => {
      const blob = new Blob([res]);
      const url = window.URL.createObjectURL(blob);
      var anchor = document.createElement("a");
      anchor.download = "tasklog_"+taskId+".log";
      anchor.href = url;
      anchor.click();
      });
  }

  enableDeploy(status: string)
  {
    if(status.toLowerCase() === environment.draftStatus.toLowerCase())
    {
      return true;
    }
    else if(status.toLowerCase() === environment.submittedStatus.toLowerCase())
    {
      return false;
    }
    else if(status.toLowerCase() === environment.uploadedStatus.toLowerCase())
    {
      return true;
    }
  }

}
