import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { ResourceGroup, Resources, ResourcesJson } from '../../model/resourcegroup.model';
import { ConfigHostGroup, Inventory, IpAddress, vnetpeering } from '../../model/configproject.model';
import { Log } from '../../model/project.model';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-hostgroup',
  templateUrl: './hostgroup.component.html',
  styleUrls: ['./hostgroup.component.css']
})
export class HostgroupComponent implements OnInit {
  projectId: string;
  environmentId: string;
  hostgroupForm: any;
  resourcegroupDetails: ResourceGroup[] = [];
  mstresourceDetails: Resources[] = [];
  mstresourcepropDetails: ResourcesJson[] = [];
  selectedrgid: string = "";
  respropkey: string[] = [];
  inventory: any;
  listhostgroup: ConfigHostGroup[] = [];
  hostgroupproject: ConfigHostGroup = new ConfigHostGroup();
  _invip: IpAddress[] = [];
  _invprop: any[] = [];
  ispointerchecked: string = "yes";
  vnets: string[] = [];
  errorlog: Log = new Log();
  isProcessing: boolean = false;
  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    try {
      this.errorlog.application = "Azap"; this.errorlog.type = "Error";
      this.errorlog.modulename = "HostGroup-pageload";
      this.projectId = this.webapiService._commonconfigproject.pcid;
      this.environmentId = this.webapiService._commonconfigproject.eid;
      if (this.projectId != undefined && this.projectId != "" && this.environmentId != "" && this.environmentId != undefined) {
        this.LoadRgDrpdn();
        this.ispointerchecked = "no";
        this.Gethostgrouplist();
      }
      this.ResetForm();
    }
    catch (error) {
      this.errorlog.remark = JSON.stringify(error);
      this.webapiService.ErrorLogfn(this.errorlog)
    }
  }
  ResetForm() {
    this.hostgroupForm = this.formbulider.group({
      _id: [''],
      hostgroupname: ['', Validators.required],
      resourcegrpid: ['', Validators.required],
      mstresourceid: ['', Validators.required],
      controllerrg: ['terraform-rg'],
      controllervnet: ['terraform-rg-vnet'],
      targetvnet: [],
      targetrg: []
    });
    this.mstresourcepropDetails = [];
    this.respropkey = [];
  }
  LoadRgDrpdn() {
    this.resourcegroupDetails = [];
    this.webapiService.getResourceGroupsbyEnvId(this.environmentId).subscribe(res => {
      if (res != undefined && res.length > 0) {
        this.resourcegroupDetails = JSON.parse(JSON.stringify(res));
      }
    }, (error) => {
      this.errorlog.modulename = "Hostgroup-ResourcegroupDropdown";
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
  LoadResbyRG(rgid: string) {
    this.selectedrgid = rgid;
    this.mstresourceDetails = [];
    if (rgid != "") {
      this.webapiService.getconfigmstresbyrgId(rgid).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.mstresourceDetails = JSON.parse(JSON.stringify(res));
        }
      }, (error) => {
        if (error != null)
          this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    //this.loadvnets(rgid);
  }
  LoadRespropbyRes(resid: string) {
    this.mstresourcepropDetails = [];
    if (resid != "") {
      this.webapiService.getconfigmstrespropbyrgId(this.selectedrgid, resid).subscribe(res => {
        if (res != undefined && res.resources.length > 0) {
          this.mstresourcepropDetails = JSON.parse(JSON.stringify(res.resources));
        }
        if (res != undefined && res.inventoryProperty != undefined) {
          this.inventory = res.inventoryProperty;
          this.respropkey = Object.keys(res.inventoryProperty);
          this.respropkey = this.respropkey.filter(item => item !== "_id").filter(item => item !== "mstresourceid");
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
  }
  savehostgroup() {
    if (!this.isProcessing) {
      this.isProcessing = true;

      const _hostgroup = this.hostgroupForm.value;
      //this.hostgroupproject = _hostgroup;
      this.hostgroupproject.configprojectid = this.projectId;
      let servertype = this.mstresourceDetails.find(x => x.id == _hostgroup.mstresourceid).userreadablename;
      this.hostgroupproject.resourcetype = servertype;
      this.hostgroupproject.inventory = new Inventory();
      this.respropkey.forEach(x => {
        this.hostgroupproject.inventory[x] = $(".invprp_" + x).val();
      });
      this.hostgroupproject.inventory.ipaddress = [];
      this.hostgroupproject.inventory.ipaddress = this._invip;

      // assign properties
      this.hostgroupproject.hostgroupname = _hostgroup.hostgroupname;
      this.hostgroupproject.mstresourceid = _hostgroup.mstresourceid;
      this.hostgroupproject.resourcegrpid = _hostgroup.resourcegrpid;
      this.hostgroupproject._id = _hostgroup._id;

      // vnet peering details
      // this.hostgroupproject.vnetpeering = new vnetpeering();

      // this.hostgroupproject.vnetpeering.ip_ansible_controller_rg = _hostgroup.controllerrg;
      // this.hostgroupproject.vnetpeering.ip_ansible_controller_vnet = _hostgroup.controllervnet;
      // this.hostgroupproject.vnetpeering.ip_target_resource_group =
      //   this.resourcegroupDetails.find(x => x._id == _hostgroup.resourcegrpid).terraformuniquename;

      // this.hostgroupproject.vnetpeering.ip_target_vnets = _hostgroup.targetvnet;

      this.hostgroupproject.createdby = localStorage.getItem("azapUserEmail");
      this.hostgroupproject.updatedby = localStorage.getItem("azapUserEmail");

      this.webapiService.saveHostGroup(this.hostgroupproject).subscribe(res => {
        this.Gethostgrouplist();
        this.ResetForm();
        if (this.isProcessing) {
          this.isProcessing = false;
        }
        this.toastr.success("Host Group details saved successfully!", "Create Host Group");
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
        if (this.isProcessing) {
          this.isProcessing = false;
        }
      });
    }
  }
  assignipData(ipval: string, name: string) {
    try {
      let ip: IpAddress = new IpAddress();
      ip.address = ipval;
      ip.terraformuniquename = name;
      if (this._invip.find(x => x.terraformuniquename == name)) {
        this._invip.find(x => x.terraformuniquename == name).address = ipval;
      }
      else {
        this._invip.push(ip);
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  Gethostgrouplist() {
    this.webapiService.getHostGroupbyId(this.projectId).subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.listhostgroup = res;
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
  editHostgroup(confighostgroupid: string) {
    let tempmstresprop: ResourcesJson[];
    this.webapiService.getexistHostGroupbyId(confighostgroupid).subscribe(res => {
      if (typeof res != 'undefined') {
        this.hostgroupForm.controls['_id'].setValue(res._id);
        this.hostgroupForm.controls['hostgroupname'].setValue(res.hostgroupname);
        this.hostgroupForm.controls['resourcegrpid'].setValue(res.resourcegrpid);
        this.LoadResbyRG(res.resourcegrpid);
        this.hostgroupForm.controls['mstresourceid'].setValue(res.mstresourceid);
        if (res != undefined && res.inventory != undefined) {
          this.webapiService.getconfigmstrespropbyrgId(res.resourcegrpid, res.mstresourceid).subscribe(resmst => {
            this.mstresourcepropDetails = [];
            if (resmst != undefined && resmst.resources.length > 0) {
              tempmstresprop = JSON.parse(JSON.stringify(resmst.resources));
              tempmstresprop.forEach(z => {
                if (res.inventory.ipaddress.find(q => q.terraformuniquename == z.terraformuniquename) != undefined) {
                  let ip = res.inventory.ipaddress.find(q => q.terraformuniquename == z.terraformuniquename);
                  z["address"] = ip["address"];
                }
                else {
                  z["address"] = "";
                }
                this.mstresourcepropDetails.push(z);
              });
            }
          }, (error) => {
            if (error != null)
              this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
          this.mstresourcepropDetails = res.inventory.ipaddress;
          this.inventory = res.inventory;
          this.respropkey = Object.keys(res.inventory);
          this.respropkey = this.respropkey.filter(item => item !== "ipaddress");
        }

        // VNET Peering Details
        // this.hostgroupForm.controls['controllerrg'].setValue(res.vnetpeering.ip_ansible_controller_rg);
        // this.hostgroupForm.controls['controllervnet'].setValue(res.vnetpeering.ip_ansible_controller_vnet);
        // this.hostgroupForm.controls['targetvnet'].setValue(res.vnetpeering.ip_target_vnets);
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });

  }
  deleteHostGroup(configprojectid: string) {
    this.webapiService.deleteHostGroup(configprojectid).subscribe(() => {
      this.Gethostgrouplist();
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
  edithostgroupContent(hostgroupid: string, mstresid: string, configprojectid: string, resourcegrpid: string) {
    this.webapiService._commonconfigproject.hgid = hostgroupid;
    this.webapiService._commonconfigproject.resid = mstresid;
    this.webapiService._commonconfigproject.rgid = resourcegrpid;
    this._router.navigateByUrl('/configuration/association');
  }

  loadvnets(resourcegroupid: string) {
    this.vnets = [];
    this.webapiService.getvirtualnetworkbyrgid(resourcegroupid).subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        res.forEach(element => {
          this.vnets.push(element.vnetname);
        });

        //res.for(let i = index, i < res.length) { this.vnets.push(res[i].vnetname) }
      }
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
}
