import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { ConfigPlaybook, playbookipproperty } from 'src/app/model/configproject.model';
import { Log } from 'src/app/model/project.model';
import { WebapiService } from 'src/app/service/webapi.service';

@Component({
  selector: 'app-playbookproperty',
  templateUrl: './playbookproperty.component.html',
  styleUrls: ['./playbookproperty.component.css']
})
export class PlaybookpropertydialogComponent {
  mstconfigroleid: string = "";
  mstassociationid: string = "";
  ippropertylist: playbookipproperty[] = [];
  rolevarlist: ConfigPlaybook = new ConfigPlaybook();
  ismstplaybook: boolean = true;
  uniqueid: string = "";
  vnets: string[] = [];
  isvnet: boolean = false;
  errorlog: Log = new Log();
  isProcessing: boolean = false;

  constructor(public dialogRef: MatDialogRef<PlaybookpropertydialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private http: HttpClient, private webapiService: WebapiService, private _router: Router, private toastr: ToastrService) { this.mstconfigroleid = data.mstconfigroleid; this.mstassociationid = data.associationid; this.uniqueid = data.uniqueid; this.ismstplaybook = data.ismstplaybook }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "PlaybookPopup - ngOnInit";
    this.errorlog.type = "Error";
    try{
    this.GetPlaybookProperty();
    }
    catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  GetPlaybookProperty() {
    let rgname = "";
    this.webapiService.getplaybookipproperty(this.mstconfigroleid).subscribe(ipres => {
      if (typeof ipres != 'undefined') {
        this.webapiService.getConfigRolesbyConfigRoleId(this.mstconfigroleid).subscribe(roleres => {
          this.isvnet = roleres.isrequired;
          if (this.isvnet)
            this.loadvnets(this.webapiService._commonconfigproject.rgid);
          this.webapiService.getrgbyrgid(this.webapiService._commonconfigproject.rgid).subscribe(rgres => {
            rgname = rgres.name;

            this.ippropertylist = ipres[0].ipvariables;
            this.ippropertylist.forEach(x => { x.isrequired = (x.required.toLowerCase() == "yes") ? true : false; })
            if (!this.ismstplaybook) {
              let _customrolevar = [];
              this.webapiService.getexistplaybook(this.mstassociationid, this.mstconfigroleid).subscribe(rolevar => {
                this.rolevarlist = rolevar;
                _customrolevar = Object.keys(this.rolevarlist.rolevars);
                _customrolevar.forEach(x => {
                  if (x == "ip_target_resource_group") {
                    this.ippropertylist.find(y => y.name == x).defaultdeploymentvalue = rgname;
                    this.ippropertylist.find(y => y.name == x).isdefault = true;
                  }
                  else { this.ippropertylist.find(y => y.name == x).defaultdeploymentvalue = this.rolevarlist.rolevars[x]; }
                });
              });
            }
          });
        });
      }
    });

  }
  onNoClick(): void {
    this.dialogRef.close();
  }
  Saveplaybookip() {
    this.errorlog.modulename = "PlaybookPopup - saveplaybook";
    try{
    if (!this.isProcessing) {
      this.isProcessing = true;
      let valid: boolean = true;
      $('[required]').each(function () {
        //let s=  $(this).closest('tr').children('td').eq(1).text();
        if ($(this).is(':invalid') || !$(this).val()) { $(this).addClass("highlightcolor"); valid = false; }
        else {         
          $(this).removeClass("highlightcolor");         
        }
      })
      if (valid) {
        this.isProcessing = false;
        let rolevars = {};
        $(".clsproperty").each(function () {
          let prop = $(this).closest('tr').children('td').eq(1).text().trim();
          rolevars[prop] = $('.' + prop).val();
        });
        this.rolevarlist.configassociationid = this.mstassociationid;
        this.rolevarlist.mstconfigroleid = this.mstconfigroleid;
        this.rolevarlist._id = this.uniqueid;
        this.rolevarlist.rolevars = rolevars;
        this.rolevarlist.createdby = localStorage.getItem("azapUserEmail");
      this.rolevarlist.updatedby = localStorage.getItem("azapUserEmail");
        this.webapiService.updateconfigplayBook(this.rolevarlist).subscribe(res => {
          this.dialogRef.close();
            this.isProcessing = false;
          this.showNotification("Playbook properties saved successfully!!", "Configure Playbook")
          this._router.navigateByUrl('/playbook');
        });
      }
      else
      {this.isProcessing = false;}
    }
  }
  catch (error) {
    this.errorlog.remark = error.message + "-" + error.stack;
    this.webapiService.ErrorLogfn(this.errorlog);
     this.isProcessing = false;
  }
  }
  loadvnets(resourcegroupid: string) {
    this.vnets = [];
    this.webapiService.getvirtualnetworkbyrgid(resourcegroupid).subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        res.forEach(element => {
          this.vnets.push(element.vnetname);
        });
      }
    }, (error) => {
      this.ErrorLogfn(error);
    });
  }
  showNotification(message, title) {
    this.toastr.success(message, title)
  }
  ErrorLogfn(error: any) {
    this.errorlog.remark = error;
    this.errorlog.type = "Error";
    this.errorlog.loggedby=localStorage.getItem("azapUserEmail");
    this.webapiService.createlogapi(this.errorlog).subscribe(res => {
    });
    this.webapiService.showErrorNotification("Please try again later!", "Something went wrong!!");
  }
}
