import { Component, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { InfraTemplates } from '../../create-template-project';
import { Project } from '../../model/project.model';
import { WebapiService } from '../../service/webapi.service';
import { DatePipe } from '@angular/common';
import { Log } from '../../model/project.model';

@Component({
  selector: 'app-createprojectconfig',
  templateUrl: './project.component.html',
  styleUrls: ['./project.component.css']
})
export class CreateprojectconfigComponent implements OnInit {
  pipe = new DatePipe('en-US');
  projectdetailsDataSource: any;
  projectDetails: any;
  projEnvId: string = "";
  projectForm: any;
  projDetails: Project;
  projectId: string;
  displaysubmit: boolean = false;
  loadSpinner: boolean; issubmitted: boolean = false;
  ispointerchecked: string = "yes";
  isProcessing: boolean = false;
  errorlog: Log = new Log();

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router
    , private infraTemplate: InfraTemplates, private toastr: ToastrService, private route: ActivatedRoute) { }

  ngOnInit(): void {
    // if (this.webapiService._createProjectfromTemplate.initiateRequest == false) {
    //   // Recent Projects
    //   /*Edit Project from Recent Projects*/
    //   if (this.webapiService._commonprojectdata.externalpid != undefined && this.webapiService._commonprojectdata.externalpid != "") {
    //     this.webapiService._commonprojectdata.pid = this.webapiService._commonprojectdata.externalpid;
    //     this.webapiService._commonprojectdata.externalpid = "";

    //     this.getProjectbyId(this.webapiService._commonprojectdata.pid);
    //     this.getProjectDetails();

    //   }
    //   else {
    //     this.webapiService._commonprojectdata.pid = "";
    //   }
    //   /*Edit Project from Recent Projects*/
    // }

    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Createprojectconfig - ngOnInit";
    this.errorlog.type = "Error";

    try {
      // /*Clear Id values*/
      this.webapiService._commonconfigproject.pcid = "";
      // this.webapiService._commonconfigproject.eid = "";
      if ((this.webapiService._commonconfigproject.configprojectenvid != undefined && this.webapiService._commonconfigproject.configprojectenvid != "") || (this.webapiService._commonconfigproject.eid != undefined && this.webapiService._commonconfigproject.eid != "")) {
        this.projEnvId = (this.webapiService._commonconfigproject.configprojectenvid != undefined && this.webapiService._commonconfigproject.configprojectenvid != "") ? this.webapiService._commonconfigproject.configprojectenvid : this.webapiService._commonconfigproject.eid;
        this.ispointerchecked = "no";
        this.webapiService._commonconfigproject.configprojectenvid = "";
      }
      this.ResetForm();

      this.getProjectDetails();
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  ResetForm() {
    this.projectId = '';
    this.projectForm = this.formbulider.group({
      projId: [''],
      projectname: ['', Validators.required],
      projectdescription: [''],
    });
  }
  applyFilter(filterValue: string) {
    filterValue = filterValue.trim();
    filterValue = filterValue.toLowerCase();
    this.projectdetailsDataSource = this.projectDetails;
    this.projectdetailsDataSource = this.projectdetailsDataSource.filter(x => x.name.toLowerCase().includes(filterValue));

    if (filterValue.trim() === '') {
      this.projectdetailsDataSource = this.projectDetails;
    }
  }

  getProjectDetails() {
    try {
      this.errorlog.modulename = "Createprojectconfig - getProjectDetails";
      this.projectdetailsDataSource = []; this.projectDetails = [];
      this.webapiService.getconfigprojects().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          res.forEach(x => {
            if (x.environmentid == this.projEnvId) {
              this.projectdetailsDataSource.push(x);
            }
          });
          this.projectDetails = this.projectdetailsDataSource;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
      this.ResetForm();
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  getProjectbyId(projectId: string) {
    try {
      this.errorlog.modulename = "Createprojectconfig - getProjectbyId";
      this.webapiService.getconfigprojectbyId(projectId).subscribe(res => {
        this.projectForm.controls['projectname'].setValue(res.name);
        this.projectForm.controls['projectdescription'].setValue(res.description);
        this.projectForm.controls['projId'].setValue(res._id);
        this.projectId = res._id;
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  saveProject(isSubmit: boolean) {
    try {
      this.errorlog.modulename = "Createprojectconfig - saveProject";

      if (!this.isProcessing) {
        this.isProcessing = true;

        const _proj = this.projectForm.value;
        this.projDetails = new Project();
        this.projDetails.name = _proj.projectname;
        this.projDetails.description = _proj.projectdescription;
        this.projDetails._id = _proj.projId;
        this.projDetails.createdby=localStorage.getItem("azapUserEmail");
        this.projDetails.updatedby=localStorage.getItem("azapUserEmail");
        if (this.projEnvId != "" && this.projEnvId != undefined) {
          if (_proj.projId == '' || _proj.projId == undefined) {
            this.projDetails.environmentid = this.projEnvId;
            this.projDetails.status = "draft";
            this.webapiService.saveconfigproject(this.projDetails).subscribe(res => {
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.getProjectDetails();
              this.toastr.success("Config Project details saved successfully!", "Create Config Project")
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
            });
          }
          else if (_proj.projId != '') {
            this.projDetails.status = "draft";
            this.webapiService.updateconfigproject(this.projDetails).subscribe(() => {
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.getProjectDetails();
              this.toastr.success("Config Project details updated successfully!", "Update Config Project")
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
            });
          }
        }
      }
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  onFormSubmit(data) {
    try {
      this.errorlog.modulename = "Createprojectconfig - onFormSubmit";
      this.saveProject(true);
      this.projectForm.reset();
      this._router.navigateByUrl('/');
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  editProjectContent(projectId: string, environmentid: string) {
    try {
      this.errorlog.modulename = "Createprojectconfig - editProjectContent";
      this.webapiService._commonconfigproject.pcid = projectId;
      this.webapiService._commonconfigproject.eid = environmentid;
      this._router.navigateByUrl('/configuration/hostgroup');
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  editProject(projectId: string, projectName: string, projectDescription: string, status: string) {
    try {
      this.errorlog.modulename = "Createprojectconfig - editProject";
      this.projectForm.controls['projId'].setValue(projectId);
      this.projectForm.controls['projectname'].setValue(projectName);
      this.projectForm.controls['projectdescription'].setValue(projectDescription);
      this.projectId = projectId;
      if (status.toLowerCase() == "submitted")
        this.issubmitted = true;
      else
        this.issubmitted = false;
    } catch (error) {

      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  deleteProject(projectId: string) {
    this.webapiService.deleteconfigproject(projectId).subscribe(() => {
      this.getProjectDetails();
    }, (error) => {
      this.errorlog.modulename = "Createprojectconfig - deleteProject";
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }
}
