import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { Project } from '../../model/project.model';
import { ConfigAssociation, ConfigHostGroup } from '../../model/configproject.model';
import { Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-reviewconfig',
  templateUrl: './review.component.html',
  styleUrls: ['./review.component.css']
})
export class ReviewconfigComponent implements OnInit {
  projDetails: Project = new Project();
  projectId: string;
  environmentId: string;
  hostgroupid: string;
  mstresid: string;
  associationid: string;
  hostgroupDetails: ConfigHostGroup[] = [];
  rolelist: ConfigAssociation[] = [];
  isactive = false;
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.projectId = this.webapiService._commonconfigproject.pcid;
    this.environmentId = this.webapiService._commonconfigproject.eid;
    this.hostgroupid = this.webapiService._commonconfigproject.hgid;
    this.mstresid = this.webapiService._commonconfigproject.resid;
    this.associationid = this.webapiService._commonconfigproject.configassid;
    if (this.projectId != undefined && this.projectId != "") {
      this.isactive = true;
      this.GetProjectDetails();
    }
  }
  GetProjectDetails() {
    this.webapiService.getconfigprojectbyId(this.projectId).subscribe(res => {
      this.projDetails = res;
      this.Gethostgrouplist();
    });
  }
  Gethostgrouplist() {
    this.webapiService.getHostGroupbyId(this.projectId).subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.hostgroupDetails = res;
        this.hostgroupDetails.forEach(y => { this.Getrolelist(y._id); });
      }
    });
  }
  Getrolelist(hostgroupid) {
    this.webapiService.getrolelistbyhgId(hostgroupid).subscribe(res => {
      if (typeof res != 'undefined') {
        res.forEach(y => {
          let customroles = [];
          y.rolelist.forEach(x => {
            customroles.push(x.rolename + "\n");
          });
          y.roles = customroles;
          this.rolelist.push(y);
        });
      }
    });
  }
  saveProject() {
    if (!this.isProcessing) {
      this.isProcessing = true;
      if (this.projDetails.environmentid != "" && this.projDetails.environmentid != undefined) {
        this.projDetails.status = "submitted";
        this.projDetails.updatedby = localStorage.getItem("azapUserEmail");
        let notificationmessage: string;
        notificationmessage = "\"" + this.projDetails.name + "\" submitted successfully! It usually takes 10-15 minutes to generate package. Once ready, it will be available in 'Deployment (Config)' screen!";
        this.webapiService.updateconfigproject(this.projDetails).subscribe(() => {
          //this.showNotification("\"" + this.projDetails.name + "\" submitted successfully!!", "InfraStructure Project:")
          this.showNotification(notificationmessage, "Configuration Project:");
          if (this.isProcessing) {
            this.isProcessing = false;
          }
          this.webapiService._commonconfigproject.configprojectenvid = this.projDetails.environmentid;
          this._router.navigateByUrl('/configuration/project');
        });
      }
    }
  }
  showNotification(message, title) {
    this.toastr.success(message, title)
  }
}
