import { Injectable } from '@angular/core';
import { Component, OnInit } from '@angular/core';
import { WebapiService } from './service/webapi.service';
import { Project, EnvironmentDetails } from './model/project.model'
import { Resources, ResourcesJson } from 'src/app/model/resourcegroup.model';
import { ResourceGroup, ResourceJsonTemplateDetails, CreateResource } from './model/resourcegroup.model';
import { InfraTemplate } from './model/template.model';
import { Observable, of as observableOf } from 'rxjs';
import { Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { environment } from 'src/environments/environment';

@Injectable({
    providedIn: 'root'
})

export class InfraTemplates {
    _projectCreated = "Project Created successfully. ";
    _environmentCreated = "Environment Created successfully. ";
    _rgCreated = "Resource Group Created successfully. "
    _resourceCreated = "Resource(s) Created successfully."

    projDetails: Project;
    environmentDetails: EnvironmentDetails;
    resourceGroup: ResourceGroup[];
    projectstatus: string;
    showSpinner: boolean = false;
    infraTemplate: InfraTemplate;
    notificationTitle: string = "Setup Project using Blueprint";

    constructor(private webapiService: WebapiService, private toastr: ToastrService) { }

    GetDateAsString() {
        let date = new Date();
        // format: dd/mm/yyyy_hh:mm:ss
        var dateformat =
            ("00" + (date.getMonth() + 1)).slice(-2) +
            ("00" + date.getDate()).slice(-2) +
            ("00" + date.getFullYear()).slice(-2) + "_" +
            ("00" + date.getHours()).slice(-2) +
            ("00" + date.getMinutes()).slice(-2) +
            ("00" + date.getSeconds()).slice(-2);

        return dateformat;
    }

    CreateProject() {
        this.projDetails = new Project();
        let value = this.getRandomInt(10, 50);
        this.projDetails.name = this.infraTemplate.project.name + "_" + value;
        this.projDetails.description = this.infraTemplate.project.description;
        this.projDetails.status = "draft";
        this.projDetails.createdby = localStorage.getItem("azapUserEmail");
        this.projDetails.updatedby = localStorage.getItem("azapUserEmail");
        this.projDetails.templateid = this.infraTemplate._id;
        this.webapiService.saveProject(this.projDetails).subscribe(res => {
            if (typeof res != undefined && res != '') {
                this.projectstatus = this._projectCreated + "Project Name:" + this.projDetails.name + ".";
                this.projDetails._id = res._id;

                this.showNotification(this.projectstatus, this.notificationTitle);

                this.environmentDetails = new EnvironmentDetails();
                this.CreateEnvironment();
            }
        });
        this.showSpinner = false;
    }

    getRandomInt(min, max) {
        min = Math.ceil(min);
        max = Math.floor(max);
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }

    showNotification(message, title) {
        this.toastr.success(message, title)
    }

    CreateEnvironment() {
        if (this.infraTemplate.environment.name != '' && this.infraTemplate.environment != undefined) {
            this.environmentDetails.projectid = this.projDetails._id;
            this.environmentDetails.name = this.infraTemplate.environment.name;
            this.environmentDetails.description = this.infraTemplate.environment.description;
            this.environmentDetails.createdby = localStorage.getItem("azapUserEmail");
            this.environmentDetails.updatedby = localStorage.getItem("azapUserEmail");
            this.environmentDetails.status = environment.draftStatus;
            this.webapiService.saveEnvironment(this.environmentDetails).subscribe(res => {
                if (typeof res != undefined && res != '') {
                    this.projectstatus = this._environmentCreated;
                    this.environmentDetails._id = res._id;
                    this.showNotification(this.projectstatus, this.notificationTitle);
                    this.CreateResourceGroup();
                }
                else {
                    this.showSpinner = false;
                }
            });
        }
    }

    CreateResourceGroup() {
        if (this.infraTemplate.resourcegroup != undefined) {
            this.resourceGroup = [];
            let rgIndex = 0;
            for (var rgDetails of this.infraTemplate.resourcegroup) {
                rgIndex = rgIndex + 1;
                let currentRG = new ResourceGroup();
                currentRG = rgDetails;
                currentRG.name = currentRG.name.toLowerCase().replace("timestamp", this.GetDateAsString());
                currentRG.environmentid = this.environmentDetails._id;
                currentRG.projectid = this.projDetails._id;
                currentRG.locktype = "NoLock";
                currentRG.createdby = localStorage.getItem("azapUserEmail");
                currentRG.updatedby = localStorage.getItem("azapUserEmail");
                let rgjson = JSON.stringify(currentRG);

                this.webapiService.createResourceGroup(rgjson, rgIndex).subscribe(res => {
                    currentRG._id = res._id;
                    this.resourceGroup.push(currentRG);
                    if (this.resourceGroup.length == this.infraTemplate.resourcegroup.length) {
                        this.projectstatus = this._rgCreated;
                        this.showNotification(this.projectstatus, this.notificationTitle);
                        //this.CreateResource();
                        this.CreateResource_new();
                    }
                });
            }
        }
        else {
            this.showSpinner = false;
        }
    }

    CreateResource_new() {
        let resourcecreated: boolean = false;
        if (this.resourceGroup != undefined && this.resourceGroup.length > 0) {
            for (let rg = 0; rg < this.resourceGroup.length; rg++) {
                for (let res = 0; res < this.resourceGroup[rg].resources.length; res++) {

                    let currentResource = new CreateResource();
                    currentResource.mstresourceid = this.resourceGroup[rg].resources[res].mstresourceid;
                    currentResource.properties = {};

                    currentResource.resourcegrpid = this.resourceGroup[rg]._id;
                    currentResource.createdby = localStorage.getItem("azapUserEmail");
                    currentResource.updatedby = localStorage.getItem("azapUserEmail");
                    currentResource.reference = [];
                    resourcecreated = true;
                    this.webapiService.saveResourcesJson(JSON.stringify(currentResource), res + 1).subscribe(res => {
                        currentResource._id = res._id;
                    });
                }
            }
            if (resourcecreated) {
                this.projectstatus = this._resourceCreated;
                this.showNotification(this.projectstatus, this.notificationTitle);
            }
        }
        this.showSpinner = false;
    }

    private RetrieveTemplateMetadata(templateId: string) {
        this.webapiService.getInfraTemplate(templateId).subscribe(res => {
            if (typeof res != undefined && res != '') {
                this.CreateProject();
            }
        });
    }

    GetInfraTemplate(templateId: string): Observable<any> {
        /*This Method will retrieve Template based on Id and create project based on template */
        this.projDetails = new Project();
        this.infraTemplate = new InfraTemplate();
        this.showSpinner = true;

        this.webapiService.getInfraTemplate(templateId).
            subscribe(res => {
                if (typeof res != undefined && res != '') {
                    this.infraTemplate = res;
                    this.CreateProject();
                    return true;
                }
            });
        return observableOf(true);
    }
}
