import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Log } from '../../model/project.model';

@Component({
  selector: 'app-blueprints',
  templateUrl: './blueprints.component.html',
  styleUrls: ['./blueprints.component.css']
})
export class TemplateComponent implements OnInit {

  templateDetails: any;
  templateDetailsDataSource: any;
  templatecategory: string;
  errorlog: Log = new Log();

  constructor(private webapiService: WebapiService, private _router: Router, private route: ActivatedRoute) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Template - ngOnInit";
    this.errorlog.type = "Error";
    try {
      this.route.queryParams.subscribe(params => {
        if (params.category != undefined && params.category != '') {
          this.templatecategory = params.category;
          this.getAllTemplates(this.templatecategory);
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  openReferenceArchitecture(imgname: string) {
    try {
      if (imgname != "") {
        $('.imgrefarchitect').attr('src', '/assets/images/architectimages/' + imgname);
      }
      else {
        $('.imgrefarchitect').attr('src', '/assets/images/notfound.png');
      }
      $('#RefernceArcitectModal').modal('show');
    } catch (error) {
      this.errorlog.modulename = "Template - openReferenceArchitecture";
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  getAllTemplates(category: string) {
    try {
      this.errorlog.modulename = "Template - getAllTemplates";
      this.templateDetails = [];
      this.webapiService.getAllTemplateDetails(category).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.templateDetails = res;
          this.templateDetailsDataSource = res;
        }
        else if (typeof res != 'undefined' && res.length == 0) {
          this.templateDetails = [];
          this.templateDetailsDataSource = [];
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  createproject(templateId: string) {
    this.webapiService._createProjectfromTemplate.initiateRequest = true;
    this.webapiService._createProjectfromTemplate.templateId = templateId;
    this._router.navigateByUrl('/infrastructure/project');
  }

  applyFilter(filterValue: string) {
    try {
      this.errorlog.modulename = "Template - applyFilter";
      filterValue = filterValue.trim(); // Remove whitespace
      filterValue = filterValue.toLowerCase();

      this.templateDetailsDataSource = this.templateDetails;
      this.templateDetailsDataSource = this.templateDetailsDataSource.filter(x => x.templatename.toLowerCase().includes(filterValue));

      if (filterValue.trim() === '') {
        this.templateDetailsDataSource = this.templateDetails;
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}
