import { Component, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common'
import { DeploymentDetails } from '../../model/template.model';
import { WebapiService } from '../../service/webapi.service';
import { Router } from '@angular/router';
import { Log } from '../../model/project.model';
//import { environment } from 'src/environments/environment';
import { environment } from 'src/environments/environment';
import { DatePipe } from '@angular/common';
import { MsalService } from '@azure/msal-angular';

@Component({
  selector: 'app-deployment',
  templateUrl: './deployment.component.html',
  styleUrls: ['./deployment.component.css']
})


export class DeploymentComponent implements OnInit {
  pipe = new DatePipe('en-US');
  validateDeploymentPackage = "Initialized Package validation!!!";
  deployDeploymentPackage = "Initialized Package Deployment!!!";
  rollbackDeploymentPackage = "Initialized Package Decommission!!!";
  deploymentStatusMsg: string = "";
  deployDetails: DeploymentDetails[] = [];
  isrecordavailable: boolean = false;
  errorlog: Log = new Log();
  logDetails: any = [];//to beupdated
  displayLoading: boolean = false;
  validateAction: string = '';
  deployAction: string = '';
  destroyAction: string = '';
  pipelinesucceededStatus: string = '';
  isLoggedInUserAdmin: boolean = false;
  validatePipelineName: string = '';
  deployPipelineName: string = '';
  decommPipelineName: string = '';

  constructor(private webapiService: WebapiService, private _router: Router) {
    this.validateAction = environment.pipelineValidateAction.toLowerCase();
    this.deployAction = environment.pipelineDeployAction.toLowerCase();
    this.destroyAction = environment.pipelineDestroyAction.toLowerCase();
    this.pipelinesucceededStatus = environment.pipelinesucceededstatus.toLowerCase();

    this.validatePipelineName = environment.validatePipelineName.toLowerCase();
    this.deployPipelineName = environment.deployPipelineName.toLowerCase();
    this.decommPipelineName = environment.decommPipelineName.toLowerCase();

  }

  ngOnInit(): void {
    try {
      this.errorlog.application = "Azap"; this.errorlog.type = "error";
      this.errorlog.modulename = "Infrastructure Deployment - ngOnInit";

      //let userRoles: string[] = this.authService.instance.getAllAccounts()[0].idTokenClaims.roles;
      //this.isLoggedInUserAdmin = (userRoles!=undefined) ? userRoles.indexOf(environment.adminRole) > -1: false;
      
      this.getDeploymentPackageDetails();
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  groupBy = function (xs, key) {
    return xs.reduce(function (rv, x) {
      (rv[x[key]] = rv[x[key]] || []).push(x);
      return rv;
    }, {});
  };

  getDeploymentPackageDetails() {
    try {
      this.errorlog.modulename = "Infrastructure Deployment - getDeploymentPackageDetails";
      this.webapiService.getDeploymentPackageDetails().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.isrecordavailable = true;
          this.deployDetails = res;
          this.deployDetails = this.groupBy(this.deployDetails, 'name');
        }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  validatePackage(packageName: string, environmentName: string) {
    try {
      this.errorlog.modulename = "Infrastructure Deployment - getDeploymentPackageDetails";
      this.deploymentStatusMsg = this.validateDeploymentPackage;
      this.showDeploymentStatus();
      this.webapiService.validateEnvironmentInfra(packageName, environmentName).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) { }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  deployPackage(packageName: string, environmentName: string) {
    try {
      this.errorlog.modulename = "Infrastructure Deployment - getDeploymentPackageDetails";
      this.deploymentStatusMsg = this.deployDeploymentPackage;
      this.showDeploymentStatus();
      this.webapiService.deploymentEnvironmentInfra(packageName, environmentName).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) { }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  rollbackPackage(packageName: string, environmentName: string) {
    try {
      this.errorlog.modulename = "Infrastructure Deployment - getDeploymentPackageDetails";
      this.deploymentStatusMsg = this.rollbackDeploymentPackage;
      this.showDeploymentStatus();
      this.webapiService.rollbackEnvironmentInfra(packageName, environmentName).subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) { }
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  configPackage(packageName: string, environmentId: string) {
    try {
      this.errorlog.modulename = "Infrastructure Deployment - getDeploymentPackageDetails";
      this.webapiService._commonconfigproject.configprojectenvid = environmentId;
      this._router.navigateByUrl('/configuration/project');
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  showDeploymentStatus() {   
    $('#Modal').modal('show');
  }

  downloadpackage(packageName: string, packageVersion: string) {
    let packagelink = environment.terraformpackagedownloadlink.replace(environment.packagename, packageName).replace(environment.packageversion, packageVersion);
     window.open(packagelink, "_blank");
    // this.webapiService.getDownloadPackage(packagelink).subscribe(res => {
    //   const blob = new Blob([res], { 
    //     type: 'application/octet-stream'
    //   });
    //   const url = window.URL.createObjectURL(blob);
    //   var anchor = document.createElement("a");
    //   anchor.download = packageName + "_" + packageVersion +  ".nupkg";
    //   anchor.href = url;
    //   anchor.click();
    //   });
  // this.webapiService.getReleaseLog(packagelink).subscribe(res => {
  //   const blob = new Blob([res]);
  //   const url = window.URL.createObjectURL(blob);
  //   var anchor = document.createElement("a");
  //   anchor.download =packageName+ ".nupkg";
  //   anchor.href = url;
  //   anchor.click();
  //   });
  }
  downloareleaseid(releaseid: string) {
    let packagelink = environment.releaseLogDetails.replace("{{releaseId}}", releaseid);
    this.webapiService.getReleaseLog(packagelink).subscribe(res => {
      const blob = new Blob([res], { 
        type: 'application/zip'
      });
      const url = window.URL.createObjectURL(blob);
      var anchor = document.createElement("a");
      anchor.download = "ReleaseLogs_"+releaseid+".zip";
      anchor.href = url;
      anchor.click();
      });
  }
  downloadtasklog(tasklogurl:string,taskId:string)
  {
    this.webapiService.getReleaseLog(tasklogurl).subscribe(res => {
      const blob = new Blob([res]);
      const url = window.URL.createObjectURL(blob);
      var anchor = document.createElement("a");
      anchor.download = "tasklog_"+taskId+".log";
      anchor.href = url;
      anchor.click();
      });
  }
  displayLogDetails(packageid,action) {
    this.webapiService.getDeploymentLog(packageid, action).subscribe(res => {
      if (typeof res != 'undefined' ) { 
        this.logDetails=res;
        $('#ModalLogDetails').modal('show');
      }
    });
  }

  displayValidate(releaseDefinitionName: string, status: string) {
    if ((releaseDefinitionName.toLowerCase().includes(this.validateAction)
      && !(status.toLowerCase() === this.pipelinesucceededStatus)) ||
      (releaseDefinitionName === '' && status === '')) {
      return true;
    }
    return false;
  }

  displayDeploy(releaseDefinitionName: string, status: string) {
    if ((releaseDefinitionName.toLowerCase().includes(this.validateAction)
      && status.toLowerCase()===(this.pipelinesucceededStatus)) || 
      (releaseDefinitionName.toLowerCase().includes(this.deployAction) &&
      !(status.toLowerCase() === this.pipelinesucceededStatus))) {
      return true;
    }
    return false;
  }

  displayrollback(releaseDefinitionName: string, status: string) {
    if ((releaseDefinitionName.toLowerCase().includes(this.deployAction)
      && status.toLowerCase() === this.pipelinesucceededStatus) || 
      (releaseDefinitionName.toLowerCase().includes(this.destroyAction) &&
      !(status.toLowerCase() === this.pipelinesucceededStatus))) {
      return true;
    }
    return false;
  }

  displayConfig(releaseDefinitionName: string, status: string) {
    if ((releaseDefinitionName.toLowerCase().includes(this.deployAction)
      && status.toLowerCase() === this.pipelinesucceededStatus)) {
      return true;
    }
    return false;
  }

}

