import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { Project, projectEnvironment, replicateEnvironment, deleteEnvironment, EnvironmentDetails, Environment } from '../../model/project.model'
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { Log } from '../../model/project.model';
import { environment } from 'src/environments/environment';
import { Azurecontainer, AzureResourceGroup, AzureServiceprinciple, AzureStorageaccount, AzureSubscription, Azuretenant } from '../../model/azuretenant.model';

@Component({
  selector: 'app-environment',
  templateUrl: './environment.component.html',
  styleUrls: ['./environment.component.css']
})
export class EnvironmentComponent implements OnInit {
  azureServicePrincipledetails: AzureServiceprinciple[] = [];
  terraprojectId: string = "";
  environmentId: string;
  environmentDetails: Environment[] = [];
  environmentForm: any;
  newEnvironment: EnvironmentDetails;
  ispointerchecked: string = "yes";
  isProcessing: boolean = false;
  errorlog: Log = new Log();
  environmentStatus: string = "";
  issubmitted: boolean = false;
  azuretenantdetails: Azuretenant[] = [];
  azuresubscriptiondetails: AzureSubscription[] = [];
  azureResourceGroupdetails: AzureResourceGroup[] = [];
  azureStorageAccountDetails: AzureStorageaccount[] = [];
  azureStorageContainerDetails: Azurecontainer[] = [];
  constructor(private webapiService: WebapiService, private formbulider: FormBuilder,
    private route: ActivatedRoute, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap"; this.errorlog.type = "error";
    this.errorlog.modulename = "Infrastructure Environment - ngOnInit";
    try {

      this.ResetForm();

      this.terraprojectId = this.webapiService._commonprojectdata.pid;
      if (this.terraprojectId != undefined && this.terraprojectId != "") {
        this.ispointerchecked = "no";
        this.loadEnvironmentDetails();
        this.LoadAzureTenantDetails();
      }
      else {
        this._router.navigateByUrl('/infrastructure/project');
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  ResetForm() {
    this.environmentForm = this.formbulider.group({
      environmentId: [''],
      environmentname: ['', Validators.required],
      environmentdescription: [''],
      environmentStatus: [''],
      tenantid: ['', Validators.required],
      subscriptionid: ['', Validators.required],
      serviceprincipalid: ['', Validators.required],
      infrastateresourcegroupid: ['', Validators.required],
      storageaccountid: ['', Validators.required],
      storagecontainerid : ['', Validators.required]
    });
  }
  LoadAzureTenantDetails() {
    this.errorlog.modulename = "Environment - LoadAzureTenant Dropdown";
    try {
      this.webapiService.getallAzureTenantDetails().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          res.forEach(ele => {
            if (ele.tenantid != "") {
              this.azuretenantdetails.push(ele);
            }
          });
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  loadEnvironmentDetails() {
    try {
      this.errorlog.modulename = "Infrastructure Environment - loadEnvironmentDetails";
      this.environmentDetails = [];
      this.webapiService.getEnvironmentDetails(this.terraprojectId).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.environmentDetails = res;
        }

      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  deleteEnvironment(envId: string) {
    try {
      this.errorlog.modulename = "Infrastructure Environment - deleteEnvironment";
      this.webapiService.deleteEnvironment(envId).subscribe(() => {
        this.loadEnvironmentDetails();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
    this.ResetForm();
  }

  editEnvironmentRG(environmentId: string) {
    this.webapiService._commonprojectdata.eid = environmentId;
    this._router.navigateByUrl('/infrastructure/resourcegroup');
  }

  editenvironment(envId: string) {
    this.ResetForm();
    let env = this.environmentDetails.find(x => x._id == envId);
    this.environmentForm.controls['environmentname'].setValue(env.name);
    this.environmentForm.controls['environmentdescription'].setValue(env.description);
    this.environmentForm.controls['environmentId'].setValue(env._id);
    this.environmentForm.controls['environmentStatus'].setValue(env.status);
    if (env.tenantid != null && env.tenantid != "") {
      this.environmentForm.controls['tenantid'].setValue(env.tenantid);
      this.Loadsubsbytenant(env.tenantid, env.subscriptionid, env.serviceprincipalid, env.infrastateresourcegroupid, env.storageaccountid, env.storagecontainerid);
    }
    if (env.status != 'draft') {
      this.issubmitted = true;
    }
  }

  saveEnvironment() {
    try {
      this.errorlog.modulename = "Infrastructure Environment - saveEnvironment";
      const _projenv = this.environmentForm.value;
      if((this.environmentDetails.length == 0 && _projenv.environmentId == '') || 
      (this.environmentDetails.length !=0 && _projenv.environmentId != undefined && _projenv.environmentId != '' ))
      {
      if (!this.isProcessing) {
        this.isProcessing = true;
        this.newEnvironment = new EnvironmentDetails();
        this.newEnvironment = _projenv;
        this.newEnvironment.name = _projenv.environmentname;
        this.newEnvironment.description = _projenv.environmentdescription;
        this.newEnvironment.projectid = this.terraprojectId;
        this.newEnvironment.createdby = localStorage.getItem("azapUserEmail");
        this.newEnvironment.updatedby = localStorage.getItem("azapUserEmail");
        this.newEnvironment.status = _projenv.environmentStatus;

        if (this.newEnvironment.name != '' && this.newEnvironment.name != undefined) {
          if (_projenv.environmentId != '' && _projenv.environmentId != undefined) {
            this.newEnvironment._id = _projenv.environmentId;
            this.webapiService.updateEnvironment(this.newEnvironment).subscribe(() => {
              this.environmentForm.reset();
              this.loadEnvironmentDetails();
              this.toastr.success("Environment details updated successfully!", "Update Environment");
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
          else {
            this.newEnvironment.status = environment.draftStatus;
            this.webapiService.saveEnvironment(this.newEnvironment).subscribe(() => {
              this.environmentForm.reset();
              this.loadEnvironmentDetails();
              this.toastr.success("Environment details saved successfully!", "Create Environment");
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            }, (error) => {
              this.errorlog.remark = error;
              this.webapiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
        }
      }
    }
    else{
       this.webapiService.showErrorNotification("Only one environment can be added!", "Save Environment failed");
    }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  replicateEnvironment(envId: string) {
    this.webapiService.replicateEnvironment(envId).subscribe(() => {
      this.loadEnvironmentDetails();
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  displayEdit(environmentStatus: string) {
    if (environmentStatus === null || environmentStatus === '') { return true; }
    else if (environmentStatus.toLowerCase() === environment.draftStatus.toLowerCase() ||
      environmentStatus.toLowerCase() === 'deployed') {
      return true;
    }
    else {
      return false;
    }
  }

  displayDelete(environmentStatus: string) {
    if (environmentStatus === null || environmentStatus === '') { return false; }
    else if (environmentStatus.toLowerCase() === environment.draftStatus.toLowerCase()) {
      return true;
    }
    else {
      return false;
    }
  }
  Loadsubsbytenant(tenantid: string, selectsubid: string = "", selectspid: string = "", selectrgid: string = "", selectstorageaccountid: string="", selectcontainerid:string="") {
    this.environmentForm.controls['subscriptionid'].setValue('');
    this.environmentForm.controls['serviceprincipalid'].setValue('');
    this.environmentForm.controls['infrastateresourcegroupid'].setValue('');
    this.environmentForm.controls['storageaccountid'].setValue('');
    this.environmentForm.controls['storagecontainerid'].setValue('');
    if (tenantid != "") {
      try {
        this.azuresubscriptiondetails = [];
        this.azureServicePrincipledetails=[];
        this.azureResourceGroupdetails=[];
        this.azureStorageAccountDetails=[];
        this.azureStorageContainerDetails=[];

        this.webapiService.getallAzureSubscribeDetails(tenantid).subscribe(res => {
          if (res != undefined && res.length > 0) {
            this.azuresubscriptiondetails = res;
            if (selectsubid != "") {
              this.environmentForm.controls['subscriptionid'].setValue(selectsubid);
              this.Loadspbysubs(selectsubid, selectspid, selectrgid, selectstorageaccountid, selectcontainerid);
            }
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
      }
      catch (error) {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      }
    }
  }
  Loadspbysubs(subscriptionid: string, selectspid: string = "", selectrgid: string = "", selectstorageaccountid:string="", selectcontainerid:string="") {
    try {
      this.environmentForm.controls['serviceprincipalid'].setValue('');
      this.environmentForm.controls['infrastateresourcegroupid'].setValue('');
      this.environmentForm.controls['storageaccountid'].setValue('');
      this.environmentForm.controls['storagecontainerid'].setValue('');
      let tenantid = $('#tenantid').val().toString();
      if (tenantid != "" && subscriptionid != "") {
        this.azureServicePrincipledetails = [];
        this.azureResourceGroupdetails=[];
        this.azureStorageAccountDetails=[];
        this.azureStorageContainerDetails=[];

        this.webapiService.getallAzureSPDetails(tenantid, subscriptionid).subscribe(res => {
          if (res != undefined && res.length > 0) {
            this.azureServicePrincipledetails = res;
            if (selectspid != ""){
              this.environmentForm.controls['serviceprincipalid'].setValue(selectspid);
            this.Loadrgbysp(selectspid, selectrgid, selectstorageaccountid, selectcontainerid)
            }
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  Loadrgbysp(spid: string, selectrgid: string = "", selectstorageaccountid: string="", selectcontainerid:string="") {
    this.environmentForm.controls['infrastateresourcegroupid'].setValue('');
    this.environmentForm.controls['storageaccountid'].setValue('');
    this.environmentForm.controls['storagecontainerid'].setValue('');
    let tenantid = $('#tenantid').val().toString();
    let subscriptionid = $('#subscriptionid').val().toString();
    try {
      if (tenantid != "" && subscriptionid != "" && spid != "") {
        this.azureResourceGroupdetails = [];
        this.azureStorageAccountDetails=[];
        this.azureStorageContainerDetails=[];

        this.webapiService.getallAzurergDetails(tenantid, subscriptionid, spid).subscribe(res => {
          if (res != undefined && res.length > 0) {
            this.azureResourceGroupdetails = res;
            if (selectrgid != ""){
              this.environmentForm.controls['infrastateresourcegroupid'].setValue(selectrgid);
              this.Loadstorageaccountbyrg(selectrgid,selectstorageaccountid, selectcontainerid)
            }
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
  Loadstorageaccountbyrg(rgid: string, selectstorageaccountid: string = "", selectcontainerid: string="") {
    try {
      this.environmentForm.controls['storageaccountid'].setValue('');
      this.environmentForm.controls['storagecontainerid'].setValue('');
      let tenantid = $('#tenantid').val().toString();
      let subscriptionid = $('#subscriptionid').val().toString();
      let spid = $('#serviceprincipalid').val().toString();
      if (tenantid != "" && subscriptionid != "" && spid!= "" && rgid!= "") {
        this.azureStorageAccountDetails = [];
        this.azureStorageContainerDetails=[];
        
        this.webapiService.getallAzureStorageAccountDetails(tenantid, subscriptionid, spid, rgid).subscribe(res => {
          if (res != undefined && res.length > 0) {
            this.azureStorageAccountDetails = res;
             if (selectstorageaccountid != ""){
              this.environmentForm.controls['storageaccountid'].setValue(selectstorageaccountid);
              this.Loadstoragecontainerbysa(selectstorageaccountid, selectcontainerid)
             }
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
Loadstoragecontainerbysa(said: string, selectcontainerid: string="") {
  try {
      this.environmentForm.controls['storagecontainerid'].setValue('');
      let tenantid = $('#tenantid').val().toString();
      let subscriptionid = $('#subscriptionid').val().toString();
      let spid = $('#serviceprincipalid').val().toString();
      let rgid = $('#infrastateresourcegroupid').val().toString();
      if (tenantid != "" && subscriptionid != "" && spid!= "" && rgid!= "" && said!="") {
        this.azureStorageContainerDetails = [];
        this.webapiService.getallAzureStorageContainerDetails(tenantid, subscriptionid, spid, rgid, said).subscribe(res => {
          if (res != undefined && res.length > 0) {
            this.azureStorageContainerDetails = res;
            if (selectcontainerid != ""){
              this.environmentForm.controls['storagecontainerid'].setValue(selectcontainerid);
            }
          }
        }, (error) => {
          this.errorlog.remark = error;
          this.webapiService.ErrorLogfn(this.errorlog);
        });
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}