import { Component, Inject, OnInit } from "@angular/core";
import {
  MatDialog,
  MatDialogRef,
  MAT_DIALOG_DATA,
} from "@angular/material/dialog";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { ToastrService } from "ngx-toastr";
import {
  Environment,
  EnvironmentDetails,
  Log,
  Project,
} from "src/app/model/project.model";
import { WebapiService } from "src/app/service/webapi.service";
import {
  Azurecontainer,
  AzureResourceGroup,
  AzureServiceprinciple,
  AzureStorageaccount,
  AzureSubscription,
  Azuretenant,
} from "src/app/model/azuretenant.model";
import { FormBuilder, Validators } from "@angular/forms";
import {
  CreateResource,
  CreateExportedResource,
  ResourceGroup,
  Resources,
} from "src/app/model/resourcegroup.model";
import { concatMap } from 'rxjs/operators';
import { from, Observable } from 'rxjs';
import { environment } from "src/environments/environment";

@Component({
  selector: "app-projectexport",
  templateUrl: "./projectexport.component.html",
  styleUrls: ["./projectexport.component.css"],
})
export class ProjectExportDialogComponent implements OnInit {
  azureServicePrincipledetails: AzureServiceprinciple[] = [];
  terraprojectId: string = "";
  environmentId: string;
  // environmentDetails: Environment[] = [];
  exportProjectForm: any;
  newEnvironment: EnvironmentDetails;
  ispointerchecked: string = "yes";
  isProcessing: boolean = false;
  errorlog: Log = new Log();
  environmentStatus: string = "";
  projectStatus: string = "";
  projectId: string = "";
  issubmitted: boolean = false;
  azuretenantdetails: Azuretenant[] = [];
  azuresubscriptiondetails: AzureSubscription[] = [];
  azureResourceGroupdetails: AzureResourceGroup[] = [];
  azureStorageAccountDetails: AzureStorageaccount[] = [];
  azureStorageContainerDetails: Azurecontainer[] = [];
  projDetails: Project;
  environmentDetails: EnvironmentDetails;
  resourceGroup: ResourceGroup[];
  loadSpinner: boolean;
  currentRGName: string;
  resourcesList = [];
  newResourceGroup = [];
  oldResourceGroup = [];
  closeDialogCounter = 0
  environmentDetail: Environment[] = [];
  newEnvironmentId: any;
  newProjectId: any;

  constructor(
    public secdialog: MatDialog,
    public dialogRef: MatDialogRef<ProjectExportDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private http: HttpClient,
    public apiService: WebapiService,
    private formbulider: FormBuilder,
    private toastr: ToastrService
  ) {
    this.projectId = data.projectId
  }

  ngOnInit(): void {
    this.ResetForm();
    this.LoadAzureTenantDetails();
  }

  ResetForm() {
    this.exportProjectForm = this.formbulider.group({
      projectId: [""],
      projectStatus: [""],
      projectName: ["", Validators.required],
      projectDescription: [""],
      environmentId: [""],
      environmentName: ["", Validators.required],
      environmentDescription: [""],
      environmentStatus: [""],
      tenantid: ["", Validators.required],
      subscriptionid: ["", Validators.required],
      serviceprincipalid: ["", Validators.required],
      infrastateresourcegroupid: ["", Validators.required],
      storageaccountid: ["", Validators.required],
      storagecontainerid: ["", Validators.required],
    });
  }

  saveDetails() {
    this.loadSpinner = true;
    const _proj = this.exportProjectForm.value;
    this.projDetails = new Project();
    this.projDetails.name = _proj.projectName;
    this.projDetails.description = _proj.projectDescription;
    this.projDetails._id = _proj.projectId;
    this.projDetails.createdby = localStorage.getItem("azapUserEmail");
    this.projDetails.updatedby = localStorage.getItem("azapUserEmail");
    if (_proj.projectId == "" || _proj.projectId == undefined) {
      this.projDetails.status = "draft";
      this.apiService.saveProject(this.projDetails).subscribe(
        (res) => {
          this.projDetails._id = res._id;
          this.newProjectId = res._id;
          this.CreateEnvironment();
        },
        (error) => {
          this.errorlog.remark = error;
          this.toastr.error("Project with same name already exist!", "Export Project");
          this.loadSpinner = false;
        }
      );
    }
  }

  CreateEnvironment() {
    const _proj = this.exportProjectForm.value;
    this.environmentDetails = new EnvironmentDetails();
    if (_proj.environmentName != "" && _proj.environmentName != undefined) {
      this.environmentDetails.projectid = this.projDetails._id;
      this.environmentDetails.name = _proj.environmentName;
      this.environmentDetails.description = _proj.environmentDescription;
      this.environmentDetails.createdby = localStorage.getItem("azapUserEmail");
      this.environmentDetails.updatedby = localStorage.getItem("azapUserEmail");
      this.environmentDetails.subscriptionid = _proj.subscriptionid;
      this.environmentDetails.tenantid = _proj.tenantid;
      this.environmentDetails.serviceprincipalid = _proj.serviceprincipalid;
      this.environmentDetails.infrastateresourcegroupid = _proj.infrastateresourcegroupid;
      this.environmentDetails.storageaccountid = _proj.storageaccountid;
      this.environmentDetails.storagecontainerid = _proj.storagecontainerid;
      this.environmentDetails.status = "draft";
      this.apiService
        .saveEnvironment(this.environmentDetails)
        .subscribe((res) => {
          if (typeof res != undefined && res != "") {
            this.environmentDetails._id = res._id;
            this.newEnvironmentId = res._id;
            this.getExportedResourceGroupsbyProjectId();
          } else {
            this.loadSpinner = false;
          }
        });
    }
  }

  getExportedResourceGroupsbyProjectId() {
    if (this.projectId != "" && this.projectId != undefined) {
      this.resourceGroup = []
      this.apiService.getExportedResourceGroupsbyProjectId(this.projectId).subscribe((res) => {
        if (typeof res != undefined && res != "" && res?.length > 0) {
          this.resourceGroup = res;
          this.CreateResourceGroup()
        }
        else {
          this.dialogRef.close();
          this.loadSpinner = false;
        }
      });
    }
  }

  CreateResourceGroup() {
    if (this.resourceGroup?.length > 0) {

      from(this.resourceGroup).pipe(
        concatMap((rgDetails) => {
          let currentRGId = rgDetails._id
          let currentRG = new ResourceGroup();
          currentRG = rgDetails;
          currentRG.name = 'update_resource_group_name' + this.GetDateAsString() + '_' + this.getRandomInt(0, 400);
          currentRG.environmentid = this.newEnvironmentId;
          currentRG.projectid = this.newProjectId;
          currentRG.createdby = localStorage.getItem("azapUserEmail");
          currentRG.updatedby = localStorage.getItem("azapUserEmail");
          let rgjson = JSON.stringify(currentRG);
          return new Observable(observer => {
            this.apiService.exportResourceGroup(rgjson, false).subscribe({
              next: (createResourceGroupResponse: any) => {
                this.apiService.getExportedResourcesByRgId(rgDetails._id).subscribe(getResourcesResponse => {
                  this._createResources(createResourceGroupResponse._id, getResourcesResponse).subscribe({
                    complete: () => {
                      observer.complete();
                    }, error: (err: any) => {
                      //TODO: handle error
                      observer.error("Error creating resources");
                    }
                  });
                }, (err: any) => {
                  //TODO: handle error
                  observer.error("Error retreiving resource list of group => " + rgDetails.name);
                });
              }, error: (err: any) => {
                // TODO: handle error
                observer.error("Error creating group => " + currentRG.name);
              }
            });
          });
        })
      ).subscribe({
        complete: () => {
          this.toastr.success("Project exported successfully!", "Azure Export Project");
          this.dialogRef.close();
          this.loadSpinner = false;
        },
        error: (err) => {
          this.errorlog.remark = err.toString();
          this.apiService.ErrorLogfn(this.errorlog);
          this.toastr.error("There was an error cloing the project", "Azure Export Project");
          this.loadSpinner = false;
        }
      });
    }
    else {
      this.dialogRef.close()
      this.loadSpinner = false;
    }
  }


  private _createResources(resourceGroupId: any, resources: Array<any>): Observable<any> {
    return from(resources).pipe(
      concatMap((resourceItem) => {
        let currentResource = new CreateExportedResource();
        currentResource.mstresourceid = resourceItem.mstresourceid;
        currentResource.properties = resourceItem.properties;
        currentResource.terraformuniquename = resourceItem.terraformuniquename;
        currentResource.isdeleted = resourceItem.isdeleted;
        currentResource.resourcegrpid = resourceGroupId;
        currentResource.preexisting = resourceItem.preexisting;
        currentResource.createdby = localStorage.getItem("azapUserEmail");
        currentResource.updatedby = localStorage.getItem("azapUserEmail");
        currentResource.reference = [];
        return this.apiService.saveExportResourcesJson(JSON.stringify(currentResource), false);
      })
    );
  }

  async CreateResource_new() {
    let resourcecreated: boolean = false;
    this.closeDialogCounter = 0
    if (this.resourceGroup != undefined && this.resourceGroup.length > 0) {
      var rgCounter: number = -1;
      from(this.resourceGroup).pipe(
        concatMap((rg) => {
          return this.apiService.getExportedResourcesByRgId(rg._id);
        })
      ).subscribe(res => {
        rgCounter++;
        if (typeof res != "undefined" && res.length > 0) {
          this.resourcesList = res;
        }
        if (this.resourcesList != undefined && this.resourcesList.length > 0) {
          var rsCounter: number = -1;
          from(this.resourcesList).pipe(
            concatMap((resr) => {
              let currentResource = new CreateExportedResource();
              currentResource.mstresourceid = resr.mstresourceid;
              currentResource.properties = resr.properties;
              currentResource.terraformuniquename = resr.terraformuniquename;
              currentResource.isdeleted = resr.isdeleted;
              currentResource.resourcegrpid = this.newResourceGroup[rgCounter]._id;
              currentResource.preexisting = resr.preexisting;
              currentResource.createdby = localStorage.getItem("azapUserEmail");
              currentResource.updatedby = localStorage.getItem("azapUserEmail");
              currentResource.reference = [];
              resourcecreated = true;
              return this.apiService.saveExportResourcesJson(JSON.stringify(currentResource), false);
            })
          ).subscribe(res => {
            rsCounter++;
            if (this.resourceGroup.length == (rgCounter + 1)) {
              if (this.resourcesList.length == (rsCounter + 1)) {
                this.toastr.success("Project exported successfully!", "Azure Export Project");
                this.dialogRef.close();
                // this.closeDialogCounter++;
              }
            }
          });

        }
      });
    }
  }

  Loadsubsbytenant(
    tenantid: string,
    selectsubid: string = "",
    selectspid: string = "",
    selectrgid: string = "",
    selectstorageaccountid: string = "",
    selectcontainerid: string = ""
  ) {
    this.exportProjectForm.controls["subscriptionid"].setValue("");
    this.exportProjectForm.controls["serviceprincipalid"].setValue("");
    this.exportProjectForm.controls["infrastateresourcegroupid"].setValue("");
    this.exportProjectForm.controls["storageaccountid"].setValue("");
    this.exportProjectForm.controls["storagecontainerid"].setValue("");
    if (tenantid != "") {
      try {
        this.azuresubscriptiondetails = [];
        this.azureServicePrincipledetails = [];
        this.azureResourceGroupdetails = [];
        this.azureStorageAccountDetails = [];
        this.azureStorageContainerDetails = [];

        this.apiService.getallAzureSubscribeDetails(tenantid).subscribe(
          (res) => {
            if (res != undefined && res.length > 0) {
              this.azuresubscriptiondetails = res;
              if (selectsubid != "") {
                this.exportProjectForm.controls["subscriptionid"].setValue(
                  selectsubid
                );
                this.Loadspbysubs(
                  selectsubid,
                  selectspid,
                  selectrgid,
                  selectstorageaccountid,
                  selectcontainerid
                );
              }
            }
          },
          (error) => {
            this.errorlog.remark = error;
            this.apiService.ErrorLogfn(this.errorlog);
          }
        );
      } catch (error) {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      }
    }
  }
  Loadspbysubs(
    subscriptionid: string,
    selectspid: string = "",
    selectrgid: string = "",
    selectstorageaccountid: string = "",
    selectcontainerid: string = ""
  ) {
    try {
      this.exportProjectForm.controls["serviceprincipalid"].setValue("");
      this.exportProjectForm.controls["infrastateresourcegroupid"].setValue("");
      this.exportProjectForm.controls["storageaccountid"].setValue("");
      this.exportProjectForm.controls["storagecontainerid"].setValue("");
      let tenantid = $("#tenantid").val().toString();
      if (tenantid != "" && subscriptionid != "") {
        this.azureServicePrincipledetails = [];
        this.azureResourceGroupdetails = [];
        this.azureStorageAccountDetails = [];
        this.azureStorageContainerDetails = [];
        this.apiService
          .getallAzureSPDetails(tenantid, subscriptionid)
          .subscribe(
            (res) => {
              if (res != undefined && res.length > 0) {
                this.azureServicePrincipledetails = res;
                if (selectspid != "") {
                  this.exportProjectForm.controls[
                    "serviceprincipalid"
                  ].setValue(selectspid);
                  this.Loadrgbysp(
                    selectspid,
                    selectrgid,
                    selectstorageaccountid,
                    selectcontainerid
                  );
                }
              }
            },
            (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
            }
          );
      }
    } catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  Loadrgbysp(
    spid: string,
    selectrgid: string = "",
    selectstorageaccountid: string = "",
    selectcontainerid: string = ""
  ) {
    this.exportProjectForm.controls["infrastateresourcegroupid"].setValue("");
    this.exportProjectForm.controls["storageaccountid"].setValue("");
    this.exportProjectForm.controls["storagecontainerid"].setValue("");
    let tenantid = $("#tenantid").val().toString();
    let subscriptionid = $("#subscriptionid").val().toString();
    try {
      if (tenantid != "" && subscriptionid != "" && spid != "") {
        this.azureResourceGroupdetails = [];
        this.azureStorageAccountDetails = [];
        this.azureStorageContainerDetails = [];
        this.apiService
          .getallAzurergDetails(tenantid, subscriptionid, spid)
          .subscribe(
            (res) => {
              if (res != undefined && res.length > 0) {
                this.azureResourceGroupdetails = res;
                if (selectrgid != "") {
                  this.exportProjectForm.controls[
                    "infrastateresourcegroupid"
                  ].setValue(selectrgid);
                  this.Loadstorageaccountbyrg(
                    selectrgid,
                    selectstorageaccountid,
                    selectcontainerid
                  );
                }
              }
            },
            (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
            }
          );
      }
    } catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  Loadstorageaccountbyrg(
    rgid: string,
    selectstorageaccountid: string = "",
    selectcontainerid: string = ""
  ) {
    try {
      this.currentRGName = this.azureResourceGroupdetails.find(
        (x) => x._id == rgid
      ).name;
      this.exportProjectForm.controls["storageaccountid"].setValue("");
      this.exportProjectForm.controls["storagecontainerid"].setValue("");
      let tenantid = $("#tenantid").val().toString();
      let subscriptionid = $("#subscriptionid").val().toString();
      let spid = $("#serviceprincipalid").val().toString();
      if (tenantid != "" && subscriptionid != "" && spid != "" && rgid != "") {
        this.azureStorageAccountDetails = [];
        this.azureStorageContainerDetails = [];

        this.apiService
          .getallAzureStorageAccountDetails(
            tenantid,
            subscriptionid,
            spid,
            rgid
          )
          .subscribe(
            (res) => {
              if (res != undefined && res.length > 0) {
                this.azureStorageAccountDetails = res;
                if (selectstorageaccountid != "") {
                  this.exportProjectForm.controls["storageaccountid"].setValue(
                    selectstorageaccountid
                  );
                  this.Loadstoragecontainerbysa(
                    selectstorageaccountid,
                    selectcontainerid
                  );
                }
              }
            },
            (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
            }
          );
      }
    } catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  Loadstoragecontainerbysa(said: string, selectcontainerid: string = "") {
    try {
      this.exportProjectForm.controls["storagecontainerid"].setValue("");
      let tenantid = $("#tenantid").val().toString();
      let subscriptionid = $("#subscriptionid").val().toString();
      let spid = $("#serviceprincipalid").val().toString();
      let rgid = $("#infrastateresourcegroupid").val().toString();
      if (
        tenantid != "" &&
        subscriptionid != "" &&
        spid != "" &&
        rgid != "" &&
        said != ""
      ) {
        this.azureStorageContainerDetails = [];
        this.apiService
          .getallAzureStorageContainerDetails(
            tenantid,
            subscriptionid,
            spid,
            rgid,
            said
          )
          .subscribe(
            (res) => {
              if (res != undefined && res.length > 0) {
                this.azureStorageContainerDetails = res;
                if (selectcontainerid != "") {
                  this.exportProjectForm.controls[
                    "storagecontainerid"
                  ].setValue(selectcontainerid);
                }
              }
            },
            (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
            }
          );
      }
    } catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  onNoClick(): void {
    this.dialogRef.close();
  }

  GetDateAsString() {
    let date = new Date();
    // format: dd/mm/yyyy_hh:mm:ss
    var dateformat =
      ("00" + (date.getMonth() + 1)).slice(-2) +
      ("00" + date.getDate()).slice(-2) +
      ("00" + date.getFullYear()).slice(-2) +
      "_" +
      ("00" + date.getHours()).slice(-2) +
      ("00" + date.getMinutes()).slice(-2) +
      ("00" + date.getSeconds()).slice(-2);

    return dateformat;
  }

  LoadAzureTenantDetails() {
    this.errorlog.modulename = "Environment - LoadAzureTenant Dropdown";
    try {
      this.apiService.getallAzureTenantDetails().subscribe(
        (res) => {
          if (typeof res != "undefined" && res.length > 0) {
            res.forEach((ele) => {
              if (ele.tenantid != "") {
                this.azuretenantdetails.push(ele);
              }
            });
            this.loadEnvironmentDetails();
          }
        },
        (error) => {
          this.errorlog.remark = error;
          this.apiService.ErrorLogfn(this.errorlog);
        }
      );
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }

  loadEnvironmentDetails() {
    try {
      this.environmentDetail = [];
      this.apiService.getEnvironmentDetails(this.projectId).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.environmentDetail = res;
          this.setEnvironment(this.environmentDetail[0]._id);
        }

      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
  }

  setEnvironment(envId: string) {
    this.ResetForm();
    let env = this.environmentDetail.find(x => x._id == envId);
    // this.exportProjectForm.controls['environmentName'].setValue(env.name);
    // this.exportProjectForm.controls['environmentDescription'].setValue(env.description);
    this.exportProjectForm.controls['environmentId'].setValue(env._id);
    this.exportProjectForm.controls['environmentStatus'].setValue(env.status);
    if (env.tenantid != null && env.tenantid != "") {
      this.exportProjectForm.controls['tenantid'].setValue(env.tenantid);
      this.Loadsubsbytenant(env.tenantid, env.subscriptionid, env.serviceprincipalid, env.infrastateresourcegroupid, env.storageaccountid, env.storagecontainerid);
    }
  }
}
