import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { Project } from '../../model/project.model'
import { ActivatedRoute, Router } from '@angular/router';
import { InfraTemplates } from '../../create-template-project';
import { ToastrService } from 'ngx-toastr';
import { DatePipe } from '@angular/common';
import { Log } from '../../model/project.model';
import { MatDialog } from '@angular/material/dialog';
import { ProjectExportDialogComponent } from './dialogs/projectexport.component';

@Component({
  selector: 'app-project',
  templateUrl: './project.component.html',
  styleUrls: ['./project.component.css']
})
export class CreateProjectComponent implements OnInit {
  pipe = new DatePipe('en-US');
  projectdetailsDisplayColumns: string[] = ['Id', 'position', 'Name', 'Description', 'Status', 'CreatedOn', 'UpdatedOn', 'View', 'Edit', 'Delete'];
  projectdetailsDataSource: any;
  projectDetails: any;
  projectForm: any;
  projDetails: Project;
  projectId: string;
  displaysubmit: boolean = false;
  loadSpinner: boolean;
  issubmitted: boolean = false;
  errorlog: Log = new Log();
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private formbulider: FormBuilder, private _router: Router
    , private infraTemplate: InfraTemplates, private toastr: ToastrService, private route: ActivatedRoute,public dialog: MatDialog) { }

  ngOnInit(): void {
    try {
      this.errorlog.application = "Azap"; this.errorlog.type = "error";
      this.errorlog.modulename = "Infrastructure Project - Page Load";

      if (this.webapiService._createProjectfromTemplate.initiateRequest == false) {
        // Recent Projects
        /*Edit Project from Recent Projects*/
        if (this.webapiService._commonprojectdata.externalpid != undefined && this.webapiService._commonprojectdata.externalpid != "") {
          this.webapiService._commonprojectdata.pid = this.webapiService._commonprojectdata.externalpid;
          this.webapiService._commonprojectdata.externalpid = "";

          this.getProjectbyId(this.webapiService._commonprojectdata.pid);
          this.getProjectDetails();

        }
        else {
          this.webapiService._commonprojectdata.pid = "";
        }
        /*Edit Project from Recent Projects*/
      }
      /*Create project from template */
      if (this.webapiService._createProjectfromTemplate.initiateRequest &&
        this.webapiService._createProjectfromTemplate.templateId != undefined) {

        this.loadSpinner = true;

        this.webapiService._createProjectfromTemplate.initiateRequest = false;
        this.infraTemplate.GetInfraTemplate(this.webapiService._createProjectfromTemplate.templateId).
          subscribe(res => {
            this.LoadProjectDetails();
          });

      }


      /*Clear Id values*/
      this.webapiService._commonprojectdata.pid = "";
      this.webapiService._commonprojectdata.eid = "";
      this.webapiService._commonprojectdata.resid = "";
      this.webapiService._commonprojectdata.rgid = "";
      /*Clear Id values*/

      this.ResetForm();

      this.getProjectDetails();
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  ResetForm() {
    this.projectId = '';
    this.projectForm = this.formbulider.group({
      projId: [''],
      projectname: ['', Validators.required],
      projectdescription: [''],
    });
  }
  applyFilter(filterValue: string) {
    filterValue = filterValue.trim(); // Remove whitespace
    filterValue = filterValue.toLowerCase();
    //this.projectdetailsDataSource.filter = filterValue;

    this.projectdetailsDataSource = this.projectDetails;
    this.projectdetailsDataSource = this.projectdetailsDataSource.filter(x => x.name.toLowerCase().includes(filterValue));

    if (filterValue.trim() === '') {
      this.projectdetailsDataSource = this.projectDetails;
    }
  }

  getProjectDetails() {
    this.projectdetailsDataSource = [];
    this.webapiService.getProjects().subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        this.projectdetailsDataSource = res;
        this.projectDetails = res;
        //this.loadSpinner = false;
      }
    });
    this.ResetForm();
  }

  getProjectbyId(projectId: string) {
    this.webapiService.getProjectbyId(projectId).subscribe(res => {
      this.projectForm.controls['projectname'].setValue(res.name);
      this.projectForm.controls['projectdescription'].setValue(res.description);
      this.projectForm.controls['projId'].setValue(res._id);
      this.projectId = res._id;
    });
  }

  saveProject(isSubmit: boolean) {
    try {
      this.errorlog.modulename = "Infrastructure Project - saveProject";

      if (!this.isProcessing) {
        this.isProcessing = true;

        const _proj = this.projectForm.value;
        this.projDetails = new Project();
        this.projDetails.name = _proj.projectname;
        this.projDetails.description = _proj.projectdescription;
        this.projDetails._id = _proj.projId;
        this.projDetails.createdby=localStorage.getItem("azapUserEmail");
        this.projDetails.updatedby=localStorage.getItem("azapUserEmail");
        if ((_proj.projId == '' || _proj.projId == undefined) && !isSubmit) {
          this.projDetails.status = "draft";
          this.webapiService.saveProject(this.projDetails).subscribe(res => {
            this.getProjectDetails();
            if (this.isProcessing) {
              this.isProcessing = false;
            }
            this.toastr.success("Project details saved successfully!", "Create Project");
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
        }
        else if (_proj.projId != '' && isSubmit) {
          this.projDetails.status = "submitted";
          this.webapiService.updateProject(this.projDetails).subscribe(() => {
            this.getProjectDetails();
            if (this.isProcessing) {
              this.isProcessing = false;
            }
            this.toastr.success("Project details submitted successfully!", "Create Project");
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
        }
        else if (_proj.projId != '' && !isSubmit) {
          this.projDetails.status = "draft";
          this.webapiService.updateProject(this.projDetails).subscribe(() => {
            this.getProjectDetails();
            if (this.isProcessing) {
              this.isProcessing = false;
            }
            this.toastr.success("Project details updated successfully!", "Update Project");
          }, (error) => {
            this.errorlog.remark = error;
            this.webapiService.ErrorLogfn(this.errorlog);
          });
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
    if (this.isProcessing) {
      this.isProcessing = false;
    }
  }

  onFormSubmit(data) {
    this.saveProject(true);
    this.projectForm.reset();
    this._router.navigateByUrl('/');
  }

  editProjectContent(projectId: string) {
    this.webapiService._commonprojectdata.pid = projectId;
    this._router.navigateByUrl('/infrastructure/environment');
  }

  editProject(projectId: string, projectName: string, projectDescription: string, status: string) {
    try {
      this.errorlog.modulename = "Infrastructure Project - editProject";
      this.projectForm.controls['projId'].setValue(projectId);
      this.projectForm.controls['projectname'].setValue(projectName);
      this.projectForm.controls['projectdescription'].setValue(projectDescription);
      this.projectId = projectId;
      if (status.toLowerCase() == "submitted")
        this.issubmitted = true;
      else
        this.issubmitted = false;
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  deleteProject(projectId: string) {
    this.webapiService.deleteProject(projectId).subscribe(() => {
      this.getProjectDetails();
    }, (error) => {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    });
  }

  LoadProjectDetails() {
    setTimeout(() => {
      this.getProjectDetails();
      this.loadSpinner = false;
    }, 3000);
  }

  ExportProject(projectId: string){
    try {
      this.webapiService.secjsondataasstring == "";
      this.webapiService.jsondataasstring == "";
      if (projectId != "" && projectId != null) {
        const dialogRef = this.dialog.open(ProjectExportDialogComponent, {
          width: "1500px",
          data: {
             projectId:projectId
           //  resid: 1,
           //  envid: 1,
           //  rgid: 1,
           //  uniqId: 1,
          },
        });
        dialogRef.afterClosed().subscribe((result) => {
          this.LoadProjectDetails();
        });
      }
     } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.webapiService.ErrorLogfn(this.errorlog);
       }
     }

}
