import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { from } from 'rxjs';
import { concatMap } from 'rxjs/operators';
import { Log } from 'src/app/model/project.model';
import { CreateExportedResource, ResourceGroup } from 'src/app/model/resourcegroup.model';
import { WebapiService } from 'src/app/service/webapi.service';
 
@Component({
  selector: 'app-resourcegroupexport',
  templateUrl: './resourcegroupexport.component.html',
  styleUrls: ['./resourcegroupexport.component.css']
})
export class ResourcegroupExportComponent implements OnInit, OnChanges {
  @Input() resourceGroup:ResourceGroup;
  @Input() selectedRGId:string;

  @Output() loadSpinner = new EventEmitter()
  @Output() RefreshRG = new EventEmitter<Boolean>()
  errorlog: Log = new Log();
  projectsDataSource: any;
  environmentDetails:any;

  selectedEnvId:string;
  selectedProjectId:string;
  isEnableClone:boolean=false;
  showCheckbox:boolean=false;
  lastIndex:number;
  isDeepClone:boolean=false;
  selectedRgDetails:ResourceGroup;

  constructor(private apiService: WebapiService,private toastr: ToastrService) { }

  ngOnChanges(): void {
    this.selectedRgDetails = {...this.resourceGroup}
    $('#ddlProject').val("");
    this.environmentDetails=[];
    this.isEnableClone=false;
    this.showCheckbox=false;
    this.loadRecentProjects();
    
  }

  ngOnInit(): void {
  }

  GetDateAsString() {
    let date = new Date();
    // format: dd/mm/yyyy_hh:mm:ss
    var dateformat =
      ("00" + (date.getMonth() + 1)).slice(-2) +
      ("00" + date.getDate()).slice(-2) +
      ("00" + date.getFullYear()).slice(-2) +
      "_" +
      ("00" + date.getHours()).slice(-2) +
      ("00" + date.getMinutes()).slice(-2) +
      ("00" + date.getSeconds()).slice(-2);
    return dateformat;
  }
  getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
  }

  loadRecentProjects() {
    try {
      this.errorlog.modulename = "Projects - loadProjects to clone a Resource group export";
      this.projectsDataSource = [];
      this.apiService.getProjects().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.projectsDataSource = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }

  loadEnvironmentDetails(projectId:string) {
    this.selectedProjectId= projectId;
    try {
      this.errorlog.modulename = "ResourceGroup Environment - loadEnvironmentDetails";
      this.apiService.getEnvironmentDetails(projectId).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.environmentDetails = res;
          this.selectedEnvId=res[0]._id;
          this.changeEnvironment(this.selectedEnvId);
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  changeEnvironment(environmentId:string){

    this.selectedEnvId=environmentId;
    this.isEnableClone=false;
    try {
      this.errorlog.modulename = "ResourceGroup - checkResourceGroupTerraformUniqueName";    

      this.apiService.checkResourceGroupTerraformUniqueName(this.selectedRGId,this.selectedEnvId)
      .subscribe(res=>{
        this.lastIndex= res.lastrgindex;
        this.isDeepClone=res.rguniquenamevalidate;
        if(res.rguniquenamevalidate==true){
          this.isEnableClone=true;
          this.showCheckbox=false;
        }else{
          this.showCheckbox=true;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });     
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
    
  }
  checkboxstatus(values:any){
    if(values.currentTarget.checked) 
    {
      this.isEnableClone=true;
    }
    else{ 
      this.isEnableClone=false;
    }
  }
  clone(){
    this.loadSpinner.emit();
    this.selectedRgDetails.projectid=this.selectedProjectId;
    this.selectedRgDetails.environmentid= this.selectedEnvId;
    this.selectedRgDetails.createdby = localStorage.getItem("azapUserEmail");
    this.selectedRgDetails.updatedby = localStorage.getItem("azapUserEmail");
    this.selectedRgDetails._id=null;
    this.selectedRgDetails.name = 'update_resource_group_name'+ this.GetDateAsString()+'_'+this.getRandomInt(0,400)
    let rgjson = JSON.stringify(this.selectedRgDetails);
    this.apiService.exportResourceGroup(rgjson, !this.isDeepClone).subscribe((res) => {
      this.CreateResource_new(res._id,res.name)
    });

    // this.apiService.createResourceGroup(rgjson, this.lastIndex+1).subscribe(res => {   
    //    this.CreateResource_new(res._id)
    // });

  }

 
  CreateResource_new(rgId,rgGroupName){
    try{
      this.errorlog.modulename = "ResourceGroup - Create new Resource";  
      this.apiService.getExportedResourcesByRgId(this.selectedRGId).subscribe(response => {
        var dataSource = [];
        if (typeof response != 'undefined' && response.length > 0) {
          dataSource = response;
          var dynamicArray = [];
          from(dataSource).pipe(
            concatMap((res)=>{
              let currentResource = new CreateExportedResource();
              currentResource.mstresourceid = res.mstresourceid;
              if(this.isDeepClone){
                currentResource.properties = res.properties;
               }
              else{
              if(res.preexisting)
              { 
                currentResource.properties={"resource_group_name":rgGroupName , "name": ""}
              }
              else {
                currentResource.properties=[];
               }
              }
              currentResource.resourcegrpid = rgId;
              currentResource.createdby = localStorage.getItem("azapUserEmail");
              currentResource.updatedby = localStorage.getItem("azapUserEmail");
              currentResource.preexisting=res.preexisting;
              currentResource.reference = [];
              currentResource.isdeleted=res.isdeleted;
              currentResource.terraformuniquename=res.terraformuniquename;
              return this.apiService.saveExportResourcesJson(JSON.stringify(currentResource),!this.isDeepClone);
            })
          ).subscribe(res => {
           
           });

         /* for (let res = 0; res < dataSource.length; res++) {

            let currentResource = new CreateExportedResource();
            currentResource.mstresourceid = dataSource[res].mstresourceid;
            currentResource.properties = dataSource[res].properties;
            currentResource.resourcegrpid = rgId;
            currentResource.createdby = localStorage.getItem("azapUserEmail");
            currentResource.updatedby = localStorage.getItem("azapUserEmail");
            currentResource.preexisting=dataSource[res].preexisting;
            currentResource.reference = [];
            currentResource.isdeleted=dataSource[res].isdeleted;
            currentResource.terraformuniquename=dataSource[res].terraformuniquename;

            this.apiService.saveExportResourcesJson(JSON.stringify(currentResource),this.isDeepClone).subscribe(res => {
                currentResource._id = res._id;
            });
        }*/

        }
        this.toastr.success("Resource Group details cloned successfully!", "Clone Resource Group");        
       this.RefreshRG.emit(true);
        $('#Modal').modal('hide');
      });
    }catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
      this.RefreshRG.emit(false);
    }
  }

}
