import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../../service/webapi.service';
import { FormBuilder, Validators } from '@angular/forms';
import { EnvironmentDetails } from '../../model/project.model';
import { CreateResource, ResourceGroup,CreateExportedResource } from '../../model/resourcegroup.model';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { Log } from '../../model/project.model';
import { concatMap } from 'rxjs/operators';
import { from } from 'rxjs';

@Component({
  selector: 'app-resourcegroup',
  templateUrl: './resourcegroup.component.html',
  styleUrls: ['./resourcegroup.component.css']
})
export class ResourcegroupComponent implements OnInit {

  resourcegroupDetails: ResourceGroup[] = [];
  resourceGroupForm: any;
  // store querystring values
  projectId: string;
  environmentId: string;
  terraformuniquename: string = "";
  rgDetails: ResourceGroup;
  environmentName: string;
  environmentDescription: string;
  envDetail: EnvironmentDetails;
  ispointerchecked: string = "yes";
  locktype = ["NoLock", "CanNotDelete", "ReadOnly"];
  isProcessing: boolean = false;
  errorlog: Log = new Log();
  isEnableClone:boolean=false;
  projectsDataSource: any;
  environmentDetails:any;
  selectedRGId:string;
  selectedEnvId:string;
  selectedProjectId:string;
  selectedRgDetails:ResourceGroup;
  showCheckbox:boolean=false;
  lastIndex:number;
  isChecked:boolean=false;
  isDeepClone:boolean=false;
  loadSpinner: boolean;

  constructor(private apiService: WebapiService, private formbulider: FormBuilder,
    private route: ActivatedRoute, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap"; this.errorlog.type = "error";
    this.errorlog.modulename = "Infrastructure Resourcegroup - ngOnInit";
    try {

      this.resetform();
      this.environmentId = this.apiService._commonprojectdata.eid;
      this.projectId = this.apiService._commonprojectdata.pid;
      if (this.environmentId != undefined && this.environmentId != "") {
        this.ispointerchecked = "no";
        this.loadResourceGroups();
      }
      else {
        this._router.navigateByUrl('/infrastructure/project');
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  resetform() {
    this.resourceGroupForm = this.formbulider.group({
      environmentId: [''],
      projectId: [''],
      subscription: [''],
      rgname: ['', Validators.required],
      region: ['', Validators.required],
      tag: ['', Validators.required],
      rgId: [''],
      terraformuniquename: [''],
      locktype: ['NoLock'],
      prexisting:[false]
    });
  }
  GetDateAsString() {
    let date = new Date();
    // format: dd/mm/yyyy_hh:mm:ss
    var dateformat =
      ("00" + (date.getMonth() + 1)).slice(-2) +
      ("00" + date.getDate()).slice(-2) +
      ("00" + date.getFullYear()).slice(-2) +
      "_" +
      ("00" + date.getHours()).slice(-2) +
      ("00" + date.getMinutes()).slice(-2) +
      ("00" + date.getSeconds()).slice(-2);
    return dateformat;
  }
  getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
  }
  deleteResourceGroup(rgId: string) {
    try {
      this.errorlog.modulename = "Infrastructure Resourcegroup - deleteResourceGroup";
      this.apiService.deleteResourceGroup(rgId).subscribe(() => {
        this.loadResourceGroups();
      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  saveResourceGroup() {
    try {
      this.errorlog.modulename = "Infrastructure Resourcegroup - saveResourceGroup";
      if (!this.isProcessing) {
        this.isProcessing = true;
        this.rgDetails = new ResourceGroup();
        const _rg = this.resourceGroupForm.value;

        this.rgDetails.environmentid = this.environmentId;
        this.rgDetails.projectid = this.projectId;
        this.rgDetails.name = _rg.rgname;
        this.rgDetails.subscription = _rg.subscription;
        this.rgDetails.location = _rg.region;
        this.rgDetails.tags = _rg.tag;
        this.rgDetails.locktype = _rg.locktype;
        this.rgDetails.preexisting=_rg.prexisting
        this.rgDetails.createdby = localStorage.getItem("azapUserEmail");
        this.rgDetails.updatedby = localStorage.getItem("azapUserEmail");

        if (this.rgDetails.name != "" && this.rgDetails.name != undefined && this.rgDetails.location != ""
          && this.rgDetails.location != undefined) {
          if (_rg.rgId != undefined && _rg.rgId != '') {
            this.rgDetails._id = _rg.rgId;
            this.apiService.updateResourceGroup(this.rgDetails).subscribe(() => {
              this.resetform();
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.loadResourceGroups();
              this.toastr.success("Resource Group details updated successfully!", "Update Resource Group")
            }, (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
          else {
            let saveRG: string;
            saveRG = JSON.stringify(this.rgDetails);
            this.apiService.createResourceGroup(saveRG).subscribe(() => {
              this.resetform();
              if (this.isProcessing) {
                this.isProcessing = false;
              }
              this.loadResourceGroups();
              this.toastr.success("Resource Group details saved successfully!", "Create Resource Group")
            }, (error) => {
              this.errorlog.remark = error;
              this.apiService.ErrorLogfn(this.errorlog);
              if (this.isProcessing) {
                this.isProcessing = false;
              }
            });
          }
          this.resourceGroupForm.reset();
          this.resetform();
        }
      }
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  loadResourceGroups() {
    try {
      this.errorlog.modulename = "Infrastructure Resourcegroup - loadResourceGroups";
      this.resourcegroupDetails = [];

      this.apiService.getResourceGroupsbyEnvId(this.environmentId).subscribe(res => {
        if (res != undefined && res.length > 0) {
          this.resourcegroupDetails = JSON.parse(JSON.stringify(res));
        }
      }, (error) => {
        this.errorlog.modulename = "Infrastructure Resourcegroup - loadResourceGroups";
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }

  editResourceGroup(rgId: string, subscription: string, rgName: string, region: string, tag: string, terra: string, locktype: string,prexisting:any) {
    try {
      this.errorlog.modulename = "Infrastructure Resourcegroup - editResourceGroup";
      
      this.resourceGroupForm.controls['rgId'].setValue(rgId);
      this.resourceGroupForm.controls['subscription'].setValue(subscription);
      this.resourceGroupForm.controls['rgname'].setValue(rgName);
      this.resourceGroupForm.controls['region'].setValue(region);
      this.resourceGroupForm.controls['tag'].setValue(tag);
      this.resourceGroupForm.controls['terraformuniquename'].setValue(terra);
      this.resourceGroupForm.controls['locktype'].setValue(locktype);
      this.resourceGroupForm.controls['prexisting'].setValue(prexisting);

      this.terraformuniquename = terra;
    }
    catch (error) {
      this.errorlog.remark = error;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  editres(rgid) {
    this.apiService._commonprojectdata.rgid = rgid;
    this._router.navigateByUrl('/infrastructure/resources');
  }

  cloneResourceGroup(rgid:string){
    this.selectedRGId=rgid;
    this.selectedRgDetails= {...this.resourcegroupDetails.find(x=>x._id==rgid)};
    this.environmentDetails=[];

    $('#Modal').modal('show');
  }

  showSpinner(){
    this.loadSpinner=true;
  }

  refreshRGList(status){
    this.loadSpinner=false;
    if(status) this.loadResourceGroups();
  }

  
 
}




