import { Component, Inject, ViewChild } from '@angular/core';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { WebapiService } from 'src/app/service/webapi.service';
import { MstDrpdwnmapping, ResourcesJson } from 'src/app/model/resourcegroup.model';
import { ITreeOptions, TreeComponent, TreeNode } from '@circlon/angular-tree-component';
import { ToastrService } from 'ngx-toastr';
import { Log } from 'src/app/model/project.model';
import { debounceTime, finalize, map, startWith, switchMap, tap } from 'rxjs/operators';
import { FormControl } from '@angular/forms';
import { environment } from 'src/environments/environment';
@Component({
  selector: 'app-resourceschema',
  templateUrl: 'resourceschema.component.html',
  styleUrls: ['./resourceschema.component.css'],
})
export class ResourcesDialogComponent {
  @ViewChild('tree') tree: TreeComponent;
  resourceid: string; regid: string; uniqId: string = ""; envid: string;
  DataObj: any; IsDisabled: boolean = false; isDisplayError: boolean = false;
  nodesdata: any; drpdnres: any; count: any = 0;
  jsondataasstring: string;
  resoutput: ResourcesJson = new ResourcesJson();
  isProcessing: boolean = false;
  errorlog: Log = new Log();
  isloading = false; resourcesInput: FormControl;
  resourcevalidationmsg: string = "";
  constructor(public secdialog: MatDialog,
    public dialogRef: MatDialogRef<ResourcesDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private http: HttpClient, public apiService: WebapiService, private toastr: ToastrService) 
    { this.resourceid = data.resid; this.envid = data.envid, this.regid = data.rgid, 
      this.uniqId = data.uniqId }
  ngOnInit(): void {
    this.resourcevalidationmsg = environment.resourcevalidationmsg;
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "ResourcesDialog - ngOnInit";
    this.errorlog.type = "Error";
    this.apiService.getresourceipjson(this.resourceid).subscribe(res => {
      this.nodesdata = res;
    });
    this.resetdrp();
  }
  resetdrp() {
    this.resourcesInput = new FormControl('');
    this.resourcesInput
      .valueChanges
      .pipe(
        debounceTime(500),
        tap(() => this.isloading = true),
        switchMap(value => this.apiService.getresourcedrpdwnip({ envid: this.envid, name: value }, 1)
          .pipe(
            finalize(() => this.isloading = false),
          )
        )
      ).subscribe(users => {
        if (users != null && users.length > 0)
          this.drpdnres = users;
      }
      )
  }

  options: ITreeOptions = {
    displayField: 'name',
    useVirtualScroll: false,
    nodeHeight: 25,
    allowDrag: false,
    allowDrop: false
  };
  selectedNode;

  onTreeLoad(tree): void {
    try {
      if (this.uniqId != "") {
        this.apiService.getResourcespropDataJson(this.regid, this.resourceid, this.uniqId).subscribe(res => {
          let inputArray = [];
          inputArray.push(JSON.parse(res.datajson));
          this.DataObj = inputArray[0];
          if (this.DataObj != undefined && this.uniqId != "") {
            tree.treeModel.nodes.forEach(ele => {
              if (ele.children != undefined && ele.children.length > 0) {
                this.Autopopulatechild(ele, "", tree, 0);
              }
              else {
                if (this.DataObj[ele.propertyname] != undefined) {
                  if (ele.type.toLowerCase() == "map") {
                    let maparr = this.DataObj[ele.propertyname].replace(/{/g, "").replace(/}/g, "").replace(/\"/g, "").split(':');
                    $("." + ele.propertyname + "_" + ele.id + "_key").val(maparr[0]);
                    $("." + ele.propertyname + "_" + ele.id + "_val").val(maparr[1]);
                  }
                  ele.value = this.DataObj[ele.propertyname];
                }
              }
            });
            tree.treeModel.update();
          }
        });
      }
    }
    catch (error) {
      this.errorlog.modulename = "ResourcesDialog - onTreeLoad";
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }

  Autopopulatechild(ele: any, childinput: any, tree, j: any) {
    try {
      if (ele.children != undefined && ele.children.length > 0) {
        let inputarr = []; let subchild = {};
        let arr = ele.children; let len = 0;
        if (childinput != "") {
          if (childinput[ele.propertyname] != undefined) {
            subchild[ele.propertyname] = childinput[ele.propertyname];
            inputarr.push(subchild);
            len = inputarr[0][ele.propertyname].length;
          }
        } else {
          if (this.DataObj[ele.propertyname] != undefined) {
            subchild[ele.propertyname] = JSON.parse(this.DataObj[ele.propertyname]);
            inputarr.push(subchild);
            len = inputarr[0][ele.propertyname].length;
          }
        }
        if (len > 1) {
          for (var i = 0; i < len - 1; i++) {
            let tempnode = tree.treeModel.getNodeById(ele.id);
            this.copyNode(tempnode, tree);
          }
        }
        const unique = [...new Set(ele.children.map(item => item.propertyname))];
        for (var i = 0; i < len; i++) {
          var arr1 = [];
          if (len > 1) {
            arr1 = ele.children.slice(j, unique.length + j);
            childinput = "";
          }
          else { arr1 = ele.children; }
          arr1.forEach((x, index) => {
            if (x.children != undefined && x.children.length > 0) {
              if (childinput != "") {
                if (childinput[ele.propertyname] != undefined)
                  childinput = childinput[ele.propertyname][i];
              } else {
                childinput = inputarr[0][ele.propertyname][i];
              }
              this.Autopopulatechild(x, childinput, tree, 0);
            }
            else {
              if (childinput != "") {
                if (inputarr[0][x.propertyname] == undefined) {
                  x.value = inputarr[0][ele.propertyname][i][x.propertyname];
                }
                else {
                  x.value = inputarr[0][x.propertyname];
                }
              }
              else {
                x.value = inputarr[0][ele.propertyname][i][x.propertyname];
              }
            }
          });
          j = j + unique.length;
        }
      }
    }
    catch (error) {
      this.errorlog.modulename = "ResourcesDialog - Autopopulatechild";
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  generateKeys(obj) {
    return obj ? Object.keys(obj) : null
  }
  onNoClick(): void {
    this.dialogRef.close();
  }
  BindRefernceval(drpdwnval: string, node: any) {
    if(node.data.validationpattern!=""){
    var pattern = new RegExp(node.data.validationpattern);
    if (pattern.test(drpdwnval.trim())) {
      $("." + node.data.propertyname + "_" + node.data.id).removeClass("errorfocus");
      $(".span_" + node.data.propertyname + "_" + node.data.id).css("display", "none");
      this.IsDisabled = false;
    }
    else {
      $("." + node.data.propertyname + "_" + node.data.id).addClass("errorfocus");
      $(".span_" + node.data.propertyname + "_" + node.data.id).css("display", "block");
      this.IsDisabled = true;
    }
  }
    node.data.value = drpdwnval.trim();
    this.tree.treeModel.update();
    this.resourcesInput.setValue("");
  }
  BindReferncevalsearch(drpdwnval: string, node: any) {
    node.data.value = drpdwnval.trim();
    this.tree.treeModel.update();
    this.resourcesInput.setValue("");
  }
  BindMapval(drpdwnval: string, node: any, type: string) {
    $("." + node.propertyname + "_" + node.id + "_" + type).val(drpdwnval);
    node.data.value = drpdwnval;
    this.tree.treeModel.update();
  }
  DrpdnRefernceval(drpdwnval: string, node: any) {
    // const parentNode = node.realParent ? node.realParent : node.treeModel.virtualRoot;
    //  $("." + node.data.propertyname + "_" + node.data.id).val(drpdwnval);
    node.data.value = drpdwnval;
    //  this.tree.treeModel.update();
  }
  BindlistRefernceval(drpdwnval: string, node: any) {
    let selectedarr = [];
    if (node.data.value != undefined && node.data.value != null && node.data.value.toString() != "") {
      var customval = node.data.value.split(',').map(function (str) { return '"' + str + '"'; }).join(',');
      selectedarr = JSON.parse("[" + customval + "]");
    }
    if (!selectedarr.includes(drpdwnval))
      selectedarr.push(drpdwnval);
    node.data.value = JSON.stringify(selectedarr).slice(1, -1).replace(/\"/g, "");
    this.tree.treeModel.update();
    this.resourcesInput.setValue("");
  }
  checkboxstatus(drpdwnvalstr: string, node: any) {
    let drpdwnval: boolean
    if (drpdwnvalstr == 'true')
      drpdwnval = false;
    else
      drpdwnval = true;
    node.data.value = drpdwnval;
    this.tree.treeModel.update();
  }
  copyNode(node: any, tree) {
    const parentNode = node.realParent ? node.realParent : node.treeModel.virtualRoot;
    let customarray = [];
    node.data.children.forEach(x => {
      let newNode = this.duplicatenode(x);
      customarray.push(newNode);
    });
    customarray = this.uniqueByKey(customarray, 'propertyname');
    const pcopyNode = JSON.stringify(node.data);
    const pnewNode = JSON.parse(pcopyNode);
    this.deleteIds(pnewNode);
    parentNode.data.children.forEach((x, index) => {
      if (x.propertyname == pnewNode.propertyname && index == node.index) {
        customarray.forEach(y => {
          parentNode.data.children[index].children.push(y);
        });
      }
    });
    tree.treeModel.update();
  }
  duplicatenode(x: any) {
    let childarr = [];
    let childobj = {};
    if (x.children != undefined && x.children.length > 0) {
      x.children.forEach(y => {
        if (y.children != undefined && y.children.length > 0) {
          let n = this.duplicatenode(y);
          childarr.push(n);
        }
        else {
          const childnode = (y.type == "bool") ? { name: y.name, type: y.type, propertyname: y.propertyname, value: false, required: y.required, defaultvalue: y.defaultvalue } : { name: y.name, type: y.type, propertyname: y.propertyname, required: y.required, defaultvalue: y.defaultvalue };
          childarr.push(childnode);
        }
      });
      childarr = this.uniqueByKey(childarr, 'propertyname');
      childobj = childarr[0];
    }
    let newNode;
    if (x.type == "list(map)" || x.type == "list(object)") { newNode = { name: x.name, type: x.type, propertyname: x.propertyname, children: childarr, required: x.required, defaultvalue: x.defaultvalue }; }
    else if (x.type == "bool")
      newNode = { name: x.name, type: x.type, propertyname: x.propertyname, value: false, required: x.required, defaultvalue: x.defaultvalue };
    else
      newNode = { name: x.name, type: x.type, propertyname: x.propertyname, required: x.required, defaultvalue: x.defaultvalue };
    return newNode
  }
  uniqueByKey(array, key) {
    return [...new Map(array.map((x) => [x[key], x])).values()];
  }
  deleteIds(node: TreeNode) {
    node.id = null;
    if (node.children) {
      node.children.forEach(child => this.deleteIds(child));
    }
  }
  save(tree: any) {
    try {
      if (!this.isProcessing) {
        this.isProcessing = true;

        this.count = 0;

        var arrstr = {};
        tree.nodes.forEach(ele => {
          if (ele.children != undefined && ele.children.length > 0) {
            if (ele.required.toLowerCase() == "yes") {
              const someNode = this.tree.treeModel.getNodeById(ele.id);
              someNode.expand();
            }
            let csattr = {}; let arr = []; let arr1 = []; let i = 0;
            csattr['propertyname'] = ele.propertyname;
            const unique = [...new Set(ele.children.map(item => item.propertyname))];
            ele.children.forEach((z, index) => {
              if (i < unique.length) {
                arr.push(z);
                i = i + 1;
                if (i == unique.length) {
                  let getchild = {};
                  csattr['children'] = arr;
                  getchild = this.saveChildren(csattr, z.required.toLowerCase());
                  if (arr1.length > 0) { arr1[0].push(getchild[0]); }
                  else {
                    if (Object.keys(getchild[0]).length !== 0)
                      arr1.push(getchild);
                  }
                  i = 0; arr = [];
                }
              }
            });
            if (arr1[0] != undefined && JSON.stringify(arr1[0]).slice(1, -1) != "{}")
              arrstr[ele.propertyname] = JSON.stringify(arr1[0]);
          }
          else {
            if (ele.required.toLowerCase() == "yes") {
              if ($("." + ele.propertyname + "_" + ele.id).val() == "") { $("." + ele.propertyname + "_" + ele.id).addClass("errorfocus"); this.count = this.count + 1; }
              else { $("." + ele.propertyname + "_" + ele.id).removeClass("errorfocus"); }
            }
            if (ele.value != undefined && ele.value.toString() != "") {
              if (ele.type.toLowerCase() == "map")
                arrstr[ele.propertyname] = "{\"" + $("." + ele.propertyname + "_" + ele.id + "_key").val() + "\":\"" + $("." + ele.propertyname + "_" + ele.id + "_val").val() + "\"}";
              else
                arrstr[ele.propertyname] = ele.value;
            }
          }
        });
        if (this.count == 0) {
          this.isDisplayError = false;
          this.jsondataasstring = JSON.stringify(arrstr);
          this.callSaveApi();
          //alert(JSON.stringify(arrstr));
        } else {
          this.isDisplayError = true;
          this.isProcessing = false;
        }
      }
    }
    catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  saveChildren(ele: any, isparentreq: string) {
    try {
      let childarr = [];
      let childobj = {};
      childobj[ele.propertyname] = [];
      if (ele.children != undefined && ele.children.length > 0) {
        let subchild = {};
        ele.children.forEach(y => {
          if (y.children != undefined && y.children.length > 0) {
            if (y.required.toLowerCase() == "yes") {
              const someNode = this.tree.treeModel.getNodeById(y.id);
              someNode.expand();
            }
            let csattr = {};
            csattr['propertyname'] = y.propertyname;
            let arr = []; let arr1 = []; let i = 0;
            const unique = [...new Set(y.children.map(item => item.propertyname))];
            y.children.forEach((z, index) => {
              if (i < unique.length) {
                arr.push(z);
                i = i + 1;
                if (i == unique.length) {
                  let getchild = {};
                  csattr['children'] = arr;
                  getchild = this.saveChildren(csattr, z.required.toLowerCase());
                  if (arr1.length > 0) { arr1[0].push(getchild[0]); }
                  else {
                    if (Object.keys(getchild[0]).length !== 0)
                      arr1.push(getchild);
                  }
                  i = 0; arr = [];
                }
              }
            });
            if (arr1[0] != undefined && JSON.stringify(arr1[0]).slice(1, -1) != "{}")
              subchild[y.propertyname] = arr1[0];
          }
          else {
            if (y.required.toLowerCase() == "yes" && isparentreq == "no") {
              if ($("." + y.propertyname + "_" + y.id).val() == "") { $("." + y.propertyname + "_" + y.id).addClass("errorfocus"); this.count = this.count + 1; }
              else { $("." + y.propertyname + "_" + y.id).removeClass("errorfocus"); }
            }
            if (y.value != undefined && y.value.toString() != "") {
              if (y.type.toLowerCase() == "map")
                subchild[y.propertyname] = "{\"" + $("." + y.propertyname + "_" + y.id + "_key").val() + "\":\"" + $("." + y.propertyname + "_" + y.id + "_val").val() + "\"}";
              else
                subchild[y.propertyname] = y.value;
            }
          }
        });
        childobj[ele.propertyname].push(subchild);
      }
      childarr.push(childobj);
      return childarr[0][ele.propertyname];
    }
    catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  callSaveApi() {
    try {
      {
        let inputArray = [];
        inputArray.push(JSON.parse(this.jsondataasstring));
        this.resoutput.resourcegrpid = this.regid;
        this.resoutput.mstresourceid = this.resourceid;
        this.resoutput.properties = inputArray[0];
        this.resoutput.createdby = localStorage.getItem("azapUserEmail");
        this.resoutput.updatedby = localStorage.getItem("azapUserEmail");
        this.resoutput.reference = [];
        this.resoutput.preexisting = false;
        if (this.uniqId == '') {
          this.apiService.saveResourcesJson(JSON.stringify(this.resoutput)).subscribe(() => {
            this.onNoClick();
            this.showNotification("Resource properties saved successfully!!", "Configure Resource");
            if (this.isProcessing) {
              this.isProcessing = false;
            }
          }, (error) => {
            this.errorlog.remark = error;
            this.apiService.ErrorLogfn(this.errorlog);
          });
        }
        else {
          this.resoutput._id = this.uniqId;
          this.apiService.UpdateResourcesJson(JSON.stringify(this.resoutput)).subscribe(() => {
            this.onNoClick();
            this.showNotification("Resource properties saved successfully!!", "Configure Resource");
            if (this.isProcessing) {
              this.isProcessing = false;
            }
          }, (error) => {
            this.errorlog.remark = error;
            this.apiService.ErrorLogfn(this.errorlog);
          });
        }
      }
    }
    catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
      if (this.isProcessing) {
        this.isProcessing = false;
      }
    }
  }
  showNotification(message, title) {
    this.toastr.success(message, title)
  }
}