import { Component, OnInit, ViewChild } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { debounceTime, finalize, map, startWith, switchMap, tap } from 'rxjs/operators';
import { MatDialog } from '@angular/material/dialog';
import { ResourcesDialogComponent } from './dialogs/resourceschema.component';
import { Resources, ResourceGroup } from '../../model/resourcegroup.model';
import { WebapiService } from '../../service/webapi.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Log } from '../../model/project.model';
import { ResourcePreexistingDialogComponent } from './dialogs/existingresourceschema.component';
@Component({
  selector: 'app-resources',
  templateUrl: './resources.component.html',
  styleUrls: ['./resources.component.css']
})
export class ResourcesComponent implements OnInit {
  filteredresources: Resources[] = [];
  resourcesForm: FormGroup;
  isloading = false; rgid: string; envid: string;
  isdisable = true; _id: string = ""; dataSource: ResourceGroup[] = [];
  ispointerchecked: string = "yes";
  errorlog: Log = new Log();
  preexisting: boolean = false
  resourcegroupDetails: any
  constructor(public dialog: MatDialog, private fb: FormBuilder, private apiService: WebapiService, private route: ActivatedRoute, private _router: Router) { }
  openDialog(): void {
    if (this.resourcesForm.get('resourcesInput').value != '' && this.resourcesForm.get('resourcesInput').value != null) {

      if (this.preexisting == false) {
        const dialogRef = this.dialog.open(ResourcesDialogComponent, {
          width: '1250px',
          data: { resid: $(".res_id").val(), envid: this.envid, rgid: this.rgid, uniqId: '' }
        });
        dialogRef.afterClosed().subscribe(result => {
          this.loadgrid(this.rgid);
        });
      }
      else {
        this.openPreExistingDialog();
      }
      this.ResetForm();
    }
  }
  ResetForm() {
    $(".resbutton").prop('disabled', true);
    this.resourcesForm = this.fb.group({
      resourcesInput: null
    })
    this.resourcesForm
      .get('resourcesInput')
      .valueChanges
      .pipe(
        debounceTime(500),
        tap(() => this.isloading = true),
        switchMap(value => this.apiService.typeheadSearch({ name: value }, 1)
          .pipe(
            finalize(() => this.isloading = false),
          )
        )
      ).subscribe(users => {
        this.filteredresources = users;
        if (this.resourcesForm.get('resourcesInput').value == '') { $(".resbutton").prop('disabled', true); }
      }
      )
    if (this.rgid != undefined && this.rgid != "") {
      this.ispointerchecked = "no";
      this.loadgrid(this.rgid);
    }
    else {
      this._router.navigateByUrl('/infrastructure/project');
    }
  }
  ngOnInit(): void {
    this.errorlog.application = "Azap";
    this.errorlog.modulename = "Resource - ngOnInit";
    this.errorlog.type = "Error";
    this.rgid = this.apiService._commonprojectdata.rgid;
    this.envid = this.apiService._commonprojectdata.eid;

    this.ResetForm();

    if ((this.rgid != undefined && this.rgid != "") && (this.envid != undefined && this.envid != "")){
      this.apiService.getrgbyrgid(this.rgid).subscribe((res) => {
        if (res != undefined) {
          this.resourcegroupDetails = JSON.parse(JSON.stringify(res));
          this.preexisting = res.preexisting
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.apiService.ErrorLogfn(this.errorlog);
      })
      
    }
    else {
      this._router.navigateByUrl('/infrastructure/project');
    }
  }

  loadgrid(id: string) {
    try {
      this.apiService.getResources(id).subscribe(res => {
        this.dataSource = [];
        if (typeof res != 'undefined' && res.length > 0) {
          this.dataSource = res;
        }
      });
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  displayFn(res: Resources) {
    if (res) {
      this._id = $(".resoption").attr('data-message-id');
      $(".resbutton").prop('disabled', false);
      return res;
    }
  }
  deleteResources(resId: string) {
    this.apiService.deleteResources(this.rgid, resId).subscribe(() => {
      this.loadgrid(this.rgid);
    });
  }
  editResources(uniqId: string, resId: string, record: any) {
    try {
      this.apiService.secjsondataasstring == "";
      this.apiService.jsondataasstring == "";
      if (resId != '' && resId != null) {
        if (this.preexisting && record.preexisting) {
          const dialogRef = this.dialog.open(ResourcePreexistingDialogComponent, {
            width: '750px',
            height:'300px',
            data: { resid: $(".res_id").val(), rgid: this.rgid,
             resourcegroupname: this.resourcegroupDetails?.name ,
            properties:record.properties,uniqId: uniqId}
          });
          dialogRef.afterClosed().subscribe(result => {
            this.loadgrid(this.rgid);
          });
          this.ResetForm();
        }
        else {
          const dialogRef = this.dialog.open(ResourcesDialogComponent, {
            width: '1500px',
            data: { resid: resId, envid: this.envid, rgid: this.rgid, uniqId: uniqId }
          });
          dialogRef.afterClosed().subscribe(result => {
            this.loadgrid(this.rgid);
          });
        }
      }
    } catch (error) {
      this.errorlog.remark = error.message + "-" + error.stack;
      this.apiService.ErrorLogfn(this.errorlog);
    }
  }
  getoptionid(input: string) {
    $(".res_id").val(input);
  };

  openPreExistingDialog() {
    if ($(".res_id").val() == undefined || $(".res_id").val() == null)
      return;

    const dialogRef = this.dialog.open(ResourcePreexistingDialogComponent, {
      width: '750px',
      height:'300px',
      data: { resid: $(".res_id").val(), rgid: this.rgid, resourcegroupname: this.resourcegroupDetails?.name,
      properties:null }
    });
    dialogRef.afterClosed().subscribe(result => {
      this.loadgrid(this.rgid);
    });
    this.ResetForm();

  }
}
