import { Component, OnInit } from '@angular/core';
import { Environment, Project } from '../../model/project.model';
import { ResourceGroup } from '../../model/resourcegroup.model';
import { WebapiService } from '../../service/webapi.service';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
@Component({
  selector: 'app-review',
  templateUrl: './review.component.html',
  styleUrls: ['./review.component.css']
})
export class InfrareviewComponent implements OnInit {
  projectId: string = "";
  projDetails: Project = new Project();
  environmentDetails: Environment[] = [];
  resourcegroupDetails: ResourceGroup[] = [];
  resdataSource: ResourceGroup[] = [];
  isactive = false;
  isProcessing: boolean = false;

  constructor(private webapiService: WebapiService, private _router: Router, private toastr: ToastrService) { }

  ngOnInit(): void {
    this.projectId = this.webapiService._commonprojectdata.pid;

    if (this.webapiService._commonprojectdata.pid != undefined && this.webapiService._commonprojectdata.pid != '') {
      this.isactive = true;
      this.getProjectbyId(this.projectId);
      this.loadEnvironmentDetails(this.projectId);
      this.loadResourceGroups(this.projectId);
    }
  }
  getProjectbyId(projectId: string) {
    this.webapiService.getProjectbyId(projectId).subscribe(res => {
      this.projDetails = res;
    });
  }

  loadEnvironmentDetails(projectId: string) {
    this.environmentDetails = [];

    this.webapiService.getEnvironmentDetails(projectId).subscribe(res => {
      if (res != undefined && res.length > 0) {
        this.environmentDetails = res;
      }
    });
  }
  loadResourceGroups(projectId: string) {
    this.resourcegroupDetails = [];
    let rgids = [];
    this.webapiService.getResourceGroupsbyProjectId(projectId).subscribe(res => {
      if (res != undefined && res.length > 0) {
        this.resourcegroupDetails = res;
        res.forEach(ele => {
          rgids.push(ele._id);
        });
        rgids.forEach(rg => { this.loadresgrid(rg); });
      }
    });
  }
  loadresgrid(id: string) {
    this.webapiService.getResources(id).subscribe(res => {
      if (typeof res != 'undefined' && res.length > 0) {
        res.forEach(element => {
          this.resdataSource.push(element);
        });
      }
    });
  }
  saveProject() {
    if (!this.isProcessing) {
      this.isProcessing = true;
      if (this.projectId != "") {
        this.projDetails.status = "submitted";
        this.projDetails.updatedby=localStorage.getItem("azapUserEmail");
        let notificationmessage: string;
        notificationmessage = "\"" + this.projDetails.name + "\" submitted successfully! It usually takes 10-15 minutes to generate package. Once ready, it will be available in 'Deployment (Infra)' screen!";
        this.webapiService.updateProject(this.projDetails).subscribe(() => {
          //this.showNotification("\"" + this.projDetails.name + "\" submitted successfully!!", "InfraStructure Project:")

          this.environmentDetails.forEach((currentEnvironment, index) => {
            if(currentEnvironment.status.toLowerCase() === "draft" ||
            currentEnvironment.status.toLowerCase() === "deployed")
            {
              this.webapiService.setEnvironmentStatus(currentEnvironment._id, "submitted")
              .subscribe(() => { });
            }
          });
          
          this.showNotification(notificationmessage, "InfraStructure Project:");

          if (this.isProcessing) {
            this.isProcessing = false;
          }
          this._router.navigateByUrl('/infrastructure/project?from=infra');
        });
      }
    }
  }
  showNotification(message, title) {
    this.toastr.success(message, title)
  }
}
