import { Component, OnInit } from '@angular/core';
import { WebapiService } from '../service/webapi.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Log } from '../model/project.model';

@Component({
  selector: 'app-recentprojects',
  templateUrl: './recentprojects.component.html',
  styleUrls: ['./recentprojects.component.css']
})
export class RecentProjectsComponent implements OnInit {

  recentprojectsDisplayColumns: string[] = ['Id', 'position', 'Name', 'Status', 'View', 'Edit', 'Delete'];
  recentprojectsDataSource: any;
  errorlog: Log = new Log();

  constructor(private webapiService: WebapiService, private _router: Router) { }

  ngOnInit(): void {
    this.errorlog.application = "Azap"; this.errorlog.type = "error";
    this.loadRecentProjects();
  }

  ToggleEdit(val: string) {
    return !(val.toLowerCase() == "submitted" || val == "approved" || val == "rejected");
  }

  loadRecentProjects() {
    try {
      this.errorlog.modulename = "RecentProjects - loadRecentProjects";
      this.recentprojectsDataSource = [];
      this.webapiService.getProjects().subscribe(res => {
        if (typeof res != 'undefined' && res.length > 0) {
          this.recentprojectsDataSource = res;
        }
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  editProjectContent(projectId: string) {
    try {
      this.webapiService._commonprojectdata.externalpid = projectId;
      this.webapiService._createProjectfromTemplate.initiateRequest = false;
      this._router.navigateByUrl('/infrastructure/project');
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }

  deleteProject(projectId: string) {
    try {
      this.webapiService.deleteProject(projectId).subscribe(() => {
        this.loadRecentProjects();
      }, (error) => {
        this.errorlog.remark = error;
        this.webapiService.ErrorLogfn(this.errorlog);
      });
    }
    catch (error) {
      this.errorlog.remark = error;
      this.webapiService.ErrorLogfn(this.errorlog);
    }
  }
}
