import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse, HttpHeaders } from '@angular/common/http';
import { catchError, retry, tap } from 'rxjs/operators';
import { throwError, Observable, ObservedValueOf } from 'rxjs';
import { Project, replicateEnvironment, deleteEnvironment, EnvironmentDetails, Log } from '../model/project.model';
import { Template, CreateTemplate, MSTBluePrint } from '../model/template.model';
import { ProjectViewModel } from '../model/project-list.model';
import { ResourceGroup, Resources } from '../model/resourcegroup.model';
import { Router } from '@angular/router';
import { CommonProject, CommonConfigProject, CommonTenant } from '../model/commonproject.modal';
import { MstRole, ConfigHostGroup, ConfigAssociation, playbookipproperty, ConfigPlaybook, CommonMstConfigRole } from '../model/configproject.model';
import { Mstresource, CommonMSTResourceDetails } from '../model/mstresource.model';
import { BlueprintId } from '../model/template.model';
import { ToastrService } from 'ngx-toastr';
import { environment } from 'src/environments/environment';
import { autheticationDetails, User } from '../model/authentication.model';

@Injectable({
  providedIn: 'root'
})

export class WebapiService {
  ApiEndpoint:string= "";
  jsondataasstring: string;secjsondataasstring:string;
  iserror: boolean=true;seciserror:boolean=true;
 _commonprojectdata= new CommonProject();
 _commonconfigproject=new CommonConfigProject();
 _createProjectfromTemplate = new CreateTemplate();
 _commonmstresource = new CommonMSTResourceDetails();
 _commonblueprintedit = new BlueprintId();
 _commonblueprintdetails = new MSTBluePrint();
 _commonmstconfigroleid = new CommonMstConfigRole();
 _authentication = new autheticationDetails();
_commonadmintenant =new CommonTenant();
 //azapToken:string = localStorage.getItem("azapToken");

  constructor(private http: HttpClient,private _router: Router, private toastr: ToastrService) {
    this.ApiEndpoint=environment.apiUrl;
    console.log("EnvironmentUrl",environment.apiUrl);
    console.log('env'+ environment.redirectUri);
  }

  /* Service Methods - Dashboard component */
  getProjects(): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<Project>(this.ApiEndpoint + '/Project/GetAllProjects', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getProjectDetails(): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ProjectViewModel>(this.ApiEndpoint + '/project/GetProjectDetails',httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  deleteProject(deleteProject: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/Project/DeleteProject?id=' + deleteProject,httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  /* Service Methods - Dashboard component */

  /* Service Methods - Project component */

  getEnvironmentDetails(projectId: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/environment/getenvironmentbyprojectid?id=' + projectId,httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  deleteEnvironment(deleteEnv: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
   return this.http.delete<any>(this.ApiEndpoint + '/environment/deleteenvironment?envId=' + deleteEnv, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  replicateEnvironment(environmentId: string): Observable<number> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Environment/ReplicateEnvironment?environment=' + environmentId, new Object(), httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));;
  }

  saveEnvironment(environmentDetails: EnvironmentDetails): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/environment/createenvironment', environmentDetails,httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  createUser(userDetails: User): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
     return this.http.post<any>(this.ApiEndpoint + '/User/CreateUser', userDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  saveProject(projDetails: Project): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Project/CreateProject', projDetails,httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  updateProject(projDetails: Project): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/Project/UpdateProject', projDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getProjectbyId(projectId: string) {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Project/GetProjectById?id=' + projectId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  updateEnvironment(environmentDetails: EnvironmentDetails): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/environment/updateenvironment', environmentDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getEnvironmentDetailsbyEnvId(envId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Environment/GetEnvironmentByEnvironmentId?id=' + envId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  setEnvironmentStatus(environmentId: string, status: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/Environment/SetEnvironmentStatus?envId=' + environmentId + '&status=' + status, null, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }
  /* Service Methods - Project component */

  /* Service Methods - ResourceGroup component */

  createResourceGroup(resourceGroup: string, resourceindex:number = 0): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };

    return this.http.post<any>(this.ApiEndpoint + '/Resourcegroup/CreateResourcegroup?resourcegroupIndex=' + resourceindex, resourceGroup, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  exportResourceGroup(resourceGroup: string, generateTerraformUniquename: boolean): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Resourcegroup/ExportResourceGroup?generateTerraformUniquename=' + generateTerraformUniquename,resourceGroup,httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  updateResourceGroup(resourceGroup: ResourceGroup): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/Resourcegroup/UpdateResourcegroup', resourceGroup, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  deleteResourceGroup(rgId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/Resourcegroup/DeleteResourcegroup?rgId=' + rgId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  getResourceGroupsbyEnvId(envId: string): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetResourcegroupByEnvironmentId?envid=' + envId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  getResourceGroupsbyProjectId(projectid: string): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetResourcegroupByProjectId?projectid=' + projectid, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  getExportedResourceGroupsbyProjectId(projectid: string): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetExportedResourcegroupByProjectId?projectid=' + projectid,httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  /* Service Methods - ResourceGroup component */

  /* Service Methods - Template Details */

  getAllTemplateDetails(category: string): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstTemplates/GetAllMstTemplates?category=' + category, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getPopularTemplates(): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };    
    return this.http.get<any>(this.ApiEndpoint + '/MstTemplates/GetPopularMstTemplates', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getInfraTemplate(templateId: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstTemplates/GetTemplateDetails?tempid=' + templateId, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }


  /* Service Methods - Template Details */

  /* Deployment Screen service methods */

  getDeploymentPackageDetails(): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Package/GetProjectDeploymentDetails', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  
  validateEnvironmentInfra(packageName: string, environment: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Package/validateinfra?packageName=' + packageName + '&environment=' + environment,new Object(), httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }

  deploymentEnvironmentInfra(packageName: string, environment: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Package/deployinfra?packageName=' + packageName + '&environment=' + environment, new Object(), httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }

  rollbackEnvironmentInfra(packageName: string, environment: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Package/rollbackinfra?packageName=' + packageName + '&environment=' + environment, new Object(), httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }

  getDeploymentLog(packageId: string, action: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Package/GetDeploymentLogs?dtlpackageid=' + packageId + '&releasedefinitionname=' + action, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  /* Deployment Screen service methods */

  /* MST Rsource Methods WebAPI*/
  getmstiacproviders(): Observable<any> {
   
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstIacProvider/GetAllProviders', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getmstiac(): Observable<any> {
   
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstIacs/GetAllMstIacs', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getmstresources(): Observable<any> {
   
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetAllMstResources')
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  createmstresource(createresource: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Resource/CreateMstResource', createresource, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  updatemstresource(mstresource: Mstresource): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/Resource/UpdateMstResource', JSON.stringify(mstresource), httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  deletemstresource(mstresourceId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/Resource/DeleteMstResourceById?id=' + mstresourceId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  getrgbyrgid(rgid:string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetResourcegroupByRgId?rgid='+rgid, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  /* MST Rsource Methods WebAPI*/

  getResources(id: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetResourceByRgId?id=' + id, httpOptions)
    .pipe(
      catchError(this.handleError));;
  }

  getExportedResourcesByRgId(id: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetExportedResourcesByRgId?id=' + id,httpOptions)
    .pipe(
      catchError(this.handleError));;
  }

  typeheadSearch(filter: { name: string } = { name: '' }, page = 1): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetResourcesByName?name=' + filter.name, httpOptions)
      .pipe(
        tap((response: Resources[]) => {
          return response;
        })
      );
  }
  getResourcesJson(id: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourcesJson/GetMstResourcesJsonByMstResId?mstresid=' + id, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getResourcesDataJson(regid: string, resid: string, uniqid: string, mstresprpid: string = ""): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourcesJson/GetMstDataJson?rgid=' + regid + '&mstresid=' + resid + '&resourceid=' + uniqid + '&mstrespropertyid=' + mstresprpid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  saveResourcesJson(resource: any, resourceindex: number = 0): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Resource/CreateResource?resourceIndex=' + resourceindex, resource, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  saveExportResourcesJson(resource: any,generateTerraformUniquename:boolean): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Resource/ExportResource?generateTerraformUniquename='+ generateTerraformUniquename , resource, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  UpdateResourcesJson(resource: any): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/Resource/UpdateResource', resource, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  deleteResources(rgId: string, resId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/Resource/DeleteResourceById?rgId=' + rgId + '&resId=' + resId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

  getresourcedropdownforipproperty(resId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceProperties/GetMstIpPropertiesByMstResId?resid=' + resId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  getresourcedropdownforres(rgId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetResourcesByRgId?rgid=' + rgId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  getresourcekeysbyrgid(rgId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetResourceGroupKeysByRgId?rgid=' + rgId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
  getresourcekeysbyresid(resId: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetResourceKeysByResId?resid=' + resId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }
   /* MST RsourceProperties Methods WebAPI*/
   getinputpropertiesbyresourceid(resourceid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceProperties/GetMstIpPropertiesByMstResId?resid=' + resourceid, httpOptions)
        .pipe(
          retry(3),
          catchError(this.handleError));
  }
  getoutputpropertiesbyresourceid(resourceid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceProperties/GetMstOpPropertiesByMstResId?resid=' + resourceid, httpOptions)
        .pipe(
          retry(3),
          catchError(this.handleError));
  }
 
  createresourceproperty(resourceproperty: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/MstResourceProperties/CreateMstResourceProperty', resourceproperty, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
  updateresourceproperty(resourceproperty: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/MstResourceProperties/UpdateMstResourceProperty', resourceproperty, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }

  deleteipresourceproperty(id: string, ipvariableid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceProperties/DeleteMstResPropIPVariable?id=' + id + '&ipvarid=' + ipvariableid, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }

  deleteopresourceproperty(id: string, opvariableid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceProperties/DeleteMstResPropOPVariable?id=' + id + '&opvarid=' + opvariableid, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
  deletemstresourceproperty(resourcepropertyId: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceProperties/DeleteMstResourcepropertiesById?id=' + resourcepropertyId, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
  
  getconfigprojects(): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<Project>(this.ApiEndpoint + '/ProjectConfiguration/GetAllProjectConfig', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getconfigprojectbyId(projectId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/ProjectConfiguration/GetProjectConfigById?id=' + projectId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  saveconfigproject(projDetails: Project): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/ProjectConfiguration/CreateProjectConfig', projDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  updateconfigproject(projDetails: Project): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/ProjectConfiguration/UpdateProjectConfig', projDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  deleteconfigproject(deleteProject: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/ProjectConfiguration/DeleteProjectConfig?id=' + deleteProject, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getconfigmstresbyrgId(rgId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetConfigurableMstResourcesByResourceGroupId?id=' + rgId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getconfigmstrespropbyrgId(rgId: string, resId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resource/GetResources?mstresid=' + resId + '&resourcegroupid=' + rgId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  saveHostGroup(host: ConfigHostGroup): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/ConfigHostGroup/CreateConfigHostGroup', host, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getHostGroupbyId(projectId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ConfigHostGroup[]>(this.ApiEndpoint + '/ConfigHostGroup/GetConfigHostGroupByConfigProjectId?id=' + projectId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getexistHostGroupbyId(hgId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/ConfigHostGroup/GetConfigHostGroup?id=' + hgId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  deleteHostGroup(deletehg: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/ConfigHostGroup/DeleteConfigHostGroup?id=' + deletehg, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getroleslistbymstId(projectId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<MstRole[]>(this.ApiEndpoint + '/MstConfigRoles/GetConfigRolesByMstResourceId?id=' + projectId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  saveAssociation(role: ConfigAssociation): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/ConfigHostGroup/SaveRoles', role, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getrolelistbyhgId(hgId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ConfigAssociation[]>(this.ApiEndpoint + '/ConfigHostGroup/GetRolesByHostGroupid?hostgroupid=' + hgId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getexistrolebyroleId(roleId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ConfigAssociation>(this.ApiEndpoint + '/ConfigHostGroup/GetRolesConfigAssociationId?configassociationid=' + roleId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  deleterole(deleterole: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/ConfigHostGroup/DeleteRoles?id=' + deleterole, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  getvirtualnetworkbyrgid(resourcegroupId: string) : Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<playbookipproperty[]>(this.ApiEndpoint + '/ConfigHostGroup/GetVirtualNetworkbyRgId?rgid=' + resourcegroupId, httpOptions)
    .pipe(
      retry(3),
      catchError(this.handleError));
  }

  getplaybookipproperty(mstconfigroleid: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstPlaybookProperties/GetMstPlaybookIPProperties?mstconfigroleid=' + mstconfigroleid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getexistplaybook(assid: string,roleid:string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ConfigPlaybook>(this.ApiEndpoint + '/ConfigPlayBook/GetConfigPlayBook?configassociationid=' + assid+'&mstconfigroleid='+roleid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getrolelist(associationId: string) {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<ConfigPlaybook[]>(this.ApiEndpoint + '/ConfigPlayBook/GetConfigPlayBookWithRoles?configassociationid=' + associationId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  updateconfigplayBook(role: ConfigPlaybook): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/ConfigPlayBook/UpdateConfigPlayBook', role, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getconfigDeploymentPackageDetails(): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Package/GetProjectConfigDeploymentDetails', httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  deployAnsibleDeploymentPackage(packageDetails: string): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/Package/deployconfig', packageDetails, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  // Blueprint Methods
  saveblueprintdetails(blueprintdetails: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/MstTemplates/CreateMstTemplate', blueprintdetails, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
  }

  getblueprintdetails(blueprintid: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstTemplates/GetMstTemplatesById?id=' + blueprintid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }

  deleteblueprintdetails(blueprintid: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstTemplates/DeleteMstTemplateById?id=' + blueprintid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getresourceipjson(mstresid:string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceProperties/GetRecursiveMstResourceIpVariables?id=' + mstresid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  getresourcedrpdwnip(filter: { envid:string,name: string } = { envid:'',name: '' }, page = 1): Observable<any> {
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/GetResourceGroupComplexData?environmentId=' + filter.envid+'&searchValue='+ filter.name, httpOptions)
    .pipe(
      tap((response: any[]) => {
        return response;
      }), catchError(this.handleError)
    );
  }
  getResourcespropDataJson(regid: string, resid: string, uniqid: string, mstresprpid: string = ""): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourcesJson/GetMstResourcesJson?rgid=' + regid + '&mstresid=' + resid + '&resourceid=' + uniqid, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
  }
  // Blueprint Methods

  // MSTConfig Role Methods
getconfigroles():Observable<any> {
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json'}) };

  return this.http.get<any>(this.ApiEndpoint + '/MstConfigRoles/GetConfigRolesName', httpOptions).pipe(retry(3), catchError(this.handleError));
}  

createconfigrole(configrole: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.post<any>(this.ApiEndpoint + '/MstConfigRoles/CreateConfigRoles', configrole, httpOptions).pipe(retry(0),catchError(this.handleError));
}

deleteconfigrole(configroleid: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/MstConfigRoles/DeleteConfigRoles?id=' + configroleid, httpOptions).pipe(retry(3), catchError(this.handleError));
}
// MSTConfig Role Methods

// MST playbook properties mothods

getmstplaybookipproperties(mstconfigroleid: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/MstPlaybookProperties/GetMstPlayBookIPProperties?mstconfigroleid=' + mstconfigroleid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}

getmstplaybookopproperties(mstconfigroleid: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/MstPlaybookProperties/GetMstPlayBookOPProperties?mstconfigroleid=' + mstconfigroleid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}

deleteMstPlaybookOpProperties(playbookpropertyid: string, opvariableid: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/MstPlaybookProperties/DeleteMstPlaybookPropOPVariable?id=' + playbookpropertyid + '&opvarid=' + opvariableid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}

deleteMstPlaybookIpProperties(playbookpropertyid: string, opvariableid: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/MstPlaybookProperties/DeleteMstPlaybookPropIPVariable?id=' + playbookpropertyid + '&ipvarid=' + opvariableid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}

deleteMstPlaybookpropertybyId(playbookpropertyid: string){
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/MstPlaybookProperties/DeleteMstPlayBookPropertiesById?id=' + playbookpropertyid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}

getMSTPlayBookPropertybyConfigRoleId(mstconfigroleId):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/MstPlaybookProperties/GetMstPlaybookPropertyByMstConfigRolesId?mstconfigrolesid=' + mstconfigroleId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
}
getConfigRolesbyConfigRoleId(mstconfigroleId):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/MstConfigRoles/GetConfigRolesById?id=' + mstconfigroleId, httpOptions)
      .pipe(
        retry(3),
        catchError(this.handleError));
}


createMSTPlaybookProperty(playbopokproperty: string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/MstPlaybookProperties/CreateMstPlaybookProperty', playbopokproperty, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}

updateMSTPlaybookProperty(playbopokproperty: string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/MstPlaybookProperties/UpdateMstPlaybookProperty', playbopokproperty, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}

// Azure Tenant

getAzureTenantDetails(id: string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetAzureTenant?id=' + id, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}

getallAzureTenantDetails():Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetAllAzureTenant', httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}
getallAzureSubscribeDetails(tenantid:string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetSubscription?tenantid='+tenantid, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}
getallAzureSPDetails(tenantid:string,subid:string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetServicePrincipal?tenantid='+tenantid+'&subsid='+subid, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}
getallAzurergDetails(tenantid:string,subsid:string,spid:string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetResourceGroup?tenantid='+tenantid+'&subsid='+subsid+'&spid='+spid, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}

getallAzureStorageAccountDetails(tenantid:string,subid:string,spid:string,rgid:string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetStorageAccount?tenantid='+tenantid+'&subsid='+subid+'&spid='+spid+'&rgid='+rgid, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}

getallAzureStorageContainerDetails(tenantid:string,subid:string,spid:string,rgid:string,said:string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.get<any>(this.ApiEndpoint + '/AzureTenant/GetStorageContainer?tenantid='+tenantid+'&subsid='+subid+'&spid='+spid+'&rgid='+rgid+'&storageaccountid='+said, httpOptions)
  .pipe(
    retry(3),
    catchError(this.handleError));
}

saveAzureTenantDetails(tenantdetails: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/AzureTenant/CreateAzureTenant', tenantdetails, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}
saveSubscriptionDetails(subdetails: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/AzureTenant/CreateSubscription', subdetails, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}
saveSPDetails(spdetails: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/AzureTenant/CreateServicePrincipal', spdetails, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}
saveRGDetails(rgdetails: string):Observable<any> {
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/AzureTenant/CreateResourceGroup', rgdetails, httpOptions)
      .pipe(
        retry(0),
        catchError(this.handleError));
}
deleteAzureTenantDetails(tenatId: string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/AzureTenant/DeleteAzureTenant?id=' + tenatId, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}
deleteAzureSubscription(tenantid:string,subid: string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/AzureTenant/DeleteSubscription?subsid=' + subid+'&tenantid='+tenantid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}
deleteAzureSP(tenantid:string,subid: string,spid:string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/AzureTenant/DeleteServicePrincipal?subsid=' + subid+'&tenantid='+tenantid+'&spid='+spid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}
deleteAzureRG(tenantid:string,subid: string,spid:string,rgid:string):Observable<any>{
 
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.delete<any>(this.ApiEndpoint + '/AzureTenant/DeleteResourceGroup?subsid=' + subid+'&tenantid='+tenantid+'&spid='+spid+'&rgid='+rgid, httpOptions)
  .pipe(retry(3),catchError(this.handleError));
}
ErrorLogfn(errorlog: Log) {
  errorlog.loggedby=localStorage.getItem("azapUserEmail");
  this.createlogapi(errorlog).subscribe(res => {
  });
  this.showErrorNotification("Please try again later!", "Something went wrong!!");
}
createlogapi(log:Log)
{
  const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
  return this.http.post<any>(this.ApiEndpoint + '/Log/CreateLog', log, httpOptions).pipe(retry(0),catchError(this.handleError));
}

  private handleError(error: HttpErrorResponse) {
    if (error.status === 0) {
      // A client-side or network error occurred. Handle it accordingly.
      console.error('An error occurred:', error.error);
    }
    else if (error.status == 404) {
      //alert("Invalid Request!!!");
      window.location.href = '/';
    } else {
      // The backend returned an unsuccessful response code.
      // The response body may contain clues as to what went wrong.
      console.error(
        `Backend returned code ${error.status}, body was: `, error.error);
    }
    let msg="Backend returned code "+ error.status+ ", body was: "+error.error;
    // Return an observable with a user-facing error message.
    return throwError(msg);
  }
  showErrorNotification(message, title){
    this.toastr.error(message, title)
  }
getReleaseLog(url: string) {
  return this.http.get(url,{responseType:'arraybuffer',headers:new HttpHeaders({'authorization': 'Basic ' + btoa('username:'+environment.patkey)})} );
}

  getDownloadPackage(url: string)
  {
    let headers = new HttpHeaders({
      'Content-Type': 'application/zip',
      'Access-Control-Allow-Origin': '*',
      'Access-Control-Allow-Headers': 'Content-Type',
      'Access-Control-Allow-Methods': 'GET,POST,OPTIONS,DELETE,PUT',
      'Authorization': 'Basic ' + btoa(':'+environment.patkey),
      'responseType': 'arraybuffer'
    });
    return this.http.get(url, { responseType: 'arraybuffer', headers: headers});
  }

  checkResourceGroupTerraformUniqueName(resourceGroupId:string, environmentId:string): Observable<any> {
    
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/Resourcegroup/CheckResourceGroupTerraformUniqueName?resourcegroupid=' + resourceGroupId+'&environmentid='+environmentId, httpOptions)
      .pipe(
        catchError(this.handleError));
  }

   /* MST Data Rsource Properties Methods WebAPI*/
   getinputdatapropertiesbyresourceid(resourceid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceDataProperties/GetMstIpDataPropertiesByMstResId?resid=' + resourceid, httpOptions)
        .pipe(
          retry(3),
          catchError(this.handleError));
  }
  getoutputdatapropertiesbyresourceid(resourceid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.get<any>(this.ApiEndpoint + '/MstResourceDataProperties/GetMstDataOpPropertiesByMstResId?mstresourceid=' + resourceid, httpOptions)
        .pipe(
          retry(3),
          catchError(this.handleError));
  }
 
  createdataresourceproperty(resourceproperty: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.post<any>(this.ApiEndpoint + '/MstResourceDataProperties/CreateMstResourceDataProperty', resourceproperty, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
  updatedataresourceproperty(resourceproperty: string): Observable<any> {
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.put<any>(this.ApiEndpoint + '/MstResourceDataProperties/UpdateMstResourceDataProperty', resourceproperty, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }

  deleteipdataresourceproperty(id: string, ipvariableid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceDataProperties/DeleteMstResPropDataIPVariable?id=' + id + '&ipdatavarid=' + ipvariableid, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }

  deleteopdataresourceproperty(id: string, opvariableid: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceDataProperties/DeleteMstResPropDataOPVariable?id=' + id + '&opdatavarid=' + opvariableid, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
  deletemstdataresourceproperty(resourcepropertyId: string): Observable<any>{
 
    const httpOptions = { headers: new HttpHeaders({ 'Content-Type': 'application/json' }) };
    return this.http.delete<any>(this.ApiEndpoint + '/MstResourceDataProperties/DeleteMstResourceDatapropertiesById?id=' + resourcepropertyId, httpOptions)
    .pipe(
      catchError(this.handleError)); 
  }
}